<?php
/*
*    Copyright 2008,2009 Maarch
*
*  This file is part of Maarch Framework.
*
*   Maarch Framework is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Maarch Framework is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*    along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
*  Module Advanced Physical Archive : Generic functions of the APA module
*
* Contains all the functions to manage positions
*
* @file
* @author Yves Christian KPAKPO <dev@maarch.org>
* @date $date$
* @version $Revision$
* @ingroup apa
* @brief   Module Advanced Physical Archive :  Generic functions of the APA module
*/

/**
*
* Contains all the functions to manage container types
*
* @ingroup apa
* @brief   Module Advanced Physical Archive :  Generic functions of the APA module
*/
class apa_tools extends dbquery
{
	/**
	* Redefinition of the types object constructor : configure the sql argument order by
	*/
	function __construct()
	{
		parent::__construct();
	}

	/**
	* Get all site available for curent user
	* @return  array
	*/
	public function getsites()
	{
		$result = array();

		//builds the users entities filter of the current user
		$entitylist = $this->getentitylist();

		$this->connect();

		$this->query("select site_id, site_desc from ".$_SESSION['tablename']['apa_sites']."
					where entity_id in (".$entitylist.") order by site_desc");

		while($line = $this->fetch_object())
		{
			array_push($result,array('id'=>$line->site_id,'desc'=>$line->site_desc));
		}

		return $result;
	}

	/**
	* Get all container types
	* @return  array
	*/
	public function getcontainertypes()
	{
		$result = array();
		$this->connect();
		$this->query("select ctype_id, ctype_desc from ".$_SESSION['tablename']['apa_container_types']." order by ctype_desc");

		while($line = $this->fetch_object())
		{
			array_push($result,array('id'=>$line->ctype_id,'desc'=>$line->ctype_desc));
		}

		return $result;
	}

	/**
	* Get all positions of a site  grouped by row
	* @param $site string site identifier
	* @return  array
	*/
	public function viewrows($site)
	{
		$used = false;
		$apa_view = array();
		$tab_sort = array();
		$this->connect();

		$this->query("select pos_row,
					count(distinct pos_col) as col, count(pos_level) as level,
					sum(pos_max_uc) as max_uc, sum(pos_available_uc) as ava_uc
					FROM ".$_SESSION['tablename']['apa_positions']." WHERE site_id = '".$site."'
					group by pos_row order by pos_row");
		//$this->show();

		if($this->nb_result() > 0)
		{
			$count = 0;
			while($line = $this->fetch_object())
			{
				$tab_sort[] = $line->pos_row;
				$apa_view[$site][$count]['row'] = $line->pos_row;
				$apa_view[$site][$count]['nbr_col'] = $line->col;
				$apa_view[$site][$count]['nbr_level'] = $line->level;
				$apa_view[$site][$count]['max_uc'] = $line->max_uc;
				$apa_view[$site][$count]['ava_uc'] = $line->ava_uc;

				$count++;
			}

			array_multisort($tab_sort, $apa_view[$site]);
		}

		return $apa_view;
	}

	/**
	* Check if the rows already exist
	*
	* @param $site string site identifier
	* @param $row string row position
	* @return bolean  true if the row exist,  false if not
	*/
	public function rowalreadyexist($site, $row)
	{
		$exist = false;

		$this->connect();
		$this->query("select position_id
					from ".$_SESSION['tablename']['apa_positions']." where
						site_id= '".$site."' and pos_row= '".strtoupper($row)."'");

		if($this->nb_result() > 0)
		{
			$exist = true;
		}

		return $exist;
	}

	/**
	* Check if the position is available
	*
	* @param $site string site identifier
	* @param $row string row position
	* @param $col integer column position
	* @param $level integer level position
	* @return bolean  true if the position is used,  false if not
	*/
	public function positionisavailable($site, $row, $col, $level)
	{
		$available = false;

		$this->connect();
		$this->query("select position_id
					from ".$_SESSION['tablename']['apa_positions']." where
						site_id= '".$site."' and pos_row= '".strtoupper($row)."'
						and pos_col= '".$col."' and pos_level= '".$level."'");

		if($this->nb_result() == 0)
		{
			$available = true;
		}
		return $available;
	}

	/**
	* Get site id of a position
	*
	* @param $position integer position identifier
	* @return integer  site id
	*/
	public function getpositionsite($position)
	{
		$site = '';

		$this->connect();
		$this->query("select site_id
					from ".$_SESSION['tablename']['apa_positions']." where
						position_id= '".$position."'");

		if($this->nb_result() > 0)
		{
			$line = $this->fetch_object();
			$site = $line->site_id;
		}

		return $site;
	}
	/**
	* Get capacity of a position
	*
	* @param $position integer position identifier
	* @return integer  capacity of the positon
	*/
	public function getpositioncapacity($position)
	{
		$capacity = 0;

		$this->connect();
		$this->query("select pos_max_uc
					from ".$_SESSION['tablename']['apa_positions']." where
						position_id= '".$position."'");

		if($this->nb_result() > 0)
		{
			$line = $this->fetch_object();
			$capacity = $line->pos_max_uc;
		}

		return $capacity;
	}

	/**
	* Get available capacity of a position
	*
	* @param $position integer position identifier
	* @return integer  available capacity of the positon
	*/
	public function getpositionavailablecapacity($position)
	{
		$capacity = 0;

		$this->connect();
		$this->query("select pos_available_uc
					from ".$_SESSION['tablename']['apa_positions']." where
						position_id= '".$position."'");

		if($this->nb_result() > 0)
		{
			$line = $this->fetch_object();
			$capacity = $line->pos_available_uc;
		}

		return $capacity;
	}

	/**
	* Get the id of one position in the database
	*
	* @param $site string site identifier
	* @param $row string row position
	* @param $col integer column position
	* @param $level integer level position
	* @return integer  id of the position . If not found return false
	*/
	public function getpositionid($site,$row,$col,$level)
	{
		$found = false;
		$this->connect();
		$this->query("select position_id
					from ".$_SESSION['tablename']['apa_positions']." where
						site_id= '".$site."' and pos_row= '".strtoupper($row)."'
						and pos_col= '".$col."' and pos_level= '".$level."'");

		if($this->nb_result() >0)
		{
			$line = $this->fetch_object();
			return $line->position_id;
		}
		else
		{
			return $found;
		}
	}

	/**
	* Get the number of entry for a position
	*
	* @param $site string site identifier
	* @param $row string row position
	* @param $col integer column position
	* @param $level integer level position
	* @return integer number of entry
	*/
	public function getpositionentry($site, $row, $col, $level)
	{
		$this->connect();
		$this->query("select count(position_id) as entry from ".$_SESSION['tablename']['apa_positions']." where
						site_id= '".$site."' and pos_row= '".strtoupper($row)."'
						and pos_col= '".$col."' and pos_level= '".$level."'");

		$line = $this->fetch_object();
		return $line->entry;
	}

	/**
	* Get the number of UC for a position
	*
	* @param $position integer position identifier
	* @return integer  number of poition
	*/
	public function getucnumber($position)
	{
		$this->connect();
		$this->query("select pos_max_uc
					from ".$_SESSION['tablename']['apa_positions']." where
						position_id= '".$position."'");
		$line = $this->fetch_object();

		return $line->pos_max_uc;
	}

	/**
	* Verify if the container is used by an entity
	*
	* @param $container integer container identifier
	* @return boolean  true if the container is used, false if not
	*/
	public function useduc($container)
	{
		$used = false;
		$this->connect();

		//builds the users entities filter of the current user
		$entitylist = $this->getentitylist();

		$this->query("select count(*) as number
					from ".$_SESSION['tablename']['apa_header']." where
						arcontainer_id = ".$container." and  entity_id in (".$entitylist.")");
		$line = $this->fetch_object();

		if ($line->number >0)
		{
			$used = true;
		}

		return $used;
	}

	/**
	* Get all entities for current user
	*
	* @return string  list aof all entities
	*/
	public function getentitylist()
	{
		require_once('modules'.DIRECTORY_SEPARATOR.'entities'.DIRECTORY_SEPARATOR.'class'.DIRECTORY_SEPARATOR.'class_manage_entities.php');

		$entitylist = "";
		$tab_entitylist = array();

		$ent = new entity();

		$tab_entitylist = $ent->get_all_entities_id_user($_SESSION['user']['entities']);
		for($i=0; $i < count($tab_entitylist); $i++)
		{
			$entitylist .= $tab_entitylist[$i]." , ";
		}

		$entitylist = preg_replace("/ , $/", "", $entitylist);
		
		if (empty($entitylist))
		{
			$entitylist = "''";
		}
		
		return $entitylist;
	}

	/**
	* Get  entity type for current user
	*
	* @return string  list aof all entities
	*/
	public function getentitytype()
	{
		require_once('modules'.DIRECTORY_SEPARATOR.'entities'.DIRECTORY_SEPARATOR.'class'.DIRECTORY_SEPARATOR.'class_manage_entities.php');

		$ent = new entity();

		$entity_type = $ent->get_entity_type_level($_SESSION['user']['primaryentity']['id']);

		return $entity_type;
	}

	/**
	* Return the parent of an entity
	*
	* @param string $entity_id identifier of the entity
	* @return string  parent of the entity
	*/
	public function getentityparent($entity_id)
	{
		require_once('modules'.DIRECTORY_SEPARATOR.'entities'.DIRECTORY_SEPARATOR.'class'.DIRECTORY_SEPARATOR.'class_manage_entities.php');

		$ent = new entity();

		$entity_parent = $ent->getParentEntityId($entity_id);

		return $entity_parent;
	}

	/**

	**/
	public function getallbox($uc)
	{
		$tabbox = array();
		$userentity = $_SESSION['user']['primaryentity']['id'];

		$this->query("select header_id, arbox_id, entity_id from ".$_SESSION['tablename']['apa_header']." where arcontainer_id = '".$uc."'");
		//$this->show();
		while($line = $this->fetch_object())
		{
			$parent = $this->getentityparent($line->entity_id);

			if ($parent == $userentity)
			{
				array_push($tabbox,array('header_id'=>$line->header_id,'arbox_id'=>$line->arbox_id));
			}
		}
		return $tabbox;
	}

	/**
	* Verify if the destruction date is the same for all UA In UC
	*
	* @param $container integer container identifier
	* @param $destructdate date destruction date
	* @return boolean  true if the date is the same, false if not
	*/
	public function checkdestructdate($container, $destruct_date)
	{
		$same = true;

		$func = new functions();

		//builds the users entities filter of the current user
		$entitylist = $this->getentitylist();

		$this->connect();

		$this->query("select header_id, destruction_date from ".$_SESSION['tablename']['apa_header']." where
						arcontainer_id = '".$container."' and entity_id in (".$entitylist.")");

		while($line = $this->fetch_object())
		{
			$this_destruction_date =  $func->format_date_db($line->destruction_date);
			//echo $this_destruction_date."/".$destruct_date."<br/>";
			if ( $this_destruction_date != $destruct_date)
			{
				$same = false;
				break;
			}
		}

		return $same;
	}

	/**
	* Verify if the box (UA) exist
	*
	* @param $arbox integer box identifier
	* @return boolean  true if exist, false if not
	*/
	public function checkarbox($arbox)
	{
		$exist = false;

		$this->connect();
		$intarbox = (int)$arbox;

		$this->query("select count(*) as number from ".$_SESSION['tablename']['apa_boxes']." where arbox_id = ".$intarbox);

		$line = $this->fetch_object();

		if ($line->number >0)
		{
			$exist = true;
		}

		return $exist;
	}

	/**
	* Verify if the archive already exist
	*
	* @param $arbox integer box identifier
	* @return boolean  true if exist false if not
	*/
	public function archivealreadyexist($arbox)
	{
		$exist = false;

		$this->connect();

		$this->query("select count(*) as number from ".$_SESSION['tablename']['apa_header']." where arbox_id = ".(int)$arbox);

		$line = $this->fetch_object();

		if ($line->number >0)
		{
			$exist = true;
		}

		return $exist;
	}

	/**
	* Verify if the status of an archive
	*
	* @param $archive integer archive identifier
	* @param $status string status searched
	* @return boolean  true if match, false if not
	*/
	public function getarchivestatus($archive, $status)
	{
		$match = false;

		$this->connect();

		$this->query("select arbox_id from ".$_SESSION['tablename']['apa_header']." where header_id = '".$archive."'");
		$line = $this->fetch_object();
		$arbox = $line->arbox_id;

		$arboxstatus = $this->getarboxstatus($arbox);

		if ($arboxstatus == $status)
		{
			$match = true;
		}

		return $match;
	}

	/**
	* Verify if the box (UA) cant be acceded by user
	*
	* @param $arbox integer box identifier
	* @return boolean  true if he cant access, false if not
	*/
	public function accessarbox($arbox)
	{
		$access = false;

		//builds the users entities filter of the current user
		$entitylist = $this->getentitylist();

		$this->connect();
		$intarbox = (int)$arbox;

		$this->query("select count(*) as number from ".$_SESSION['tablename']['apa_boxes']." where
						status <> 'NEW' and arbox_id = ".$intarbox." and (entity_id in (".$entitylist.") or entity_id is NULL)");

		$line = $this->fetch_object();

		if ($line->number >0)
		{
			$access = true;
		}

		return $access;
	}

	/**
	*Get the  box status
	*
	* @param $arbox integer box identifier
	* @return status
	*/
	public function getarboxstatus($arbox)
	{
		$status = '';

		$this->connect();

		$this->query("select status from ".$_SESSION['tablename']['apa_boxes']." where arbox_id = '".$arbox."'");
		$line = $this->fetch_object();
		$status = $line->status;

		return $status;
	}

	/**
	* Verify if the box (UA) is positioned in the UC
	*
	* @param $arbox integer box identifier
	* @param $container integer container identifier
	* @return boolean  true if in the UC false if not
	*/
	public function boxincontainer($arbox, $container, $entity)
	{
		$inside = false;

		//Select the actual container of the box
		$this->query("select arcontainer_id from ".$_SESSION['tablename']['apa_boxes']." where arbox_id = '".$arbox."' and entity_id = '".$entity."'");
		$line = $this->fetch_object();
		$old_container = $line->arcontainer_id;

		//If there is a container
		if(!empty($old_container))
		{
			//Compare the actual container an the updated container
			if ($old_container ==  $container)
			{
				//If match
				$inside = true;
			}
		}
		else //If there is no value for container is like inside
		{
			$inside = true;
		}

		return $inside;
	}

	/**
	* Verify if the container (UC) exist
	*
	* @param $container integer container identifier
	* @return boolean  true if exist, false if not
	*/
	public function checkarcontainer($container, $entity)
	{
		$exist = false;

		$this->connect();

		$this->query("select count(*) as number from ".$_SESSION['tablename']['apa_containers']."
					where arcontainer_id = ".$container." and entity_id = '".$entity."'");

		$line = $this->fetch_object();

		if ($line->number >0)
		{
			$exist = true;
		}

		return $exist;
	}

	/**
	* Verify if the container (UC) cant be acceded by user
	*
	* @param $container integer container identifier
	* @return boolean  true if he cant access, false if not
	*/
	public function accessarcontainer($container)
	{
		$access = false;

		//builds the users entities filter of the current user
		$entitylist = $this->getentitylist();

		$this->connect();

		$this->query("select count(*) as number from ".$_SESSION['tablename']['apa_containers']." where
						arcontainer_id = ".$container." and entity_id in (".$entitylist.")");
		$line = $this->fetch_object();
		//$this->show();

		if ($line->number >0)
		{
			$access = true;
		}

		return $access;
	}

	/**
	* Return the position of the container (UC)
	* @param $container integer container identifier
	* @return  array
	*/
	public function containerposition($container, $entity)
	{
		$apa_position = array();
		$this->connect();

		$this->query("select position_id from ".$_SESSION['tablename']['apa_containers']."
					where arcontainer_id = '".$container."' and entity_id = '".$entity."'");
		//$this->show();
		$line = $this->fetch_object();
		$position = (int)$line->position_id;

		$this->query("select position_id, site_id, pos_row, pos_col, pos_level, pos_max_uc, pos_available_uc from ".$_SESSION['tablename']['apa_positions']." where position_id = ".$position);
		//$this->show(); exit;

		if($this->nb_result() > 0)
		{
			while($line = $this->fetch_object())
			{
				$apa_position['position'] = $line->position_id;
				$apa_position['site'] = $line->site_id;
				$apa_position['row'] = $line->pos_row;
				$apa_position['col'] = $line->pos_col;
				$apa_position['level'] = $line->pos_level;
				$apa_position['max_uc'] = $line->max_uc;
				$apa_position['available_uc'] = $line->pos_available_uc;
			}
		}

		return $apa_position;
	}

	public function get_reservation_batch ()
	{
		$this->connect();
		//
		$this->query("select param_value_int from ".$_SESSION['tablename']['param']." where id = 'apa_reservation_batch'");
		$res = $this->fetch_object();
		$reservation_batch = (string)$res->param_value_int;

		$this->query("update ".$_SESSION['tablename']['param']." set param_value_int  = param_value_int +1 where id = 'apa_reservation_batch'");

		return $reservation_batch;

	}


	/**
	* Return the label of a site
	* @param $site_id string site identifier
	* @return  string
	*/
	public function getSiteDesc ($site_id)
	{
		$siteLabel = '';
		$this->connect();
		$this->query("select site_desc from ".$_SESSION['tablename']['apa_sites']." where site_id = '$site_id'");
		$res = $this->fetch_object();
		$siteLabel = $res->site_desc;

		return $siteLabel;
	}

	/**
	* Return the label of a nature
	* @param $nature_id string nature identifier
	* @return  string
	*/
	public function getNatureDesc ($nature_id)
	{
		$natureLabel = '';
		$this->connect();
		$this->query("select arnature_desc from ".$_SESSION['tablename']['apa_natures']." where arnature_id = '$nature_id'");
		$res = $this->fetch_object();
		$natureLabel = $res->arnature_desc;

		return $natureLabel;
	}

	/**
	* Return the label of a entity
	* @param $entity_id string entity identifier
	* @return  string
	*/
	public function getEntityDesc ($entity_id)
	{
		$entityLabel = '';
		$this->connect();
		$this->query("select entity_label from ".$_SESSION['tablename']['ent_entities']." where entity_id = '$entity_id'");
		$res = $this->fetch_object();
		$entityLabel = $res->entity_label;

		return $entityLabel;
	}
}
?>
