<?php
/*
*    Copyright 2008,2009, 2010 Maarch
*
*  This file is part of Maarch Framework.
*
*   Maarch Framework is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Maarch Framework is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*    along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @brief Controler of the autofoldering administration (list, modification
*   page and suppression)
*
*
* @file
* @author  Claire Figueras  <dev@maarch.org>
* @date $date$
* @version $Revision$
* @ingroup autofoldering
*/
$core = new core_tools();
$core->load_lang(); // NOTE : core_tools is not a static class

// If mode not defined, list is the default mode
if (isset($_REQUEST['mode']) && !empty($_REQUEST['mode'])) {
    $GLOBALS['mode'] = $_REQUEST['mode'];
} else {
    $GLOBALS['mode'] = 'list';
}

if (isset($_SESSION['custom_override_id'])
    && file_exists(
        $_SESSION['config']['corepath'] . 'custom'
        . DIRECTORY_SEPARATOR . $_SESSION['custom_override_id']
        . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'autofoldering'
        . DIRECTORY_SEPARATOR . 'xml' . DIRECTORY_SEPARATOR
        . $_SESSION['autofoldering_mapping_file']
    )
) {
    $GLOBALS['xml_mapping'] = $_SESSION['config']['corepath'] . 'custom'
                            . DIRECTORY_SEPARATOR
                            . $_SESSION['custom_override_id']
                            . DIRECTORY_SEPARATOR . 'modules'
                            . DIRECTORY_SEPARATOR . 'autofoldering'
                            . DIRECTORY_SEPARATOR . 'xml' . DIRECTORY_SEPARATOR
                            . $_SESSION['autofoldering_mapping_file'];
} else {
    $GLOBALS['xml_mapping'] = 'modules' . DIRECTORY_SEPARATOR . 'autofoldering'
                 . DIRECTORY_SEPARATOR . 'xml' . DIRECTORY_SEPARATOR
                 . $_SESSION['autofoldering_mapping_file'];
}

try {
    require_once 'modules/autofoldering/class/autofoldering_tree_controler.php';
    require_once 'modules/autofoldering/class/af_security_controler.php';
    require_once 'core/class/usergroups_controler.php';

    if (isset($GLOBALS['mode']) && $GLOBALS['mode'] == 'list') {
        require_once 'modules/autofoldering/lang/fr.php';
        require_once 'apps' . DIRECTORY_SEPARATOR
            . $_SESSION['config']['app_id'] . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'class_list_show.php';
    }

} catch (Exception $e) {
    echo $e->getMessage();
}

if (isset($_REQUEST['tree_submit'])) {
    // Action to do with db
    validateAfSecSubmit();
} else {
    // Display to do
    if (isset($_REQUEST['id']) && !empty($_REQUEST['id'])) {
        $treeId = $_REQUEST['id'];
    }
    $state = true;
    switch ($GLOBALS['mode']) {
        case 'up' :
            $state = displayUp(
                $treeId, $GLOBALS['xml_mapping'], $_SESSION['config']['lang']
            );
            locationBarManagement();
            break;
        case 'del' :
            displayDel($treeId);
            break;
        case 'list' :
            $treesList = displayList($GLOBALS['xml_mapping']);
            locationBarManagement();
            break;
    }
    include 'af_trees_security_management.php';
}

/////// PRIVATE BLOCK

/**
 * Initialize session variables
 */
function initSession()
{
    $ugc = new usergroups_controler();
    $_SESSION['m_admin']['af'] = array(
        'groups' => array(),
        'load_groups' => true,
    );
    $groups = $ugc->getAllUsergroups();
    $_SESSION['m_admin']['all_groups'] = array();
    for ($i = 0; $i < count($groups); $i++) {
        $_SESSION['m_admin']['all_groups'][] = $groups[$i]->getArray();
    }
}

/**
 * Management of the location bar
 */
function locationBarManagement()
{
    $pageLabels = array(
        'up'   => _MODIFICATION,
        'list' => _TREES_LIST,
    );
    $pageIds = array(
        'up'   => 'af_tree_security_up',
        'list' => 'af_tree_security_list',
    );

    $init = false;
    if (isset($_REQUEST['reinit']) && $_REQUEST['reinit'] == 'true') {
        $init = true;
    }
    $level = '';
    if (isset($_REQUEST['level'])
        && ( $_REQUEST['level'] == 2 || $_REQUEST['level'] == 3
            || $_REQUEST['level'] == 4 || $_REQUEST['level'] == 1)
    ) {
        $level = $_REQUEST['level'];
    }

    $pagePath = $_SESSION['config']['businessappurl'] . 'index.php?page='
               . 'af_trees_security_management_controler&module=autofoldering'
               . '&mode=' . $GLOBALS['mode'];
    $pageLabel = $pageLabels[$GLOBALS['mode']];
    $pageId = $pageIds[$GLOBALS['mode']];
    $ct = new core_tools();
    $ct->manage_location_bar($pagePath, $pageLabel, $pageId, $init, $level);
}

/**
 * Validate a submit (add or up),
 * up to saving object
 */
function validateAfSecSubmit()
{
    $afSecCtrl = new af_security_controler();
    $afTree = new autofoldering_tree();
    $afTreeCtrl = new autofoldering_tree_controler();
    $func = new functions();
    if (isset($_REQUEST['id']) && !empty($_REQUEST['id'])) {
        // Update, so values exist
        $afTree->tree_id = $func->protect_string_db(
            $func->wash($_REQUEST['id'], 'num', _ID . ' ')
        );
    }
    $tree = $afTreeCtrl->get(
        $_SESSION['m_admin']['af']['tree_id'], $GLOBALS['xml_mapping']
    );
    $table = $tree->__get('target_table');
    for ($i = 0; $i < count($_SESSION['m_admin']['af']['groups']); $i++) {
        $res = $afSecCtrl->check_where_clause(
            $_SESSION['m_admin']['af']['groups'][$i]['where'], $table,
            $_SESSION['user']['UserId']
        );
        if ($res['RESULT'] == false) {
            $_SESSION['error'] = $res['TXT'];
            break;
        }
    }

    if (!empty($_SESSION['error'])) {
        // Error management depending of mode
        putInSession('af', $afTree->getArray());

        switch ($GLOBALS['mode']) {
            case 'up':
                if (!empty($_SESSION['m_admin']['af']['tree_id'])) {
                    header(
                        'location: ' . $_SESSION['config']['businessappurl']
                        . 'index.php?page='
                        . 'af_trees_security_management_controler&mode=up&id='
                        . $_SESSION['m_admin']['af']['tree_id']
                        . '&module=autofoldering'
                    );
                } else {
                    header(
                        'location: ' . $_SESSION['config']['businessappurl']
                        . 'index.php?page='
                        . 'af_trees_security_management_controler&mode=list'
                        . '&module=autofoldering'
                    );
                }
                exit();
        }
    } else {

        $afSecCtrl->deleteSecurityForTree(
            $_SESSION['m_admin']['af']['tree_id']
        );
        for ($i = 0;$i < count($_SESSION['m_admin']['af']['groups']);$i++) {
            $afSec = new af_security();
            $tmpArr = array(
                'af_security_id'    => time() * 1000 + rand() % 1000,
                'group_id'          => $func->protect_string_db(
                    $_SESSION['m_admin']['af']['groups'][$i]['group_id']
                ),
                'af_security_label' => $func->protect_string_db(
                    $_SESSION['m_admin']['af']['groups'][$i]['comment']
                ),
                'tree_id'           => $func->protect_string_db(
                    $_SESSION['m_admin']['af']['tree_id']
                ),
                'where_clause'      => $func->protect_string_db(
                    $_SESSION['m_admin']['af']['groups'][$i]['where']
                ),
            );
            $afSec->setArray($tmpArr);
            $res = $afSecCtrl->save($afSec);
        }
        #TODO : historisation

        $_SESSION['error'] = _TREE_UPDATE;
        header(
            'location: ' . $_SESSION['config']['businessappurl']
            . 'index.php?page=af_trees_security_management_controler&mode=list'
            . '&module=autofoldering'
        );
    }
}



/**
 * Initialize session parameters for update display
 * @param Long $treeId
 */
function displayUp($treeId, $xmlMapping, $lang)
{
    $state = true;
    $afTreeCtrl = new autofoldering_tree_controler();
    $afSecCtrl = new af_security_controler();
    $ugc = new usergroups_controler();
    $afTree = $afTreeCtrl->get($treeId, $xmlMapping, $lang);
    if (!isset($afTree) || empty($afTree)) {
        $state = false;
    } else {
        putInSession('af', $afTree->getArray());
        if (!isset($_SESSION['m_admin']['af']['load_groups'])
            || $_SESSION['m_admin']['af']['load_groups'] == true
        ) {
            $securities = $afSecCtrl->getAllSecurityForTree($treeId);
            for ($i = 0; $i < count($securities); $i++) {
                $groupId = $securities[$i]->__get('group_id');
                $group = $ugc->get($groupId);
                $_SESSION['m_admin']['af']['groups'][] = array(
                    'group_id'    => $groupId,
                    'group_label' => $group->__get('group_desc'),
                    'comment'    => $securities[$i]->__get('af_security_label'),
                    'where'       => $securities[$i]->__get('where_clause'),
                );
            }
            $_SESSION['m_admin']['af']['load_groups'] = false ;
        }
    }
    return $state;
}


/**
 * Initialize session parameters for list display
 */
function displayList($xmlMapping)
{
    $_SESSION['m_admin'] = array();

    $afTreeCtrl = new autofoldering_tree_controler();
    $func = new functions();
    initSession();
    $tab = array();
    $trees = $afTreeCtrl->get_all_trees(
        $xmlMapping, $_SESSION['config']['lang']
    );
    for ($i = 0;$i < count($trees); $i++) {
        $tmp[0]['column'] = 'tree_id';
        $tmp[0]['value'] = $trees[$i]->__get('tree_id');
        $tmp[1]['column'] = 'tree_label';
        $tmp[1]['value'] = $trees[$i]->__get('tree_label');

        array_push($tab, $tmp);
    }
    for ($i = 0;$i < count($tab);$i++) {
        for ($j = 0;$j < count($tab[$i]);$j++) {
            foreach (array_keys($tab[$i][$j]) as $value) {
                if ($tab[$i][$j][$value] == 'tree_id') {
                    $tab[$i][$j]['tree_id'] = $tab[$i][$j]['value'];
                    $tab[$i][$j]['label'] = _ID;
                    $tab[$i][$j]['size'] = '30';
                    $tab[$i][$j]['label_align'] = 'left';
                    $tab[$i][$j]['align'] = 'left';
                    $tab[$i][$j]['valign'] = 'bottom';
                    $tab[$i][$j]['show'] = true;
                }
                if ($tab[$i][$j][$value] == 'tree_label') {
                    $tab[$i][$j]['value'] = $func->show_string(
                        $tab[$i][$j]['value']
                    );
                    $tab[$i][$j]['tree_label'] = $tab[$i][$j]['value'];
                    $tab[$i][$j]['label'] = _DESC;
                    $tab[$i][$j]['size'] = '30';
                    $tab[$i][$j]['label_align'] = 'left';
                    $tab[$i][$j]['align'] = 'left';
                    $tab[$i][$j]['valign'] = 'bottom';
                    $tab[$i][$j]['show'] = true;
                }
            }
        }
    }

    $result = array(
        'tab'           => $tab,
        'what'          => '',
        'page_name'     => 'af_trees_security_management_controler&mode=list',
        'page_name_up'  => 'af_trees_security_management_controler&mode=up',
        'page_name_del' => 'af_trees_security_management_controler&mode=del',
        'page_name_val' => '',
        'page_name_ban' => '',
        'page_name_add' => '',
        'label_add'     => '',
        'title'         => _TREES_LIST . ' : ' . count($tab) . ' ' . _TREES,
    );

    $_SESSION['m_admin']['init'] = true;
    return $result;
}

/**
 * Delete given tree if exists and initialize session parameters
 * @param unknown_type $treeId
 */
function displayDel($treeId)
{
    $afTreeCtrl = new autofoldering_tree_controler();
    $tree = $afTreeCtrl->get($treeId);
    if (isset($tree)) {
        // Deletion
        $afTreeCtrl->delete($tree);
        $_SESSION['error'] = _TREE_DELETION . ' ' . $treeId;
        ?><script type="text/javascript">window.top.location='<?php
        echo $_SESSION['config']['businessappurl']
        .'index.php?page=af_trees_security_management_controler&mode=list'
        . '&module=autofoldering';?>';</script>
        <?php
        exit();
    } else {
        // Error management
        $_SESSION['error'] = _TREE . ' ' . _UNKNOWN;
    }
}

/**
 * Format given item with given values, according with HTML formating.
 * NOTE: given item needs to be an array with at least 2 keys:
 * 'column' and 'value'.
 * NOTE: given item is modified consequently.
 * @param $item
 * @param $label
 * @param $size
 * @param $labelAlign
 * @param $align
 * @param $valign
 * @param $show
 */
function formatItem(
    &$item,
    $label,
    $size,
    $labelAlign,
    $align,
    $valign,
    $show
) {
    $func = new functions();
    $item['value'] = $func->show_string($item['value']);
    $item[$item['column']] = $item['value'];
    $item['label'] = $label;
    $item['size'] = $size;
    $item['label_align'] = $labelAlign;
    $item['align'] = $align;
    $item['valign'] = $valign;
    $item['show'] = $show;
}

/**
 * Put given object in session, according with given type
 * NOTE: given object needs to be at least hashable
 * @param string $type
 * @param hashable $hashable
 */
function putInSession($type, $hashable, $showString = true)
{
    $func = new functions();
    foreach ($hashable as $key => $value) {
        if ($showString) {
            $_SESSION['m_admin'][$type][$key] = $func->show_string($value);
        } else {
            $_SESSION['m_admin'][$type][$key] = $value;
        }
    }
}