<?php
/*
*    Copyright 2008,2009,2010 Maarch
*
*  This file is part of Maarch Framework.
*
*   Maarch Framework is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Maarch Framework is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*    along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @brief  Contains the controler of the af_node_items object
* (create, save, modify, etc...)
*
*
* @file
* @author Claire Figueras <dev@maarch.org>
* @date $date$
* @version $Revision$
* @ingroup autofoldering
*/
try {
    require_once 'modules/autofoldering/autofoldering_tables.php';
    require_once 'core/class/ObjectControlerIF.php';
    require_once 'modules/autofoldering/class/af_node_items.php';
    require_once 'core/class/ObjectControlerAbstract.php';
    require_once 'core/class/ObjectControlerIF.php';
} catch (Exception $e){
    echo $e->getMessage().' // ';
}

/**
* @brief  Controler of the af_node_items object
* (herits from ObjectControler and implements ObjectControlerIF)
*
*<ul>
*  <li>Get an af_node_items object from an id</li>
*  <li>Save in the database a af_node_items</li>
*  <li>Manage the operation on the af_node_items related tables in the database
*   (_insert, select, _update, delete)</li>
*</ul>
* @ingroup autofoldering
*/
class af_node_items_controler extends ObjectControler implements ObjectControlerIF
{
    /**
    * Returns an af_node_items object based on a node identifier (item_id)
    *
    * @param  $itemId string  Item identifier
    * @return af_node_items object with properties from the database or null
    */
    public function get($itemId)
    {
        self::set_specific_id('item_id');
        self::set_foolish_ids(array('tree_id', 'node_id'));
        return self::advanced_get($itemId, AF_NODE_ITEMS_TABLE);
    }


    /**
    * Saves in the database an af_node_items object
    *
    * @param  $node af_node_items object to be saved
    * @return bool true if the save is complete, false otherwise
    */
    public function save($node)
    {
        if (! isset($node)) {
            return false;
        }
        self::set_specific_id('item_id');
        self::set_foolish_ids(array('tree_id', 'node_id'));
        if (isset($node->item_id) && self::nodeExists($node->item_id)) {
            return self::_update($node);
        } else {
            return self::_insert($node);
        }
    }

    /**
    * Inserts in the database an af_node_items object
    *
    * @param  $node af_node_items object
    * @return bool true if the _insertion is complete, false otherwise
    */
    private function _insert($node)
    {
        if (! isset($node)) {
            return false;
        }

        // Inserting object
        $preparation = self::_insert_prepare($node, self::$computed_properties);
        $query = "insert into " . AF_NODE_ITEMS_TABLE . " ("
               . $preparation['properties'] . ") values("
               . $preparation['values'] . ")";
        self::$db = new dbquery();
        self::$db->connect();
        $res = self::$db->query($query, true);
        if (! $res) {
            throw new SqlError($query);
        }
        self::$db->disconnect();
        return $res;
        //  return self::advanced_insert($node);
    }

    /**
    * Updates af_node_items in the database  with an af_node_items object
    *
    * @param  $node af_node_items object
    * @return bool true if the _update is complete, false otherwise
    */
    private function _update($node)
    {
        return self::advanced_update($node);
    }

    /**
    * Deletes in the database a given af_node_items
    *
    * @param  $node af_node_items object
    * @return bool true if the deletion is complete, false otherwise
    */
    public function delete($node)
    {
        self::set_specific_id('item_id');
        return self::advanced_delete($node);
    }


    /**
    * Asserts if a given af_node_items (item_id) exists in the database
    *
    * @param  $itemId String af_node_items identifier
    * @return bool true if the af_node_items exists, false otherwise
    */
    public function nodeExists($itemId)
    {
        if (! isset($itemId) || empty($itemId)) {
            return false;
        }

        self::$db = new dbquery();
        self::$db->connect();
        $query = "select item_id from " . AF_NODE_ITEMS_TABLE
               . " where item_id = " . $itemId . "";

        try {
            self::$db->query($query);
        } catch (Exception $e){
            echo _UNKNOWN . _NODE . " " . $itemId . ' // ';
        }

        if (self::$db->nb_result() > 0) {
            self::$db->disconnect();
            return true;
        }
        self::$db->disconnect();
        return false;
    }
}

/**
* @brief  SqlError class : class to manage sql Error (extends php Exception)
*
*
* @file
* @author Claire Figueras <dev@maarch.org>
* @date $date$
* @version $Revision$
* @ingroup autofoldering
*/
class SqlError extends Exception
{
    public function __construct($query)
    {
        $this->query = $query;
        parent::__construct("SQL Error : ".$query, 1);
    }
}
