<?php
/*
*    Copyright 2008,2009,2010 Maarch
*
*  This file is part of Maarch Framework.
*
*   Maarch Framework is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Maarch Framework is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*    along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @brief  Contains the controler of the af_security object
* (create, save, modify, etc...)
*
*
* @file
* @author Claire Figueras <dev@maarch.org>
* @date $date$
* @version $Revision$
* @ingroup autofoldering
*/

// Loads the required class
try {
    require_once 'modules/autofoldering/autofoldering_tables.php';
    require_once 'core/class/ObjectControlerAbstract.php';
    require_once 'core/class/ObjectControlerIF.php';
    require_once 'core/class/SecurityControler.php';
    require_once 'core/class/users_controler.php';
    require_once 'modules/autofoldering/class/af_security.php';
    require_once 'modules/autofoldering/class/autofoldering_tree_controler.php';
} catch (Exception $e) {
    echo $e->getMessage() . ' // ';
}


/**
* @brief  Controler of the af_security object
* (herits from ObjectControler and implements ObjectControlerIF)
*
*<ul>
*  <li>Get an af_security object from an id</li>
*  <li>Save in the database an af_security</li>
*  <li>Manage the operation on the af_security related tables in the database
*  (_insert, select, _update, delete)</li>
*</ul>
* @ingroup autofoldering
*/
class af_security_controler extends ObjectControler implements ObjectControlerIF
{

    /**
    * Returns an af_security object based on a af_security identifier
    *
    * @param  $afSecurityId string  af_security identifier
    * @return af_security object with properties from the database or null
    */
    public function get($afSecurityId)
    {
        self::set_foolish_ids(array('group_id', 'tree_id'));
        return self::advanced_get($afSecurityId, AF_SECURITY_TABLE);
    }

    /**
    * Returns all af_security from the database for a given tree
    *
    * @param  $treeId string  Tree identifier
    * @return Array of af_security objects with properties from the database
    */
    public function getAllSecurityForTree($treeId)
    {
        if (empty($treeId)) {
            return null;
        }

        self::$db = new dbquery();
        self::$db->connect();
        $query = "select * from " . AF_SECURITY_TABLE . " where tree_id = '"
               . $treeId . "'";

        try {
            self::$db->query($query);
        } catch (Exception $e){}

        $allSecurity = array();
        while ($res = self::$db->fetch_object()) {
            $afSec = new af_security();
            $tmpArr = array(
            	'af_security_id' => $res->af_security_id,
            	'af_security_label' => $res->af_security_label,
            	'group_id' => $res->group_id,
            	'tree_id' => $res->tree_id,
            	'where_clause' => $res->where_clause
            );
            $afSec->setArray($tmpArr);
            array_push($allSecurity, $afSec);
        }
        self::$db->disconnect();
        return $allSecurity;
    }

    /**
    * Returns all af_security from the database for a given tree
    *
    * @param  $treeId string  Tree identifier
    * @return Array of af_security objects with properties from the database
    */
    public function getTreeSecurityForUser($treeId, $userId)
    {
        if (empty($treeId) || empty($userId)) {
            return null;
        }

        self::$db = new dbquery();
        self::$db->connect();
        $usersCtrl = new users_controler();
        $groups = $usersCtrl->getGroups($userId);
        if (! isset($groups) || count($groups) == 0) {
            return null;
        }

        $goupsString = '(';
        for ($i = 0; $i < count($groups); $i ++) {
            $goupsString .= "'" . $groups[$i]['GROUP_ID'] . "',";
        }
        $goupsString = preg_replace('/,$/', ')', $goupsString);
        $query = "select * from " . AF_SECURITY_TABLE . " where tree_id = '"
               . $treeId . "' and group_id in " . $goupsString;
        try {
            self::$db->query($query);
        } catch (Exception $e) {
            echo _UNKNOWN . ' ' . _TREE_ID . " " . $treeId . ' // ';
        }
        $allSecurity = array();
        while ($res = self::$db->fetch_object()) {
            $afSec = new af_security();
            $tmpArr = array(
            	'af_security_id' => $res->af_security_id,
            	'af_security_label' => $res->af_security_label,
            	'group_id' => $res->group_id,
            	'tree_id' => $res->tree_id,
            	'where_clause' => $res->where_clause
            );
            $afSec->setArray($tmpArr);
            array_push($allSecurity, $afSec);
        }
        self::$db->disconnect();

        return $allSecurity;
    }

    /**
    * Returns a security where clause on a tree for a given user
    * (based on the af_security table)
    *
    * @param  $userId string  User identifier
    * @param  $treeId string  Tree identifier
    * @return String the where clause
    */
    public function getWhereClauseForTree($userId, $treeId)
    {
        if (empty($treeId) || empty($userId)) {
            return null;
        }

        $usersCtrl = new users_controler();
        $groups = $usersCtrl->getGroups($userId);
        $secCtrl = new SecurityControler();
        $func = new functions();
        if (! isset($groups) || count($groups) == 0) {
            return null;
        }
        $goupsString = '(';
        for ($i = 0; $i < count($groups); $i ++) {
            $goupsString .= "'" . $groups[$i]['GROUP_ID'] . "',";
        }
        $goupsString = preg_replace('/,$/', ')', $goupsString);
        self::$db = new dbquery();
        self::$db->connect();
        $query = "select where_clause from " . AF_SECURITY_TABLE
               . " where tree_id = '" . $treeId . "' and group_id in "
               . $goupsString;

        try {
            self::$db->query($query);
        } catch (Exception $e) {
            echo _UNKNOWN . ' ' . _TREE_ID . " " . $treeId . ' // ';
        }

        $where = '';
        while ($res = self::$db->fetch_object()) {
            $whereClause = $res->where_clause;
            if (! isset($whereClause) || empty($whereClause)) {
                $where = " and 1 ";
                break;
            } else {
                $where .= " and (" . $whereClause . ")";
            }
        }
        self::$db->disconnect();
        $where = $secCtrl->process_security_where_clause($whereClause, $userId);
        $where = preg_replace("/where/", ' and (', $where);

        if (! empty($where)) {
            $where .= ')';
        }
        $where = preg_replace("/^\s+and/", '', $where);
        return $func->show_string($where);
    }

    /**
    * Saves in the database an af_security object
    *
    * @param  $afSecurity af_security object to be saved
    * @return bool true if the save is complete, false otherwise
    */
    public function save($afSecurity)
    {
        if (! isset($afSecurity)) {
            return false;
        }

        self::set_foolish_ids(array('group_id', 'tree_id'));
        if (self::af_securityExists($afSecurity->af_security_id)) {
            return self::_update($afSecurity);
        } else {
            return self::_insert($afSecurity);
        }
    }

    /**
    * Inserts in the database (af_security table) an af_security object
    *
    * @param  $afSecurity af_security object
    * @return bool true if the _insertion is complete, false otherwise
    */
    private function _insert($afSecurity)
    {
        return self::advanced_insert($afSecurity);
    }

    /**
    * Updates a af_security in the database (af_security table)
    * with an af_security object
    *
    * @param  $afSecurity af_security object
    * @return bool true if the _update is complete, false otherwise
    */
    private function _update($afSecurity)
    {
        return self::advanced_update($afSecurity);
    }

    /**
    * Deletes in the database (af_security related tables) a given af_security
    *
    * @param  $afSecurity af_security object
    * @return bool true if the deletion is complete, false otherwise
    */
    public function delete($afSecurity)
    {
        return self::advanced_delete($afSecurity);
    }

    /**
    * Deletes in the database all security for a given tree
    *
    * @param  $treeId Tree identifier
    * @return bool true if the deletion is complete, false otherwise
    */
    public function deleteSecurityForTree($treeId)
    {
        if (! isset($treeId) || empty($treeId)) {
            return false;
        }
        $allSecurity = self::getAllSecurityForTree($treeId);
        $res  = false;
        for ($i = 0; $i < count($allSecurity); $i ++) {
            $res = self::delete($allSecurity[$i]);
            if ($res == false) {
                return false;
            }
        }
        return $res;
    }

    /**
    * Asserts if a given af_security (af_security_id) exists in the database
    *
    * @param  $afSecurityId String af_security identifier
    * @return bool true if the af_security exists, false otherwise
    */
    public function af_securityExists($afSecurityId)
    {
        if (! isset($afSecurityId) || empty($afSecurityId)) {
            return false;
        }

        self::$db = new dbquery();
        self::$db->connect();
        $query = "select af_security_id from " . AF_SECURITY_TABLE
               . " where af_security_id = " . $afSecurityId . "";

        try {
            self::$db->query($query);
        } catch (Exception $e) {
            echo _UNKNOWN . ' ' . _SECURITY . " " . $afSecurityId . ' // ';
        }

        if (self::$db->nb_result() > 0) {
            self::$db->disconnect();
            return true;
        }
        self::$db->disconnect();
        return false;
    }

    /**
    * Returns allowed trees in an array for the given user
    *
    * @param  $userId String User identifier
    * @param  $xml_file String Path to the autofoldering mapping file
    * @param  $lang String Language option (fr by default)
    * @return Array of autofoldering trees
    */
    public function get_trees_for_user($userId, $xml_file, $lang = 'fr')
    {
        $afTreeCtrl = new autofoldering_tree_controler();
        $usersCtrl = new users_controler();
        $allTrees = $afTreeCtrl->get_all_trees($xml_file);

        if ($userId == 'superadmin') {
            return $allTrees;
        }

        $trees = array();
        $groups = $usersCtrl->getGroups($userId);
        self::$db->connect();

        for ($i = 0; $i < count($allTrees); $i ++) {
            $treeId = $allTrees[$i]->__get('tree_id');

            for ($j = 0; $j < count($groups); $j ++) {
                $query = "select group_id from " . AF_SECURITY_TABLE
                       . " where tree_id = '" . $treeId . "' and group_id = '"
                       . $groups[$j]['GROUP_ID'] . "'";
                try {
                    self::$db->query($query);
                } catch (Exception $e) {
                    echo _UNKNOWN . ' ' . _TREE_ID . " " . $treeId . ' // ';
                }
                if (self::$db->nb_result() > 0) {
                    $trees[] = $allTrees[$i];
                    break;
                }
            }
        }
        return $trees;
    }

    public function check_where_clause($whereClause, $table, $userId)
    {
        $secCtrl = new SecurityControler();
        $db = new dbquery();
        $res = array(
            'RESULT' => false,
            'TXT' => '',
        );

        if ( empty($userId) || empty($whereClause) || empty($table)) {
            $res['TXT'] = _ERROR_PARAMETERS_FUNCTION;
            return $res;
        }

        $where = ' ' . $whereClause;
        $where = str_replace('\\', '', $where);
        $where = $secCtrl->process_security_where_clause($where, $userId);
        if (str_replace(' ', '', $where) == '') {
            $where = '';
        }
        $where = str_replace('where', ' ', $where);
        $db->connect();

        $query = 'select * from ' . $table . ' where  '. $where;

        $ok = $db->query($query, true);
        if (!$ok) {
            $res['TXT'] = _SYNTAX_ERROR_WHERE_CLAUSE . ' : ' . $whereClause;
            return $res;
        } else {
            $res['TXT'] = _SYNTAX_OK;
            $res['RESULT'] = true;
        }
        $db->disconnect();
        return $res;
    }
}
