<?php

/*
*   Copyright 2012 Maarch
*
*   This file is part of Maarch Framework.
*
*   Maarch Framework is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Maarch Framework is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @defgroup esign electronic signature Module
*/

/**
* @brief   Module esign :  manage class
*
* @file
* @author Laurent Giovannoni <dev@maarch.org>
* @date $date$
* @version $Revision$
* @ingroup esign
*/

$esignDefaultOptions = array();

/**
* @brief   Module esign : manage class
*
* @ingroup esign
*/
class manage_esign
{
    public $esignConfig = array();
    public $error;
    public $db;
    
    function __construct($configFile = 'modules/esign/xml/config_esign.xml')
    {
        if (file_exists($configFile)) {
            $this->esignConfig = $this->object2array(simplexml_load_file($configFile));
        } else {
            $this->error = 'config file not exists:' . $configFile;
        }
        if (empty($this->error)) {
            if ($this->esignConfig['batch_mode']['enabled'] == 'true') {
                if (file_exists($this->esignConfig['batch_mode']['db_query_path'])) {
                    require_once($this->esignConfig['batch_mode']['db_query_path']);
                } else {
                    $this->error = 'db_query_path not exists:'
                        . $this->esignConfig['batch_mode']['db_query_path'];
                }
                if (!is_dir($this->esignConfig['batch_mode']['lib_path'])) {
                    $this->error = 'lib path not correct:'
                        . $this->esignConfig['batch_mode']['lib_path'];
                }
            } else {
                if (file_exists('core/class/class_db.php')) {
                    require_once('core/class/class_functions.php');
                    require_once('core/class/class_db.php');
                } else {
                     $this->error = 'path to class_db not exists';
                }
            }
            if (
                $this->esignConfig['ssl']['enable'] == 'true'
                && !file_exists($this->esignConfig['ssl']['certificate_path'])
            ) {
                $this->error = 'certificate file not exists';
            }
        }
        if (empty($this->error)) {
            if ($this->esignConfig['ssl']['certificate_passphrase'] <> '') {
                $GLOBALS['esignDefaultOptions']['passphrase'] = 
                    $this->esignConfig['ssl']['certificate_passphrase'];
            }
            if ($this->esignConfig['ssl']['certificate_path'] <> '') {
                $GLOBALS['esignDefaultOptions']['local_cert'] = 
                    $this->esignConfig['ssl']['certificate_path'];
            }
            $GLOBALS['esignDefaultOptions']['trace'] = true;
            $GLOBALS['esignDefaultOptions']['cache_wsdl'] = WSDL_CACHE_NONE;
            $GLOBALS['esignDefaultOptions']['certtype'] = 'pem';
            if (isset($GLOBALS['configFile'])) {
                $this->db = new dbquery($GLOBALS['configFile']);
            } else {
                $this->db = new dbquery();
            }
            foreach (array_keys($this->esignConfig['service']) as $service) {
                if ($service <> 'comment') {
                    if (file_exists($this->esignConfig['batch_mode']['lib_path'] 
                        . $this->esignConfig['service'][$service]['lib_name'])
                    ) {
                        include($this->esignConfig['batch_mode']['lib_path'] 
                            . $this->esignConfig['service'][$service]['lib_name']
                        );
                    } else {
                        $this->error = 'lib path not exists:' 
                            . $this->esignConfig['batch_mode']['lib_path']
                            . $this->esignConfig['service'][$service]['lib_name'];
                    }
                }
            }
        }
    }

    function connectToDB()
    {
        $this->db->connect();
    }
    
    function disconnectToDB()
    {
        $this->db->close_conn();
    }
    
    function getFingerprintTosign($resId, $table)
    {
        $fingerprint = '';
        $query = "select fingerprint from " . $table 
            . " where res_id = " . $resId;
        $this->db->query($query);
        if ($this->db->nb_result() > 0) {
            $resultLine = $this->db->fetch_object();
            $fingerprint = $resultLine->fingerprint;
        }
        return $fingerprint;
    }
    
    function getFingerprintMode($resId, $table)
    {
        $fingerprintMode = '';
        $query = "select fingerprint_mode from docserver_types 
            where docserver_type_id in 
                (select docserver_type_id from docservers where docserver_id in
                    (select docserver_id from " . $table 
                    . " where res_id = " . $resId . ")
                )";
        $this->db->query($query);
        if ($this->db->nb_result() > 0) {
            $resultLine = $this->db->fetch_object();
            $fingerprintMode = $resultLine->fingerprint_mode;
        }
        return $fingerprintMode;
    }
    
    function getSignatureToValidate($resId, $table)
    {
        $esignContent = '';
        $query = "select esign_content from " . $table 
            . " where res_id = " . $resId;
        $this->connectToDB();
        $this->db->query($query);
        if ($this->db->nb_result() > 0) {
            $resultLine = $this->db->fetch_object();
            $esignContent = $resultLine->esign_content;
        }
        return $esignContent;
    }
    
    function putInfoInDB($resId, $table, $data, $dbType='POSTGRESQL')
    {
        $toUpdate = '';
        if (is_array($data)) {
            foreach (array_keys($data) as $dataElement) {
                $toUpdate .=  $dataElement . " = '" . $data[$dataElement] . "', ";
                if ($dataElement == 'esign_content') {
                    $toUpdate .=  "esign_date = '" 
                        . $this->currentDatetime($dbType) . "', ";
                }
            }
            $toUpdate = substr($toUpdate, 0, strlen($toUpdate) - 2);
        }
        if (!empty($toUpdate)) {
            $query = "update " . $table . " set " . $toUpdate 
                . " where res_id = " . $resId;
        }
        $this->db->query($query, false, true);
    }
    
    /**
    * Convert an object to an array
    * @param  $object object to convert
    */
    private function object2array($object)
    {
        $return = NULL;
        if (is_array($object)) {
            foreach($object as $key => $value) {
                $return[$key] = $this->object2array($value);
            }
        } else {
            if (is_object($object)) {
                $var = get_object_vars($object);
                if ($var) {
                    foreach ($var as $key => $value) {
                        $return[$key] = 
                            ($key && !$value) ? NULL : $this->object2array($value);
                    }
                } else return $object;
            } else return $object;
        }
        return $return;
    }
    
    /**
    * Return current datetime instruction for each SQL database
    *
    * @return string
    */
    function currentDatetime($dbType)
    {
        if ($dbType == 'SQLSERVER') {
            return ' getdate() ';
        } elseif (
            ($dbType == 'MYSQL' || $dbType == 'POSTGRESQL')
        ) {
            return ' now() ';
        } elseif ($dbType == 'ORACLE') {
            return ' sysdate ';
        }
    }
}
