<?php
/*
*
*   Copyright 2013 Maarch
*
*   This file is part of Maarch Framework.
*
*   Maarch Framework is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Maarch Framework is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @brief    modules tools Class for photo capture, 
*           contains all the functions to handle photo capture
*
* @file     class_modules_tools.php
* @author   Yves Christian Kpakpo <dev@maarch.org>
* @date     $date$
* @version  $Revision$
* @ingroup  photo_capture
*/

// Loads the required class
try {
    require_once("modules/photo_capture/photo_capture_tables.php");
} catch (Exception $e){
    echo $e->getMessage().' // ';
}

class photo_capture extends dbquery
{
    
	/**
	* Build Maarch module tables into sessions vars with a xml configuration
	* file
	*/
	public function build_modules_tables() {
		
		if (file_exists(
		    $_SESSION['config']['corepath'] . 'custom' . DIRECTORY_SEPARATOR
		    . $_SESSION['custom_override_id'] . DIRECTORY_SEPARATOR . "modules"
		    . DIRECTORY_SEPARATOR . "photo_capture" . DIRECTORY_SEPARATOR . "xml"
		    . DIRECTORY_SEPARATOR . "config.xml"
		)
		) {
			$path = $_SESSION['config']['corepath'] . 'custom'
			      . DIRECTORY_SEPARATOR . $_SESSION['custom_override_id']
			      . DIRECTORY_SEPARATOR . "modules" . DIRECTORY_SEPARATOR
			      . "photo_capture" . DIRECTORY_SEPARATOR . "xml" . DIRECTORY_SEPARATOR
			      . "config.xml";
		} else {
			$path = "modules" . DIRECTORY_SEPARATOR . "photo_capture"
			      . DIRECTORY_SEPARATOR . "xml" . DIRECTORY_SEPARATOR
			      . "config.xml";
		}
		$_SESSION['photo_capture'] = array();
		$xmlconfig = simplexml_load_file($path);
		
		foreach ($xmlconfig->PHOTOCAPTURE as $photoCapture) {
			$_SESSION['photo_capture']['type_id'] = (string) $photoCapture->type_id;
			$_SESSION['photo_capture']['standalone'] = (string) $photoCapture->standalone;
			$_SESSION['photo_capture']['width'] = (string) $photoCapture->photo_width;
			$_SESSION['photo_capture']['height'] = (string) $photoCapture->photo_height;
			$_SESSION['photo_capture']['shutter_sound'] = (string) $photoCapture->shutter_sound;
			$_SESSION['photo_capture']['quality'] = (string) $photoCapture->quality;
			$_SESSION['photo_capture']['stealth'] = (string) $photoCapture->stealth;
			$_SESSION['photo_capture']['server_width'] = (string) $photoCapture->server_side_width;
			$_SESSION['photo_capture']['server_height'] = (string) $photoCapture->server_side_height;
		}
		
		//History
		$hist = $xmlconfig->HISTORY;
		$_SESSION['history']['photoadd'] = (string) $hist->photoadd;
		$_SESSION['history']['photoup'] = (string) $hist->photoup;
		$_SESSION['history']['photodel'] = (string) $hist->photodel;
	}
	
	public function isFileTypeAllowed($mimeType, $extension) {

		if (file_exists(
		    $_SESSION['config']['corepath'] . 'custom' . DIRECTORY_SEPARATOR
		    . $_SESSION['custom_override_id'] . DIRECTORY_SEPARATOR . "modules"
		    . DIRECTORY_SEPARATOR . "photo_capture" . DIRECTORY_SEPARATOR . "xml"
		    . DIRECTORY_SEPARATOR . "extensions.xml"
		)
		) {
			$path = $_SESSION['config']['corepath'] . 'custom'
			      . DIRECTORY_SEPARATOR . $_SESSION['custom_override_id']
			      . DIRECTORY_SEPARATOR . "modules" . DIRECTORY_SEPARATOR
			      . "photo_capture" . DIRECTORY_SEPARATOR . "xml" . DIRECTORY_SEPARATOR
			      . "extensions.xml";
		} else {
			$path = "modules" . DIRECTORY_SEPARATOR . "photo_capture"
			      . DIRECTORY_SEPARATOR . "xml" . DIRECTORY_SEPARATOR
			      . "extensions.xml";
		}
		$xmlconfig = simplexml_load_file($path);
		$ext_list = array();
		$i = 0;
		foreach ($xmlconfig->FORMAT as $FORMAT) {
			$ext_list[$i] = array(
				'name' => (string) $FORMAT->name,
				'mime' => (string) $FORMAT->mime
			);
			$i++;
		}
		$type_state = false;
		for ($i=0;$i<count($ext_list);$i++) {
			if (
				$ext_list[$i]['mime'] == $mimeType 
				&& strtolower($ext_list[$i]['name']) == $extension
			) {

				$type_state = true;
				break;
			}
		}
		return $type_state;
	}
	
	private function _getWidth($image) {
		return imagesx($image);
	}
   
	private function _getHeight($image) {
		return imagesy($image);
	}
   
	public function resize($pathToImage, $width, $height) {
	
		$imageInfo = getimagesize($pathToImage);
		$mime = $imageInfo['mime'];

		switch ($mime) {
            case 'image/jpeg':
				$image =  imagecreatefromjpeg($pathToImage);
				$newImage = imagecreatetruecolor($width, $height);
				imagecopyresampled($newImage, $image, 0, 0, 0, 0, $width, $height, $this->_getWidth($image), $this->_getHeight($image));
				unlink($pathToImage);
				imagejpeg($newImage, $pathToImage);
			break;
			case 'image/gif':
				$image =  imagecreatefromgif($pathToImage);
				$newImage = imagecreatetruecolor($width, $height);
				imagecopyresampled($newImage, $image, 0, 0, 0, 0, $width, $height, $this->_getWidth($image), $this->_getHeight($image));
				unlink($pathToImage);
				imagegif($newImage, $pathToImage);
			break;
			case 'image/png':
				$image =  imagecreatefrompng($pathToImage);
				$newImage = imagecreatetruecolor($width, $height);
				imagecopyresampled($newImage, $image, 0, 0, 0, 0, $width, $height, $this->_getWidth($image), $this->_getHeight($image));
				unlink($pathToImage);
				imagepng($newImage, $pathToImage);
			break;
			default: 
				throw Exception(_WRONG_FILE_TYPE);
		}
	}
   
	public function addPhoto($coll_id, $linked_id, $from_doc = true, $title = _PHOTO_TITLE) {
        
        $func = new functions();
        $error = '';
        if (empty($coll_id)) {
            $error .= _PHOTO_CAPTURE . ': '. _MISSING . _COLL_ID . ".<br/>";
        }
		
		if (empty($linked_id)) {
			$error .= _PHOTO_CAPTURE . ': '._MISSING . _PHOTO_LINKED_ID . ".<br/>";
		}
        
        if (empty($_SESSION['photofile']['local_path'])) {
            $error .= _PHOTO_CAPTURE . ': '._FILE_MISSING . ".<br/>";
        }

        if ($_SESSION['photofile']['size'] == 0) {
            $error .= _PHOTO_CAPTURE . ': '._FILE_EMPTY . ".<br />";
        }
		
        if (empty($error)) {
            require_once 'core/class/docservers_controler.php';
            $docserverControler = new docservers_controler();
            
            // $docserver = $docserverControler->getDocserverToInsert(
                // $coll_id
            // );
			
			//Use private docserver
			$docserver = "PHOTO_CAPTURE";
			
            if (empty($docserver)) {
                $error .= _DOCSERVER_ERROR . ' : '
                    . _NO_AVAILABLE_DOCSERVER . ". " . _MORE_INFOS . ".";
            } else {			
                // some checking on docserver size limit
                $newSize = $docserverControler->checkSize(
                    $docserver, $_SESSION['photofile']['size']
                );

                if ($newSize == 0) {
                    $error .= _DOCSERVER_ERROR . ' : '
                        . _NOT_ENOUGH_DISK_SPACE . ". " . _MORE_INFOS . ".";
                } else {
					
                    $fileToStoreInfos = array(
                        "tmpDir"      => $_SESSION['config']['tmppath'],
                        "size"        => $_SESSION['photofile']['size'],
                        "format"      => $_SESSION['photofile']['format'],
                        "tmpFileName" => $_SESSION['photofile']['name']
                    );
					//
                    $storeResult = array();
                    $storeResult = $docserverControler->storeResourceOnDocserver(
                        'photo_capture', $fileToStoreInfos
                    );
					// $this->show_array($storeResult);
                    if (isset($storeResult['error']) && $storeResult['error'] <> '') {
                        $error .= $storeResult['error'];
                    } else {

                        $_SESSION['data_photo'] = array();
                        array_push(
                            $_SESSION['data_photo'],
                            array(
                                'column' => "typist",
                                'value' => $_SESSION['user']['UserId'],
                                'type' => "string"
                            )
                        );
                        array_push(
                            $_SESSION['data_photo'],
                            array(
                                'column' => "format",
                                'value' => $_SESSION['photofile']['format'],
                                'type' => "string"
                            )
                        );
                        array_push(
                            $_SESSION['data_photo'],
                            array(
                                'column' => "docserver_id",
                                'value' => $storeResult['docserver_id'],
                                'type' => "string"
                            )
                        );
                        array_push(
                            $_SESSION['data_photo'],
                            array(
                                'column' => "status",
                                'value' => 'NEW',
                                'type' => "string"
                            )
                        );
                        array_push(
                            $_SESSION['data_photo'],
                            array(
                                'column' => "offset_doc",
                                'value' => ' ',
                                'type' => "string"
                            )
                        );
                        array_push(
                            $_SESSION['data_photo'],
                            array(
                                'column' => "logical_adr",
                                'value' => ' ',
                                'type' => "string"
                            )
                        );
                        array_push(
                            $_SESSION['data_photo'],
                            array(
                                'column' => "title",
                                'value' => $title,
                                'type' => "string"
                            )
                        );
                        array_push(
                            $_SESSION['data_photo'],
                            array(
                                'column' => "coll_id",
                                'value' => $coll_id,
                                'type' => "string"
                            )
                        );
						if ($from_doc === true) {
							array_push(
								$_SESSION['data_photo'],
								array(
									'column' => "res_id_master",
									'value' => $linked_id,
									'type' => "integer"
								)
							);
						} else if ($from_doc === false){
							array_push(
								$_SESSION['data_photo'],
								array(
									'column' => "folders_system_id",
									'value' => $linked_id,
									'type' => "integer"
								)
							);
						}
						// $this->show_array($_SESSION['data_photo']);
						
						require_once "core/class/class_resource.php";
						$resource = new resource();
						//
                        $id = $resource->load_into_db(
                            PHOTO_TABLE,
                            $storeResult['destination_dir'],
                            $storeResult['file_destination_name'] ,
                            $storeResult['path_template'],
                            $storeResult['docserver_id'], 
                            $_SESSION['data_photo'],
                            $_SESSION['config']['databasetype']
                        );
						
                        if ($id == false) {
                            $error .= $resource->get_error();
                            // echo $resource->get_error();
                            // $resource->show();
                            //exit();
                        } else {
							
							$result = array(
								'id' => $id,
								'linked_id' => $linked_id,
								'coll_id' => $coll_id,
								'mime_type' => $_SESSION['photofile']['mime'],
								'ext' => $format,
								'title' => $title,
								'tmp_path' => $_SESSION['config']
								['tmppath'],
								'file' => $_SESSION['photofile']['name'],
								'file_path' => $_SESSION['photofile']['local_path']
							);
							// $_SESSION['info'] .= _NEW_PHOTO_ADDED.'<br/>';
                            if ($_SESSION['history']['photoadd'] == "true") {
								
                                require_once "core/class/class_history.php";
								$hist = new history();
								
								$hist->add(
                                    PHOTO_TABLE, $id, 'ADD', 'photoadd',
                                    _NEW_PHOTO_ADDED .' (' . $id . ')',
                                    $_SESSION['config']['databasetype'],
                                    'photo_capture'
                                );
								if ($from_doc === true) {
									require_once "core/class/class_security.php";
									$sec = new security();
									$view = $sec->retrieve_view_from_coll_id(
										$coll_id
									);
									$hist->add(
										$view, $linked_id, 'UP', 'resup',
										_NEW_PHOTO_ADDED . ' ' . _ON_DOC_NUM
										. $linked_id . ' ('.$id.')',
										$_SESSION['config']['databasetype'],
										'photo_capture'
									);
								} else if ($from_doc === false){
									require_once 'modules/folder/folder_tables.php';
									$hist->add(
										FOLD_FOLDERS_TABLE, $linked_id, 'UP', 'folderup',
										_NEW_PHOTO_ADDED . ' ' . _ON_FOLDER_NUM
										. $linked_id . ' ('.$id.')',
										$_SESSION['config']['databasetype'],
										'photo_capture'
									);
								}
							}
							
							//Reset photofile session values
							unset($_SESSION['photofile']);
							
							//
							return $result;
                        }
                    }
                }
            }
        }
		
		return $error;
    }
    
    public function getPhoto($coll_id, $id, $from_doc = true){
     
        if (empty($id)) {
            return false;
        } else {
            $db = new dbquery();
            $db->connect();
            //
			if ($from_doc === true) {
				$fieldID = 'res_id_master';
			} else if ($from_doc === false){
				$fieldID = 'folders_system_id';
			}
            $db->query(
                "select * from " . PHOTO_TABLE
                . " where ".$fieldID." = " 
                . $id ." and coll_id = '".$coll_id
				. "' and status <> 'DEL'"
            );

            // $db->show();
            if ($db->nb_result() == 0) {
                return false;
            } else {
                $line = $db->fetch_object();
            
                $resId = $line->res_id;
                $collId = $line->coll_id;
                $idMaster = $line->$fieldID;
				$title = $line->title;
				$docserver = $line->docserver_id;
                $path = $line->path;
                $filename = $line->filename;
                $format = $line->format;
				
				//Dccserver Stuff
				require_once 'core/docservers_tools.php';
				$db->query(
                        "select path_template from " 
						. _DOCSERVERS_TABLE_NAME . " where docserver_id = '" 
						. $docserver . "'"
                );
                $lineDoc = $db->fetch_object();
                $docserver = $lineDoc->path_template;
                $file = $docserver . $path . $filename;
                $file = str_replace("#", DIRECTORY_SEPARATOR, $file);
				
				if (file_exists($file)) {
					$mimeType = Ds_getMimeType(
                            $file
                    );
					$fileNameOnTmp = 'tmp_photo_file_' 
						. $_SESSION['user']['UserId']
						. '_' . rand() . '.' . strtolower($format);
					$filePathOnTmp = $_SESSION['config']
						['tmppath'] . DIRECTORY_SEPARATOR
						. $fileNameOnTmp;
					copy($file, $filePathOnTmp);
					//
					$result = array(
						'id' => $resId,
                        'linked_id' => $id,
                        'coll_id' => $collId,
                        'mime_type' => $mimeType,
                        'ext' => $format,
                        'title' => $title,
                        'tmp_path' => $_SESSION['config']
                        ['tmppath'],
                        'file' => $fileNameOnTmp,
                        'file_path' => $filePathOnTmp
					);
					return $result;
				} else {
					return false;
				}
            }
        }
    }
	
	public function showPhoto($coll_id, $id, $canModify = false, $linkMode = false) {
		
		$photo = '';
		$path_to_ajax_script = $_SESSION['config']['businessappurl']
				.'index.php?display=true&module=photo_capture&page=photo_capture_ajax_script';
				
		$photoArray = $this->getPhoto($coll_id, $id);

		if (is_array($photoArray)) {
			//Can modify?
			$canModify ? $jsAction = 'onClick="showPhotoForm(\''.$path_to_ajax_script.'&mode=up&id='
				.$photoArray['linked_id'].'&coll_id='.$photoArray['coll_id']
				.'&p_id='.$photoArray['id'].'\');" alt="'._CLICK_TO_MODIFY
				.'" title="'._CLICK_TO_MODIFY.'" class="photoLink" ' : $jsAction='';
			//
			//A link mode (smallest)
			if ($linkMode ) {
				//A completer...
				/*
				$content = "{'identifierDetailFrame' : '".$photoArray['linked_id']
					."', '' : '<img src=\"static.php?dir=tmp&filename="
					.$photoArray['file']."\" align=\"absmiddle\">'}";
				$photo = '<div ';
				$photo .= 'id="return_previsualise" ';
				$photo .= 'style="';
				$photo .= 'display: none; ';
				$photo .= 'border-radius: 10px; ';
				$photo .= 'box-shadow: 10px 10px 15px rgba(0, 0, 0, 0.4); ';
				$photo .= 'padding: 10px; ';
				$photo .= 'width: auto; ';
				$photo .= 'height: auto; ';
				$photo .= 'position: absolute; ';
				$photo .= 'top: 0; ';
				$photo .= 'left: 0; ';
				$photo .= 'z-index: 999; ';
				$photo .= 'background-color: rgba(255, 255, 255, 0.9); ';
				$photo .= 'border: 3px solid #459ed1;';
				$photo .= '" ';
				$photo .= '>';
				$photo .= '<input type="hidden" id="identifierDetailFrame" value="" />';
				$photo .= '</div>';
				$photo .= '<a href="javascript://" '.$_jsAction
					. ' onMouseOver="previsualiseAdminRead(event, '
					. htmlspecialchars($content).');"'
					. ' onMouseOut="$(\'identifierDetailFrame\').setValue(\'\');'
					. '$(\'return_previsualise\').style.display=\'none\';"'
					. '><img src="static.php?module=photo_capture&filename='
					. 'photo_capture.gif" align="absmiddle">'
					. '&nbsp;&nbsp;'._PHOTO_TITLE.'</a>';
				*/
			} else {
				//-->Photo container
				$photo .= '<div id="photoDiv" class="photoDiv" style="display:block;">';
				//Show photo
				$photo .= '<div id="photoTarget" style="display:block;">';
				$photo .= '<a href="javascript://" '.$jsAction
					.'><img src="static.php?dir=tmp&filename='
					.$photoArray['file'].'" align="absmiddle"></a>';
				$photo .= '</div>';
				//Photo toolbar
				$photo .= '<div id="photoTools">';
				if ($canModify) {
					$photo .= '<a href="javascript://" '
						.$jsAction.'><img src="'.$_SESSION['config']['businessappurl']
						.'static.php?module=photo_capture&filename='
						.'photo_capture.gif" align="absmiddle" alt="'
						._MODIFY.'" title="'._MODIFY.'" /></a><a href="javascript://" '
						.'onClick="if(confirm(\''._REALLY_DELETE.'?\')) makeAjaxAction(\''
						.$path_to_ajax_script.'&mode=del&id='
						.$photoArray['linked_id'].'&coll_id='.$photoArray['coll_id']
						.'&p_id='.$photoArray['id'].'\');"><img src="'
						.$_SESSION['config']['businessappurl']
						.'static.php?module=photo_capture&filename='
						.'picto_disabled.gif" align="absmiddle" alt="'
						._DELETE.'" title="'._DELETE.'>" /></a>';
				}
				$photo .= '</div>';
				$photo .= '</div>';//<--
			}
		} else {
			//Can modify?
			$canModify ? $jsAction = 'onClick="showPhotoForm(\''.$path_to_ajax_script.'&mode=add&id='
				.$id.'&coll_id='.$coll_id.'\');" alt="'._CLICK_TO_MODIFY
				.'" title="'._CLICK_TO_MODIFY.'" class="photoLink" ' : $jsAction='';
			//--> Photo container
			$photo .= '<div id="photoDiv" class="photoDiv" style="display:block;">';
			//Show photo
			$photo .= '<div id="photoTarget" style="display:block;">';
			$photo .= '<a href="javascript://" '
				.$jsAction.'><img src="static.php?module=photo_capture&filename=default.png" align="absmiddle"></a>';
			$photo .= '</div>';
			//Photo toolbar
			$photo .= '<div id="photoTools">';
			if ($canModify) {
				$photo .= '<a href="javascript://" '
					.$jsAction.'><img src="'.$_SESSION['config']['businessappurl']
					.'static.php?module=photo_capture&filename='
					.'capture.gif" align="absmiddle" alt="'
					._ADD.'" title="'._ADD.'" /></a>';
			}
			$photo .= '</div>';
			$photo .= '</div>';	//<--
		}
		return $photo;
	}
	
	public function deletePhoto($id, $from_doc = true){
     
        if (empty($id)) {
            return false;
        } else {
            $db = new dbquery();
            $db->connect();
			
			$db->query(
                "select coll_id, res_id_master, folders_system_id from " 
				. PHOTO_TABLE . " where res_id = " 
                . $id . " and status <> 'DEL'"
            );
			
			if ($db->nb_result() == 0) {
                return false;
            } else {
				//Get some info
				$line = $db->fetch_object();
				
                $collId = $line->coll_id;
				$resIdMaster = $line->res_id_master;
				$folderId = $line->folders_system_id;
				 
				$db->query(
					"update " . PHOTO_TABLE
					. " set status = 'DEL' where res_id = " 
					. $id
				);
				
				//History
				if ($_SESSION['history']['photodel'] == "true") {
					require_once "core/class/class_history.php";
					$hist = new history();
					
					$hist->add(
						PHOTO_TABLE, $id, 'DEL', 'photodel',
						_PHOTO_DELETED .' (' . $id . ')',
						$_SESSION['config']['databasetype'],
						'photo_capture'
					);
					if ($from_doc === true) {
						require_once "core/class/class_security.php";
						$sec = new security();
						$view = $sec->retrieve_view_from_coll_id(
							$collId
						);
						$hist->add(
							$view, $resIdMaster, 'UP', 'resup',
							_PHOTO_DELETED . ' ' . _ON_DOC_NUM
							. $resIdMaster . ' ('.$id.')',
							$_SESSION['config']['databasetype'],
							'photo_capture'
						);
					} else if ($from_doc === false){
						require_once 'modules/folder/folder_tables.php';
						$hist->add(
							FOLD_FOLDERS_TABLE, $folderId, 'UP', 'folderup',
							_PHOTO_DELETED . ' ' . _ON_FOLDER_NUM
							. $folderId . ' ('.$id.')',
							$_SESSION['config']['databasetype'],
							'photo_capture'
						);
					}
				}
			}
		}
	}
}

