<?php
/*
 *    Copyright 2008,2009, 2010 Maarch
 *
 *  This file is part of Maarch Framework.
 *
 *   Maarch Framework is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Maarch Framework is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *    along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Codes retour
 *
 *  1: fichier de configuration manquant
 *  3: Problème d'arguments
 *  7: Le fichier de configuration n'existe pas
 *  9: Impossible charger le fichier de configuration
 *  12: Problème avec l'include path PHP
 */

session_start();
date_default_timezone_set('Europe/Paris');
register_shutdown_function('session_destroy');

/**
 * 
 * Check the resource integrity
 * @param array $adrEntry
 */
function checkResource($adrEntry) {
    //var_dump($adrEntry);
    $dc = new docservers_controler();
    $retour = $dc->viewResourceAdr($adrEntry);
    // BEGIN DEBUG
    //var_dump($retour);
    // END DEBUG
    if (file_exists($retour['file_path'])) {
        @unlink($retour['file_path']);
        return true;
    }
    else {
        return false;
    }
}

/**
 * 
 * Insert the checking informations into the database
 * @param integer $startResId
 * @param integer $stopResId
 * @param integer $lastCycleId
 * @param datetime $startDateTime
 * @param integer $commitCounter
 * @param integer $failCounter
 */
function commitCheck($startResId, $stopResId, $lastCycleId, $startDateTime, $commitCounter, $failCounter) {
    $db = new request();
    $db->connect();
//$db->query("SELECT * FROM res_pja");
    $db->query("
        INSERT INTO ".$_SESSION['config']['rp_history_table']." (
          table_name,
          rp_cycle,
          start_res_id,
          stop_res_id,
          start_date,
          stop_date,
          res_count,
          fail_count
    	)
    	VALUES (
    		'".$_SESSION['config']['res_table']."',
    		'".$lastCycleId."',
    		'".$startResId."',
    		'".$stopResId."',
    		'".$startDateTime."',
    		'".date("Y-m-d H:i:s")."',
    		'".$commitCounter."',
    		'".$failCounter."'
    	)
    ");
}

/**
 * 
 * Launch a checking starting from a res ID at a specific cycle ID
 * @param integer $lastResId
 * @param integer $lastCycleId
 * @param time $startTime
 */
function checkResourceWithBounds($lastResId, $lastCycleId, $startTime) {
    $db = new request();
    $db2 = new request();
    // We initialize the number of fail during the checking
    $failCounter = 0;
    // We check every res_id > $lastResId
    $db->connect();
    $db->query("
    	SELECT r.res_id, r.docserver_id, r.filename, r.path, r.offset_doc, r.fingerprint, r.is_multi_docservers, d.path_template
    	FROM ".$_SESSION['config']['res_table']." r, ".$_SESSION['config']['docservers_table']." d
    	WHERE r.res_id > ".$lastResId." AND r.docserver_id = d.docserver_id ORDER BY r.res_id ASC"
    );
    while (($res = $db->fetch_array())) {
        $_SESSION['current_res_id'] = $res['res_id'];
        // If the resource is in OAIS mode
        if ($res['is_multi_docservers'] == 'Y') {
            // We get the path in the adr_x table
            $db2->connect();
            $db2->query("
        	SELECT a.res_id, a.docserver_id, a.filename, a.path, a.offset_doc, a.fingerprint, d.path_template
        	FROM ".$_SESSION['config']['adr_table']." a, ".$_SESSION['config']['docservers_table']." d
        	WHERE a.res_id = '".$res['res_id']."' AND a.docserver_id = d.docserver_id"
            );
            // For each entry in the adr_x table, we iterate
            while (($adr = $db2->fetch_array())) {
                $filePath = $adr['path_template'].'/'.str_replace('#', '/', $adr['path']).'/'.$adr['filename'];
                writeLog("Processing adr address of res_id = ".$res['res_id']." in cycle = ".$lastCycleId." AIP address is ".$filePath." and file in AIP is ".$adr['offset_doc'], 'VIEW', 'OK', 'DEBUG');
                if (!checkResource($adr)) {
                    writeLog("The resource is corrupted : res_id = ".$res['res_id']." AIP address is ".$filePath." and file in AIP is ".$adr['offset_doc'], 'VIEW', 'KO', 'ERROR');
                    $failCounter++;
                }
            }
        }
        else {
            $filePath = $res['path_template'].'/'.str_replace('#', '/', $res['path']).'/'.$res['filename'];
            writeLog("Processing res address res_id = ".$res['res_id']." in cycle = ".$lastCycleId." file address is ".$filePath, 'VIEW', 'OK', 'DEBUG');
            if (!checkResource($res)) {
                writeLog("A resources is corrupted : res_id = ".$res['res_id']." in cycle = ".$lastCycleId." file address is ".$filePath, 'VIEW', 'KO', 'ERROR');
                $failCounter++;
            }
        }
        // We keep the first processed res_id in memory
        if ($commitCounter == 0) {
            writeLog("Start res_id = ".$res['res_id'], 'VIEW', 'OK', 'INFO');
            $startResId = $res['res_id'];
            $startDateTime = date("Y-m-d H:i:s");
        }
        $commitCounter++;
        // The commit number limit is reached
        if ($commitCounter % $_SESSION['config']['commit_step'] == 0) {
            writeLog("Commiting the processing resources", 'VIEW', 'OK', 'INFO');
            commitCheck($startResId, $res['res_id'], $lastCycleId, $startDateTime, $commitCounter, $failCounter);
            $commitCounter = 0;
            // The commit is done, how many seconds are remained to continue?
            // If time is up, we stop the script
            $elapsedTime = time() - $startTime;
            if ($elapsedTime > $_SESSION['config']['max_execution_time']) {
                return;
            }
        }
    }
    // Time is remaining to process resources
    // and we reached the lastest resource
    // we start again at 0 increasing the cycle by an increment of 1
    $elapsedTime = time() - $startTime;
    if ($elapsedTime <= $_SESSION['config']['max_execution_time']) {
        $lastCycleId++;
        checkResourceWithBounds(0, $lastCycleId, $startTime);
    }
}

/**
 * 
 * Launch a checking starting at the lastest known position
 * @param unknown_type $startTime
 */
function checkResources($startTime) {
    $db = new request();
    $db->connect();
    // We look at the table rp_history in order to see the latest index
    $db->query("SELECT MAX(stop_date) as maxidate FROM ".$_SESSION['config']['rp_history_table']);
    if (($line = $db->fetch_object())) {
        $lastDate = $line->maxidate;
        if (empty($lastDate)) {
            $lastDate = date("Y-m-d H:i:s");
        }
        $db->connect();
        $db->query("SELECT stop_res_id, rp_cycle FROM ".$_SESSION['config']['rp_history_table']." WHERE stop_date = '".$lastDate."'");
        if (($line = $db->fetch_object())) {
            $lastResId = $line->stop_res_id;
            $lastCycleId = $line->rp_cycle;
        }
        else {
            $lastResId = 1;
            $lastCycleId = 1;
        }
    }
    else {
        $lastResId = 1;
        $lastCycleId = 1;
    }
    writeLog("Starting record patrol with rp_cycle = ".$lastCycleId." and last res_id = ".$lastResId, 'VIEW', 'OK', 'INFO');

    // We check every res_id > $lastResId
    $db->connect();
    $db->query("
    	SELECT res_id, docserver_id, filename, path, offset_doc, fingerprint, is_multi_docservers 
    	FROM ".$_SESSION['config']['res_table']." 
    	WHERE res_id > ".$lastResId."
        LIMIT 1
    ");

    // Counter initialization
    $commitCounter = 0;
    // If there is remaining resources
    if ($db->nb_result() > 0) {
        writeLog("The cycle is not finished, processing remaining resources", 'VIEW', 'OK', 'INFO');
        checkResourceWithBounds($lastResId, $lastCycleId, $startTime);
    }
    // We must start at 0 increasing rp_cycle by an increment of 1 
    else {
	if ($lastResId == 1) {
            writeLog("There is no archived files in database", 'VIEW', 'OK', 'WARN');
            exit(0);
        }
        writeLog("The cycle is finished, starting a new cycle", 'VIEW', 'OK', 'INFO');
        $lastCycleId++;
        checkResourceWithBounds(0, $lastCycleId, $startTime, $startTime);
    }
}

/**
 * Delete accents
 *
 * @param String $str
 * @param String $charset
 */
function wd_remove_accents($str, $charset ='utf-8')
{
    $str = htmlentities($str, ENT_NOQUOTES, "utf-8");
    $str = preg_replace('#\&([A-za-z])(?:uml|circ|tilde|acute|grave|cedil|ring)\;#', '\1', $str);
    $str = preg_replace('#\&([A-za-z]{2})(?:lig)\;#', '\1', $str);
    $str = preg_replace('#\&[^;]+\;#','', $str);
    return $str;
}

/**
 *
 * write a log entry with a specific log level
 * @param object $logger Log4php logger
 * @param string $logLine Line we want to trace
 * @param enum $level Log level
 */
function writeLog4php($logger, $logLine, $level) {
    $logLine = wd_remove_accents($logLine);
    switch ($level) {
        case 'DEBUG':
            $logger->debug($logLine);
            break;
        case 'INFO':
            $logger->info($logLine);
            break;
        case 'WARN':
            $logger->warn($logLine);
            break;
        case 'ERROR':
            $logger->error($logLine);
            break;
        case 'FATAL':
            $logger->fatal($logLine);
            break;
    }
}


/**
 * Write on the log file
 * @param  $eventInfo string text which is written in the log file
 */
function writeLog($message, $type = 'VIEW', $result = 'OK', $level = 'INFO')
{
    if ($_SESSION['config']['check_history'] == 'true') {
        $remote_ip = '127.0.0.1';
        Logger::configure($_SESSION['config']['log4php_path']);
        $logger = Logger::getLogger($_SESSION['config']['log4php_logger']);
        $searchPatterns = array('%ACCESS_METHOD%', '%RESULT%', '%CODE_METIER%', '%HOW%', '%WHAT%', '%REMOTE_IP%');
        $replacePatterns = array('Script', $result, 'PJA', $type, $message, $remote_ip);
        $logLine = str_replace($searchPatterns, $replacePatterns, $_SESSION['config']['log4php_format']);
        if ($_SESSION['config']['log4php_enabled'] == 'true') {
            writeLog4php($logger, $logLine, $level);
        }
    }
}

// Class to manage files includes errors
class IncludeFileError extends Exception
{
    public function __construct($file)
    {
        $this->file = $file;
        parent::__construct("Include File \"$file\" is missing!", 1);
    }
}

// File include function which test file existence before inclusion to avoid
// error and throw an IncludeFileError error in case file is missing
function MyInclude($file, $core, $custom, $notOnce = false)
{
    if (file_exists($core."/".$file) || file_exists($core."/custom/".$custom."/".$file)) {
        if ($notOnce) {
            include($file);
        }
        else {
            include_once($file);
        }
    } else {
        throw new IncludeFileError($file);
    }
}

try {
    include 'Maarch_CLITools/ArgsParser.php';
    include 'Maarch_CLITools/FileHandler.php';
    include 'Maarch_CLITools/ConsoleHandler.php';
}
catch(IncludeFileError $e) {
    echo "Maarch_CLITools required ! \n (pear.maarch.org)\n";
    exit();
}



// Define this sripts arguments
function defineScriptArgs()
{
    // Define allowed arguments in the script
    $argsparser = new ArgsParser();
    $argsparser->add_arg(
        'config', array(
            'short'     => 'f',
            'long'      => 'config',
            'mandatory' => true,
            'help'      => 'Config file path is mandatory.',
    )
    );

    return $argsparser;
}

function main() {
    // Define script arguments
    $argsparser = defineScriptArgs();

    // Parse script options
    try {
        $options = $argsparser->parse_args($GLOBALS['argv']);
        // If option = help then options = false and the script continues ...
        if ($options == false) {
            exit(0);
        }
    }
    catch(MissingArgumentError $e)
    {
        if ($e->arg_name == 'config') {
            echo "Configuration file missing\n";
            exit(1);
        } else {
            echo "Arguments Error, try -h to view help\n";
            exit(3);
        }
    }
    // Assign values to global variables
    // $GLOBALS['updateMode'] = $options['update'];
    $GLOBALS['configFile'] = $options['config'];

    // Test existence of config file
    if (!file_exists($GLOBALS['configFile'])) {
        echo "Configuration file " . $GLOBALS['configFile'] . "does not exist\n";
        exit(7);
    }

    // Load config file
    echo "Load xml config file : ".$GLOBALS['configFile']."\n";

    $xmlconfig = simplexml_load_file($GLOBALS['configFile']);

    if ($xmlconfig == FALSE) {
        echo 'Error on loading config file : ' . $GLOBALS['configFile']."\n";
        exit(9);
    }

    $config = $xmlconfig->CONFIG;
    $lang = (string) $config->lang;
    $maarchDirectory = (string) $config->MaarchDirectory;
    $maarchApps = (string) $config->MaarchApps;
    $maarchCustom = (string) $config->MaarchCustom;
    $maxExecutionTime = (string) $config->MaxExecutionTime;
    $commitStep =  (string) $config->CommitStep;
    $tmpCheckPath = (string) $config->TmpCheckPath;
    $debug = (string) $config->debug;
    $log4phpEnabled = (string) $config->Log4phpEnabled;
    $_SESSION['config']['tmppath'] = $tmpCheckPath;


    $log4PhpPath = (string) $config->Log4PhpPath;
    $log4PhpLogger = (string) $config->Log4PhpLogger;
    $log4phpFormat = (string) $config->Log4phpFormat;

    $databasetype = (string) $xmlconfig->CONFIG_BASE->databasetype;
    $databasename = (string) $xmlconfig->CONFIG_BASE->databasename;
    $databaseserver = (string) $xmlconfig->CONFIG_BASE->databaseserver;
    $databaseserverport = (string) $xmlconfig->CONFIG_BASE->databaseserverport;
    $databaseuser = (string) $xmlconfig->CONFIG_BASE->databaseuser;
    $databasepassword = (string) $xmlconfig->CONFIG_BASE->databasepassword;
    // Variables de session nécessaires
    $_SESSION['config']['lang'] = $config->lang;
    if (!empty($maarchCustom)) {
        $_SESSION['custom_override_id'] = $maarchCustom;
    }
    $_SESSION['config']['app_id'] = $maarchApps;
    $_SESSION['config']['corepath'] = $maarchDirectory;
    $_SESSION['config']['max_execution_time'] = $maxExecutionTime * 60;
    $_SESSION['config']['commit_step'] = $commitStep;
    $_SESSION['config']['debug'] = $debug;
    $_SESSION['config']['log4php_enabled'] = $log4phpEnabled;

    $_SESSION['config']['databaseserver'] = $databaseserver;
    $_SESSION['config']['databaseserverport'] = $databaseserverport;
    $_SESSION['config']['databaseuser'] = $databaseuser;
    $_SESSION['config']['databasepassword'] = $databasepassword;
    $_SESSION['config']['databasename'] = $databasename;
    $_SESSION['config']['databasetype'] = $databasetype;
    $_SESSION['tablename']['history'] = "history";
    $_SESSION['config']['log4php_logger'] = $log4PhpLogger;
    $_SESSION['config']['log4php_path'] = $log4PhpPath;
    $_SESSION['config']['log4php_format'] = $log4phpFormat;
    $GLOBALS['databasetype'] = $databasetype;

    $table = (string) $xmlconfig->CONFIG_BASE->res;
    $_SESSION['config']['res_table'] = $table;
    $adrTable = (string) $xmlconfig->CONFIG_BASE->adr;
    $_SESSION['config']['adr_table'] = $adrTable;
    $rpHistory = (string) $xmlconfig->CONFIG_BASE->rp_history;
    $_SESSION['config']['rp_history_table'] = $rpHistory;
    $history = (string) $xmlconfig->CONFIG_BASE->history;
    $_SESSION['config']['history_table'] = $history;
    $checkHistory = (string) $xmlconfig->HISTORY->check;
    $_SESSION['config']['check_history'] = $checkHistory;
    $docservers = (string) $xmlconfig->CONFIG_BASE->docservers;
    $_SESSION['config']['docservers_table'] = $docservers;
    if ($logLevel == 'DEBUG') {
        error_reporting(E_ALL);
    }

    unset($xmlconfig);

    // Include maarch files
    set_include_path(
    get_include_path() .
    PATH_SEPARATOR . $maarchDirectory."/custom/".$maarchCustom.
    PATH_SEPARATOR . $maarchDirectory);
    try {
            if ($log4phpEnabled == 'true') {
                MyInclude("apps".DIRECTORY_SEPARATOR."maarch_entreprise".DIRECTORY_SEPARATOR."tools".DIRECTORY_SEPARATOR."log4php".DIRECTORY_SEPARATOR."Logger.php",
                $maarchDirectory, $maarchCustom);
            }
            MyInclude(
            'core' . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'class_functions.php', $maarchDirectory, $maarchCustom
            );
            MyInclude(
            'core' . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'class_db.php', $maarchDirectory, $maarchCustom
            );
            MyInclude(
            'core' . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'class_core_tools.php', $maarchDirectory, $maarchCustom
            );
            MyInclude(
            'apps/'.$maarchApps.'/'
            . 'apps_tables.php', $maarchDirectory, $maarchCustom
            );
            MyInclude(
            'apps/'.$maarchApps.'/'
            . 'class/class_business_app_tools.php', $maarchDirectory, $maarchCustom
            );
            MyInclude('core/core_tables.php', $maarchDirectory, $maarchCustom);
            MyInclude(
            'core' . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'class_request.php', $maarchDirectory, $maarchCustom
            );
            MyInclude(
            'core' . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'class_history.php', $maarchDirectory, $maarchCustom
            );
            MyInclude(
            'core' . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'class_security.php', $maarchDirectory, $maarchCustom
            );
            MyInclude(
            'core' . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'class_resource.php', $maarchDirectory, $maarchCustom
            );
            MyInclude(
            'core' . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'docservers_controler.php', $maarchDirectory, $maarchCustom
            );
            MyInclude(
            'core' . DIRECTORY_SEPARATOR . 'docservers_tools.php', $maarchDirectory, $maarchCustom
            );
    } catch(IncludeFileError $e) {
        echo 'Problem with the php include path : ' . get_include_path()."\n";
        exit(12);
    }
    checkResources(time());
}

main();
