<?php

require_once 'core/tests/class/DataObjectController.php';
require_once 'core/tests/class/ViewController.php';

//*****************************************************************************
// 1 - Init
//*****************************************************************************
// Load returned View as source of data
$ViewController = new ViewController();
$view = $ViewController->loadHTML($_REQUEST['ArchivalAgreement']);

$messageController = new MessageController();
$messageController->loadMessageFile(
    'modules/records_management/lang/ArchivalAgreement.xml'
);

// Load Schema
$dataObjectController = new DataObjectController();
$dataObjectController->loadXSD(
    $_SESSION['config']['corepath'] 
    . 'modules/records_management/schema/RecordsManagement.xsd'
);

// Load data object from session
$rm_agreement = $dataObjectController->loadXML($_SESSION['ArchivalAgreement']);

//*****************************************************************************
// 2 - Update DataObject
//*****************************************************************************
$log = array();
$log[] = "Update RM_AGREEMENT...";
$rm_agreements_properties = $dataObjectController->getObjectProperties('ArchivalAgreement');
foreach($rm_agreements_properties as $property) {
    $propertyName = $property->getName();
    $formElement = $ViewController->getElementById(
        $propertyName
    );

    if(!$formElement) continue;
    
    switch($propertyName) {
    case 'agreement_id':
        continue;
    case 'allowed_file_types':
        $selectedFileTypes = $ViewController->query('.//option/@value', $formElement);
        $allowedFileTypes = array();
        for($i=0; $i<$selectedFileTypes->length; $i++) {
            $allowedFileTypes[] = $selectedFileTypes->item($i)->nodeValue;
        }
        $rm_agreement->allowed_file_types = implode(' ', $allowedFileTypes);
        break;
    case 'begin_date':
    case 'end_date':
        $input_date = $formElement->getAttribute('value');
        $input_date = substr($input_date, 6, 4) . "-" . substr($input_date, 3, 2) . "-" . substr($input_date, 0, 2);
        $rm_agreement->$propertyName = $input_date;
        $log[] = $propertyName . " => " . $input_date;
        break;
    default:
        $log[] = $propertyName . " => " . utf8_decode($formElement->getAttribute('value'));
        $rm_agreement->$propertyName = utf8_decode($formElement->getAttribute('value'));
    }
    
}

//*****************************************************************************
// 3 - Validate / save and return
//*****************************************************************************
$return = new StdClass();
$return->status = 1;
$return->log = $log;
$return->errors = array();

//*************************************
// 3.1 - Validate
//*************************************
$valid = $dataObjectController->validate($rm_agreement);
if(!$valid) {
    $return->status = 0;
    $validationErrors = $rm_agreement->getValidationErrors();
    for($i=0; $i<count($validationErrors); $i++) {
        $validationError = $validationErrors[$i];
        $returnError = $validationError->message;
        switch($validationError->id){
        case '1824':
            $parts = explode("'", $validationError->message);
            $element = $parts[1];
            if(strstr($validationError->message, 'attribute')) {
                $attribute = $parts[3];
                $value = $parts[5];
                $type = $parts[7];
                $propertyDesc = $messageController->getMessageText($element . "." .$attribute);
                if($value == '') {
                    $returnError = "La valeur de '$propertyDesc' est obligatoire.";
                } else {
                    $returnError = "La valeur '$value' n'est pas autorisée pour '$propertyDesc'.";
                }
            } else {
                $value = $parts[3];
                $type = $parts[5];
                $propertyDesc = $messageController->getMessageText('rm_archives.' . $element);
                if($value == '') {
                    $returnError = "La valeur de '$propertyDesc' est obligatoire.";
                } else {
                    $returnError = "La valeur '$value' n'est pas autorisée pour '$propertyDesc'.";
                } 
            }
            
        }
        $return->errors[] = $returnError;
    }   
    
    echo json_encode($return); 
    return;
} // End if ntot valid

if($rm_agreement->begin_date >= $rm_agreement->end_date) {
    $return->status = 0;
    $return->errors[] = "La date de début de validité doit être antérieure à la date de fin.";
    echo json_encode($return); 
    return;
}

//*************************************
// 3.2 - Save
//*************************************
$key = $dataObjectController->save($rm_agreement);
if(!$key) {
    $return->status = 0;
    $return->errors[] = "Erreur lors de la sauvegarde des données";
    echo json_encode($return); 
    return;
}
$_SESSION['ArchivalAgreement'] = $rm_agreement->asXML();

//*************************************
// 3.4 - Return
//*************************************   
echo json_encode($return); 