<?php

/*
*   Copyright 2008-2012 Maarch
*
*   This file is part of Maarch Framework.
*
*   Maarch Framework is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Maarch Framework is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @brief Contains the entity controler page
*
*
* @file
* @author Arnaud Veber
* @author Laurent Giovannoni
* @date $date$
* @version $Revision$
* @ingroup entities
*/

//SPECIFIC PROCESS
//columns to show in the list
$showCols = array(
    'identifier'         => array(
        'functionFormat' => '',
        'cssStyle'       => '',
    ),
    'description'        => array(
        'functionFormat' => '',
        'cssStyle'       => '',
    ),
    'comment'            => array(
        'functionFormat' => '',
        'cssStyle'       => '',
    ),
    'coll_id'            => array(
        'functionFormat' => '',
        'cssStyle'       => '',
    ),
    'is_enabled'         => array(
        'functionFormat' => 'isBoolean',
        'cssStyle'       => 'text-align: center;',
    ),
);
if (isset($_REQUEST['orderField'])) {
    $showCols[$_REQUEST['orderField']]['cssStyle'] 
        .= 'background-image: url(static.php?filename=black_0.1.png); ';
}

//actions to show in the list
$actions = array(
    'create', 
    'read', 
    'update',
);

include_once 'apps/' . $_SESSION['config']['app_id'] . '/admin/' 
    . 'admin_standard_page_controller.php';

function populate($dataObjectController, $viewController, $ArchivalAgreement, $mode)
{
    $view = $viewController->view;
        
    /* ------------------
    - archival_agency_entity_id
    ------------------ */
    $archival_entities = 
        $dataObjectController->enumerate(
                'RefOrganization',
                'ArchivalAgency'
            );
    $i_max = count($archival_entities);
    $archival_entity_id = $viewController->getElementById('archival_entity_id');
    if ($i_max > 1)
        $archival_entity_id->addOption('', '');
    for ($i=0; $i<$i_max; $i++) {
        $archival_entity = $archival_entities[$i];
        $option = $archival_entity_id->addOption($archival_entity->entity_id, $archival_entity->Name);
    }
    
    /* ----------------------
    - transferring entity id
    ---------------------- */
    $transferring_entities = 
        $dataObjectController->enumerate(
            'RefOrganization',
            'TransferringAgency'
        );
    $i_max = count($transferring_entities);
    $transferring_entity_id = $viewController->getElementById('transferring_entity_id');
    if ($i_max > 1)
        $transferring_entity_id->addOption('', '');
    for ($i=0; $i<$i_max; $i++) {
        $transferring_entity = $transferring_entities[$i];
        $option = $transferring_entity_id->addOption($transferring_entity->entity_id, $transferring_entity->Name);
    }
       
    /* ------
    - coll id
    ------ */
    $coll_id = $viewController->getElementById('coll_id');
    $collections = $_SESSION['collections'];
    $i_max = count($collections);
    if ($i_max > 1) $coll_id->addOption('', '');
    for ($i=0; $i<$i_max; $i++) {
        $collection = $collections[$i];
        $option = $coll_id->addOption($collection['id'], $collection['id']);
    }
    
    /* -----------------
    - allowed file types (pronom)
    ----------------- */
    $sedaParser = new DataObjectController;
    $allowed_file_type_left = $viewController->getElementById('allowed_file_types_left');
    $allowed_file_type = $viewController->getElementById('allowed_file_types');
    $sedaParser->loadXSD('modules/records_management/schema/seda/v1.0/codes/IANA_MIMEMediaType_20110216.xsd');
    //$sedaParser->loadXSD('modules/records_management/schema/seda/V1.0/codes/seda_v1-0_filetype_code.xsd');
    $fileTypes = $sedaParser->getTypeEnumeration('MIMEMediaTypeContentType');
    $i_max = count($fileTypes);
    for($i=0; $i<$i_max; $i++) {
        $fileType = $fileTypes[$i];
        $fileTypeCode = $fileType->getAttribute('value');
        
        if (!empty($ArchivalAgreement->allowed_file_types)) {
            $types = explode(' ', $ArchivalAgreement->allowed_file_types);
            if (in_array($fileTypeCode, $types))
                $option = $allowed_file_type->addOption($fileTypeCode, $fileTypeCode);
            else
                $option = $allowed_file_type_left->addOption($fileTypeCode, $fileTypeCode);
        } else {
            $option = $allowed_file_type_left->addOption($fileTypeCode, $fileTypeCode);
        }
        $option->setAttribute('title', $fileTypeCode);
    }
    

    /* --------------------
    - transfer count period
    -------------------- */
    $transfer_count_period = $viewController->getElementById('transfer_count_period');
    $periodeTransferts = $dataObjectController->getTypeEnumeration('period');
    $i_max = count($periodeTransferts);
    if ($i_max > 1) $transfer_count_period->addOption('', '');
    for ($i=0; $i<$i_max; $i++) {
        $periodeTransfert = $periodeTransferts[$i];
        $option = 
            $transfer_count_period->addOption(
                $periodeTransfert->getAttribute('value'), 
                $periodeTransfert->getElementsBytagName('name')->item(0)->nodeValue
            );
    }
    
    /* --------------------
    - Archival Profile
    -------------------- */
    $ArchivalProfile__select = $viewController->getElementById('archival_profile');
    $ArchivalProfiles = $dataObjectController->getTypeEnumeration('ArchivalProfile');
    $i_max = count($ArchivalProfiles);
    if ($i_max > 1) $ArchivalProfile__select->addOption('', '');
    for ($i=0; $i<$i_max; $i++) {
        $ArchivalProfile = $ArchivalProfiles[$i];
        $value = $ArchivalProfile->getAttribute('value');
        $option = 
            $ArchivalProfile__select->addOption(
                $value, 
                $value . ': ' . $ArchivalProfile->getElementsBytagName('name')->item(0)->nodeValue
            );
    }
     
    /* -----------------------------------
    - disable specific case when mode=read
    ----------------------------------- */
    if ($mode == 'read') {
        $viewController->disableInputs();
        $filterSelectMultiple = $viewController->getElementById('filterSelectMultiple');
        $filterSelectMultiple->setValue('');
        $filterSelectMultiple->disable();
        
        $viewController->getElementById('allowFileTypes')->disable();
        $viewController->getElementById('allowAllFileTypes')->disable();
        $viewController->getElementById('banFileTypes')->disable();
        $viewController->getElementById('banAllFileTypes')->disable();
    }
}
/* ------------------------------------
- retrieve dataObject in $ArchivalAgreement
------------------------------------- */
$ArchivalAgreement = $dataObject;

/* ----------------
- title of the page
---------------- */
$titleImageSource = $_SESSION['config']['businessappurl'].'static.php?filename=favicon.png';

if ($params['mode'] == 'create' || $params['mode'] == 'read' || $params['mode'] == 'update') {
    $viewController->loadHTMLFile('modules/records_management/html/ArchivalAgreement.html');
    $viewController->translate($messageController);
    populate($dataObjectController, $viewController, $ArchivalAgreement, $params['mode']);
    $viewController->loadDataObject($ArchivalAgreement);   
    $_SESSION['ArchivalAgreement'] = $ArchivalAgreement->asXML();
}
    

switch($params['mode']) {
    case 'list' :
        $titleText = $messageController->getMessageText(
            'rm_agreements_list', 
            false, 
            array(
                count($objectList)
            )
        );
        $str_returnShow = $listContent;
        break;
        
    case 'create' :
        $titleText = $messageController->getMessageText('rm_agreements_create');
        break;
        
    case 'read' :
        $titleText = $messageController->getMessageText('rm_agreements_read');
        break;
        
    case 'update' :
        $titleText = $messageController->getMessageText('rm_agreements_update');
        break;
        
    case 'delete' :
        
        break;
}
?>
<h1>
    <img src="<?php echo $titleImageSource; ?>" />
    <?php echo $titleText; ?>
</h1>
<div class="<?php echo $params['objectName'] ?>">
    <div id="returnAjax"><br /><br /></div>
    <?php
    if ($params['mode'] == 'create' || $params['mode'] == 'read' || $params['mode'] == 'update') {
        /* --------
        - show view
        -------- */
        $viewController->showView();
    } else {
        if (isset($str_returnShow) && !empty($str_returnShow))
            echo $str_returnShow;
    }
    ?>
</div>
<?php if ($str_defaultJs) echo $str_defaultJs; ?>
