<?php

/*
 *  Copyright 2008-2012 Maarch
 *
 *  This file is part of Maarch Framework.
 *
 *  Maarch Framework is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Maarch Framework is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @brief API to import or update an archive in Maarch
 * @file
 * @author Cyril Vazquez  <dev@maarch.org>
 * @date $date$
 * @version $Revision$
 * @ingroup records_management
 */

/**
 * Process Archive Control and update values
 */
function processArchive()
{   
    
    $Archive = 
        $_SESSION['RecordsManagement']['ArchiveTransfer']
            ->Archive[$GLOBALS['currentArchive']];
    
    $ContentDescription = $Archive->ContentDescription[0];
    
    // Control & update Originating Agencies
    $OriginatingAgencyIdentifications = 
        $GLOBALS['message']['xpath']->query(
            './seda:ContentDescription/seda:OriginatingAgency/seda:Identification',
            $GLOBALS['archive']['element']
        );
    
    for($i=0; $i<$OriginatingAgencyIdentifications; $i++) {
        $OriginatingAgencyIdentification = 
            $OriginatingAgencyIdentifications->item($i)->nodeValue;
        $RefOriginatingAgency = 
            $GLOBALS['RefOrganizations']->query(
                '//RefOrganization['
                . 'Identification="'.$OriginatingAgencyIdentification.'"'
                . ' and @role="OriginatingAgency"'
                .']'
            )->item(0);
        if(!$RefOriginatingAgency) {
            $GLOBALS['ReplyCode'] = 200;
            return false;
        }
        $GLOBALS['logger']->write(
                'Archive Originating Agency Entity id is ' . $RefOriginatingAgency->entity_id, 
                'INFO'
            );
        $ContentDescription->OriginatingAgency[$i]->entity_id = 
            $RefOriginatingAgency->entity_id;
        $GLOBALS['RecordsManagementController']->save($ContentDescription);
    }    
    
    // Control Archival Agreement
    $ArchivalAgreement = 
        $GLOBALS['message']['xpath']->query(
            './seda:ArchivalAgreement',
            $GLOBALS['archive']['element']
        )->item(0)->nodeValue;
    
    $RefArchivalAgreement = 
        $GLOBALS['ArchivalAgreements']->query(
            '//ArchivalAgreement['
            . 'identifier="'.$ArchivalAgreement.'"'
            .']'
        )->item(0);
    if(!$RefArchivalAgreement) {
        $GLOBALS['ReplyCode'] = 300;
        return false;
    }
    $GLOBALS['logger']->write(
        'Archival Agreement found with id ' . $RefArchivalAgreement->agreement_id, 
        'INFO'
    );
    
    // Control Transferring Agency VS Agreement
    if($RefArchivalAgreement->transferring_entity_id != $GLOBALS['message']['TransferringAgencyEntityId']) 
    {
        $GLOBALS['ReplyCode'] = 303;
        return false;
    }
    
    // Control Archival Agency VS Agreement
    if($RefArchivalAgreement->archival_entity_id != $GLOBALS['message']['ArchivalAgencyEntityId']) 
    {
        $GLOBALS['ReplyCode'] = 304;
        return false;
    }
    
    
    
    return true;
}


