<?php

/*
 *  Copyright 2008-2013 Maarch
 *
 *  This file is part of Maarch Framework.
 *
 *  Maarch Framework is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Maarch Framework is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @brief API to import an item in Maarch
 * @file
 * @author Laurent Giovannoni  <dev@maarch.org>
 * @date $date$
 * @version $Revision$
 * @ingroup records_management
 */

/**
 * Prepare Item for import
 * @return array $returnArray array of information to process the ios
 */
function processDocument()
{
    $Attachment =
        $GLOBALS['message']['xpath']->query(
            './seda:Attachment',
            $GLOBALS['document']['element']
        )->item(0);
    $GLOBALS['document']['binary'] = $Attachment->nodeValue;
    $GLOBALS['document']['filename'] = $Attachment->getAttribute('filename');
    $GLOBALS['document']['fileExt'] = $GLOBALS['func']->extractFileExt($GLOBALS['document']['filename']);
    $GLOBALS['document']['uri'] = $Attachment->getAttribute('uri');
    
    if (mb_strlen($GLOBALS['document']['binary']) > 1) {
        $GLOBALS['logger']->write('Digital resource encoded in message!', 'INFO');
        $fileExtraction = array();
        $fileExtraction = extractFile();
        if ($fileExtraction['status'] == 'KO') {
            $GLOBALS['logger']->write($fileExtraction['error'], 'ERROR', 150);
            return false;
        } else {
            $GLOBALS['logger']->write('Digital resource extracted', 'INFO');
        }
        $stored = storeItemInDs();
        $updated = updateDocument();  
        if ($updated && $stored) return true;        
    } elseif ($GLOBALS['document']['filename'] && $GLOBALS['document']['uri']) {
        $GLOBALS['logger']->write('Digital resource transferred as attachment!', 'INFO');
        $fileCopy = array();
        $fileCopy = copyFile();
        if ($fileCopy['status'] == 'KO') {
            $GLOBALS['logger']->write($fileCopy['error'], 'ERROR', 150);
            return false;
        } else {
            $GLOBALS['logger']->write('Digital resource extracted', 'INFO');
        }
        $stored = storeItemInDs();
        $updated = updateDocument();
        if ($updated && $stored) return true;
    } else {
        $GLOBALS['logger']->write('Physical resource!', 'INFO');
        
        $GLOBALS['document']['filename'] = 'empty';
        $GLOBALS['document']['fileExt'] = 'pdf';
        $GLOBALS['document']['uri'] = 'empty.pdf';
        
        $fileCopy = array();
        $fileCopy = copyFile();
        if ($fileCopy['status'] == 'KO') {
            $GLOBALS['logger']->write($fileCopy['error'], 'ERROR', 150);
            return false;
        } else {
            $GLOBALS['logger']->write('Digital empty resource created for the physical resource', 'INFO');
        }
        $stored = storeItemInDs();
        $updated = updateDocument();
        if ($updated && $stored) return true;
        return true;
    }
} 

/**
 * Extract file of the electronic current item and put it on the tmp dir
 * @return array $returnArray array of information to process the file
 */
function extractFile()
{
    //retrieve the encoded file in the current contains and copy it on tmp
    $fileContent = base64_decode(
        $GLOBALS['document']['binary']
    );
    $fileName = 
        $GLOBALS['document']['filename'] 
        . rand() 
        . '.' . $GLOBALS['document']['fileExt'];
    $filePath = $GLOBALS['message']['dirPathOnTmp'] . $fileName;
    $GLOBALS['document']['FilePathOnTmp'] = $filePath;
    $GLOBALS['document']['fileNameOnTmp'] = $fileName;
    $inF = fopen($filePath, "w");
    if (!fwrite($inF, $fileContent)) {
        $returnArray = array(
            'status'                => 'KO',
            'pathToTheFile'         => '',
            'nameOfTheFile'         => '',
            'extenstionOfTheFile'   => '',
            'error'                 => 'cannot write file content on the tmp'
        );
        return $returnArray;
    }
    fclose($inF);
    $returnArray = array(
        'status'                => 'OK',
        'pathToTheFile'         => $filePath,
        'nameOfTheFile'         => $fileName,
        'extenstionOfTheFile'   => $GLOBALS['document']['fileExt'],
        'error'                 => ''
    );
    return $returnArray;
}

function copyFile()
{
    $fileName = $GLOBALS['document']['filename'] . rand() . '.' . $GLOBALS['document']['fileext'];
    $filePath = $GLOBALS['message']['dirPathOnTmp'] . $fileName;
    $GLOBALS['document']['FilePathOnTmp'] = $filePath;
    $GLOBALS['document']['fileNameOnTmp'] = $fileName;
    
    if (!$inF = copy($GLOBALS['document']['uri'], $filePath)) {
        $returnArray = array(
            'status'                => 'KO',
            'pathToTheFile'         => '',
            'nameOfTheFile'         => '',
            'extenstionOfTheFile'   => '',
            'error'                 => 'cannot write file content on the tmp'
        );
        return $returnArray;
    }
    $returnArray = array(
        'status'                => 'OK',
        'pathToTheFile'         => $filePath,
        'nameOfTheFile'         => $fileName,
        'extenstionOfTheFile'   => $GLOBALS['document']['fileExt'],
        'error'                 => ''
    );
    return $returnArray;

}

/**
 * Store Item in the Maarch docserver
 * @return array $returnArray array of information to process the ios
 */
function storeItemInDs()
{
    //first store resource on the docserver
    $fileInfos = array(
        'tmpDir'      => $GLOBALS['message']['dirPathOnTmp'],
        'size'        => filesize(
                $GLOBALS['document']['FilePathOnTmp']
            ),
        'format'      => strtoupper(
                $GLOBALS['document']['fileExt']
            ),
        'tmpFileName' => $GLOBALS['document']['fileNameOnTmp'],
    );
    
    // Check size of the docserver 
    // (stop the program if the docserver will be 
    // full at 95 percent)
    $reasonableLimitSize = $GLOBALS['docserver']['size_limit_number'] * 0.95;
    $targetSize = $fileInfos['size'] + $GLOBALS['docserver']['actual_size_number'];
    if ($targetSize > $reasonableLimitSize) {
        Bt_putFileInError(
            $GLOBALS['incomingDirectory'] 
                . $GLOBALS['messages'][$GLOBALS['currentMessage']], 
            $GLOBALS['messages'][$GLOBALS['currentMessage']], 
            $GLOBALS['batchDirectory'] . DIRECTORY_SEPARATOR . 'error', 
            $GLOBALS['wb']
        );
        Bt_exitBatch(
            28, 'The docserver will be full at 95 percent:'
            . $targetSize . " > " . $reasonableLimitSize
        );
    }
    
    $docserverControler = new docservers_controler();
    $infoFileNameInTargetDocserver = array();
    $infoFileNameInTargetDocserver = 
        $docserverControler->getNextFileNameInDocserver(
            $GLOBALS['docserver']['pathOnDocserver']
        );
    if ($infoFileNameInTargetDocserver['error'] <> "") {
        Bt_putFileInError(
            $GLOBALS['incomingDirectory'] 
                . $GLOBALS['messages'][$GLOBALS['currentMessage']], 
            $GLOBALS['messages'][$GLOBALS['currentMessage']], 
            $GLOBALS['batchDirectory'] . DIRECTORY_SEPARATOR . 'error', 
            $GLOBALS['wb']
        );
        Bt_exitBatch(
            21, $infoFileNameInTargetDocserver['error']
        );
    }
    
    $copyResultArray = array();
    $infoFileNameInTargetDocserver['fileDestinationName'] .= "." 
        . strtolower($fileInfos['format']);
    $copyResultArray = Ds_copyOnDocserver(
        $GLOBALS['document']['FilePathOnTmp'], 
        $infoFileNameInTargetDocserver, 
        'MD5'
    );
    
    $docserverPathTemplate = 
        str_replace(
            DIRECTORY_SEPARATOR, 
            "#", 
            $GLOBALS['docserver']['path_template']
        );

    $docserverPathTemplate = str_replace("##", "#", $docserverPathTemplate);
    $copyResultArray['destinationDir'] = str_replace($docserverPathTemplate, '', $copyResultArray['destinationDir']);

    $GLOBALS['document']['storage_infos']['DestinationDir']
        = $copyResultArray['destinationDir'];
    $GLOBALS['document']['storage_infos']['FileDestinationName']
        = $copyResultArray['fileDestinationName'];
    $GLOBALS['document']['storage_infos']['path_template']
        = $GLOBALS['docserver']['path_template'];
    $GLOBALS['document']['storage_infos']['FileSize']
        = $copyResultArray['fileSize'];
    $GLOBALS['document']['storage_infos']['Fingerprint'] = Ds_doFingerprint(
        str_replace("#", DIRECTORY_SEPARATOR, $GLOBALS['document']['storage_infos']['path_template'])
        . str_replace("#", DIRECTORY_SEPARATOR, $GLOBALS['document']['storage_infos']['DestinationDir'])
        . $GLOBALS['document']['storage_infos']['FileDestinationName'], 
        $GLOBALS['docserver']['fingerprint_mode']
    );
    $GLOBALS['docserver']['actual_size_number'] 
        = $GLOBALS['docserver']['actual_size_number'] + $copyResultArray['fileSize'];
    $query = "update " . _DOCSERVERS_TABLE_NAME . " set actual_size_number=" 
           . $GLOBALS['docserver']['actual_size_number'] 
           . " where docserver_id='" . $GLOBALS['docserver']['docserver_id'] . "'";
    Bt_doQuery($GLOBALS['db'], $query);
    
    if ($copyResultArray['fileDestinationName']) {
        return true;
    }
}

/**
 * Store Item in the Maarch database
 * @return array $returnArray array of information to process the ios
 */
function updateDocument()
{
    $parent = $GLOBALS['document']['element']->parentNode;
    if ($parent->tagName == 'Archive') {
        $query = '/ArchiveTransfer'
            . '/Archive['.($GLOBALS['currentArchive']+1).']'
            . '/Document['.($GLOBALS['currentDocument']+1).']';
        
    } elseif ($parent->tagName == 'ArchiveObject') {
        $query = '/ArchiveTransfer'
            . '/Archive['.($GLOBALS['currentArchive']+1).']'
            . '/ArchiveObject['.($GLOBALS['currentArchiveObject']+1).']'
            . '/Document['.($GLOBALS['currentDocument']+1).']';
    }
    $GLOBALS['logger']->write(
         "Searching document in io with path $query", 
        'INFO'
    ); 
    if (
        mb_strlen($GLOBALS['document']['binary']) > 1
        || ($GLOBALS['document']['filename'] && $GLOBALS['document']['uri'])
    ) {
        $IODocument = 
            $_SESSION['RecordsManagement']['ArchiveTransfer']->query(
                $query
            )->item(0);
        
        if (!$IODocument) {
            $GLOBALS['logger']->write(
                'Could not retrieve Document in IO', 
                'ERROR'
            ); 
            return false;
        }
    }
    
    $IODocument->docserver_id = $GLOBALS['docserver']['docserver_id'];
    $IODocument->Status = "BRSAS";
    $IODocument->format = $GLOBALS['document']['fileExt'];
    $IODocument->Size = $GLOBALS['document']['storage_infos']['FileSize'];
    $IODocument->path = $GLOBALS['document']['storage_infos']['DestinationDir'];
    $IODocument->filename = $GLOBALS['document']['storage_infos']['FileDestinationName'];
    $IODocument->Integrity = $GLOBALS['document']['storage_infos']['Fingerprint'];
    $IODocument->coll_id = $GLOBALS['collection'];
    
    //life cycle workflow initialization
    if (isset($GLOBALS['policyId']) && $GLOBALS['policyId'] <> '') {
        $IODocument->policy_id = $GLOBALS['policyId'];
    }
    if (isset($GLOBALS['initCycleId']) && $GLOBALS['initCycleId'] <> '') {
        $IODocument->cycle_id = $GLOBALS['initCycleId'];
    }
    
    $key = 
        $GLOBALS['RecordsManagementController']->save(
            $IODocument
        );
    if (!$key) {
        $GLOBALS['logger']->write(
            'Unable to save Document', 
            'ERROR'
        ); 
        return false;
    }
    
    $GLOBALS['history']->add(
        "rm_documents",
        $key->document_id,
        "ADD",
        "rmdocumentadd",
        'Import du document ' . $key->document_id
            . ' du bordereau ' . $_SESSION['RecordsManagement']['ArchiveTransfer']->TransferIdentifier,
        $_SESSION['config']['databasetype'],
        'records_management'
    );
    $GLOBALS['logger']->write(
        'Document saved with document_id ' . $key->document_id, 
        'INFO'
    ); 
    return true;
}

