<?php

/*
 *  Copyright 2008-2012 Maarch
 *
 *  This file is part of Maarch Framework.
 *
 *  Maarch Framework is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Maarch Framework is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @brief API to import or update an IOS in Maarch
 * @file
 * @author Laurent Giovannoni  <dev@maarch.org>
 * @date $date$
 * @version $Revision$
 * @ingroup records_management
 */

/**
 * Prepare IOS for import or update
 * @return array $returnArray array of information to process the ios
 */
function processIo()
{   
    // Message type
    $GLOBALS['message']['type'] = $GLOBALS['message']['document']->documentElement->tagName;
    if(!$GLOBALS['message']['type']) {
        $GLOBALS['logger']->write(
            'Unable to retrieve message type!', 
            'ERROR'
        );
        $GLOBALS['ReplyCode'] = 101;
        return false;
    }
    $GLOBALS['logger']->write(
        'Message type : ' . $GLOBALS['message']['type'] , 
        'INFO'
    );  
    
    // Message type specific controls
    switch($GLOBALS['message']['type']) 
    {
    case 'ArchiveTransfer':
        return processArchiveTransfer();
        break;
        
    case 'ArchiveTransferReply':
        return processArchiveTransferReply();
        break;
        
    }
}

function processArchiveTransfer()
{
    $GLOBALS['message']['document']->save(
        $GLOBALS['message']['dirPathOnTmp'] 
                . DIRECTORY_SEPARATOR . $GLOBALS['message']['filename'] . '.xml'
    );
    libxml_use_internal_errors(true);
    $valid = $GLOBALS['message']['document']->schemaValidate(
        $_SESSION['config']['corepath'] 
        . 'modules/records_management/schema/seda/v1.0/seda_v1-0_archivetransfer.xsd'
    );
    if($valid == false) {
        $GLOBALS['logger']->write(
                'Message is not valid againt schema!', 
                'ERROR'
            ); 
        $errors = libxml_get_errors();
        foreach ($errors as $error) {
            $return = '';
            switch ($error->level) {
                case LIBXML_ERR_WARNING:
                    $return .= "Warning: ";
                    break;
                case LIBXML_ERR_ERROR:
                    $return .= "Error: ";
                    break;
                case LIBXML_ERR_FATAL:
                    $return .= "Fatal Error: ";
                    break;
            }
            $return .= $error->code . ' - ' . trim($error->message);
            $GLOBALS['logger']->write(
                $return, 
                'ERROR'
            ); 
        }
        libxml_clear_errors();
            
        
        $GLOBALS['ReplyCode'] = 101;  
        libxml_use_internal_errors(false);        
        return false;
    }
    libxml_use_internal_errors(false);
    
    if($ArchiveTransfer = 
        $GLOBALS['RecordsManagementController']->searchArchiveTransfer(
            $GLOBALS['message']['document']
        )
    ) {
        /**************************************************************************
        **  LOCAL IO : IO Found
        **************************************************************************/
        $GLOBALS['logger']->write(
            'ArchiveTransfer found in Maarch with io_id ' . $ArchiveTransfer->io_id, 
            'INFO'
        );      
        
        if($ArchiveTransfer->io_status != 'SENT') {
            $GLOBALS['logger']->write(
                'ArchiveTransfer has already been imported!', 
                'ERROR'
            );  
            return false;
        }
        
        $GLOBALS['message']['ArchivalAgencyEntityId'] = $ArchiveTransfer->ArchivalAgency[0]->entity_id;
        $GLOBALS['logger']->write(
                'Archival Entity id is ' . $GLOBALS['message']['ArchivalAgencyEntityId'], 
                'INFO'
            );
        $GLOBALS['message']['TransferringAgencyEntityId'] = $ArchiveTransfer->TransferringAgency[0]->entity_id;
        $GLOBALS['logger']->write(
                'Transferring Entity id is ' . $GLOBALS['message']['TransferringAgencyEntityId'], 
                'INFO'
            );
        
        $_SESSION['RecordsManagement']['ArchiveTransfer'] = $ArchiveTransfer; 
        $ArchiveTransfer->io_status = "BRSAS"; 
        $key = 
            $GLOBALS['RecordsManagementController']->save(
                $ArchiveTransfer,
                $saveChildren = false
            );
        
        if (!$key) {
            $GLOBALS['logger']->write(
                'Unable to update IO', 
                'ERROR'
            );
            $GLOBALS['ReplyCode'] = 102;
            return false;
        }
 
    } 
    else {
        /**************************************************************************
        **  INCOMING IO : IO not Found, save
        **************************************************************************/
        $GLOBALS['logger']->write(
            'ArchiveTransfer not found in Maarch, create rm_io', 
            'INFO'
        );
        
        $ArchiveTransfer =
            $GLOBALS['RecordsManagementController']->importMessage(
                $GLOBALS['message']['document']
            );
            
        $key = 
            $GLOBALS['RecordsManagementController']->saveMessage(
                $ArchiveTransfer
            );
        
        if (!$key) {
            $GLOBALS['logger']->write(
                'Unable to save IO', 
                'ERROR'
            );
            $GLOBALS['ReplyCode'] = 102;
            return false;
        }     
        
        $_SESSION['RecordsManagement']['ArchiveTransfer'] = $ArchiveTransfer;
        
        // Store to docserver
        $GLOBALS['logger']->write(
            'Store XML in docserver', 
            'INFO'
        );
        $fileInfos = array(
            'tmpPath' => $GLOBALS['message']['dirPathOnTmp'],
            'tmpDir' => $GLOBALS['message']['dirPathOnTmp'],
            'tmpFileName' => $GLOBALS['message']['filename'],
            'size' => $GLOBALS['message']['dirPathOnTmp'] 
                . DIRECTORY_SEPARATOR . $GLOBALS['message']['filename']
        );
        
        // CV: avoid creation of subdir with batch number
        $wb_sav = $GLOBALS['wb'];
        unset($GLOBALS['wb']);
        $storeInfos = 
            $GLOBALS['docserverControler']->storeResourceOnDocserver(
                'io_coll',
                $fileInfos
            );
        $GLOBALS['wb'] =  $wb_sav;
        
        $ArchiveTransfer->io_path = str_replace('\\', '#', $storeInfos['destination_dir']);
        $ArchiveTransfer->io_filename = $storeInfos['file_destination_name'];
        
        // Control & add local identifiers
        //****************************************************************************************
        // Archival Agency
        $ArchivalAgencyIdentification = 
            $GLOBALS['message']['xpath']->query(
                '//seda:ArchivalAgency/seda:Identification'
            )->item(0)->nodeValue;
        
        $RefArchivalAgency = 
            $GLOBALS['RefOrganizations']->query(
                '//RefOrganization['
                . 'Identification="'.$ArchivalAgencyIdentification.'"'
                . ' and @role="ArchivalAgency"'
                .']'
            )->item(0);
        if(!$RefArchivalAgency) {
            $GLOBALS['logger']->write(
                'Unknown Archival Agency', 
                'ERROR'
            );
            $GLOBALS['ReplyCode'] = 201;
            return false;
        }
        $GLOBALS['logger']->write(
                'Archival Entity id is ' . $RefArchivalAgency->entity_id, 
                'INFO'
            );
        
        $_SESSION['RecordsManagement']['ArchiveTransfer']
            ->ArchivalAgency[0]->entity_id = $RefArchivalAgency->entity_id;
        $GLOBALS['message']['ArchivalAgencyEntityId'] = $RefArchivalAgency->entity_id;
        
        // Transferring Agency
        $TransferringAgencyIdentification = 
            $GLOBALS['message']['xpath']->query(
                '//seda:TransferringAgency/seda:Identification'
            )->item(0)->nodeValue;
        
        $RefTransferringAgency = $GLOBALS['RefOrganizations']->query(
                '//RefOrganization['
                . 'Identification="'.$TransferringAgencyIdentification.'"'
                . ' and @role="TransferringAgency"'
                .']'
            )->item(0);
        if(!$RefTransferringAgency) {
            $GLOBALS['logger']->write(
                'Unknown Transferring Agency', 
                'ERROR'
            );
            $GLOBALS['ReplyCode'] = 202;
            return false;
        }
        $GLOBALS['logger']->write(
                'Transferring Entity id is ' . $RefTransferringAgency->entity_id, 
                'INFO'
            );
        
        $_SESSION['RecordsManagement']['ArchiveTransfer']
            ->TransferringAgency[0]->entity_id = $RefTransferringAgency->entity_id;
        $GLOBALS['message']['TransferringAgencyEntityId'] = $RefTransferringAgency->entity_id;
        
        
        $key = 
            $GLOBALS['RecordsManagementController']->saveMessage(
                $ArchiveTransfer
            );
        
        if (!$key) {
            $GLOBALS['logger']->write(
                'Unable to save IO', 
                'ERROR'
            );
            $GLOBALS['ReplyCode'] = 102;
            return false;
        } 
    }
    
    /**************************************************************************
    ** Controls
    **************************************************************************/
    // Schema
    libxml_use_internal_errors(true);
    if (!$GLOBALS['message']['document']->schemaValidate(
            $GLOBALS['MaarchDirectory'] . 'modules/records_management/schema/'
                . 'seda/v1.0/seda_v1-0_archivetransfer.xsd'
        )
    ) {
        libxml_display_errors();
        $GLOBALS['ReplyCode'] = 101;
        return false;
    }         
            
    return true;
}

function processArchiveTransferReply()
{
    require_once 'core/class/class_history.php';
    require_once 'core/core_tables.php';
    $history = new history();
    $MessageController = new MessageController();
    var_dump($_SESSION['config']);
    $MessageController->loadMessageFile(
        'modules/records_management/lang/ArchiveTransfer.xml'
    );
    $MessageController->loadMessageFile(
        'modules/records_management/lang/ArchiveTransferReply.xml'
    );
    
    /**************************************************************************
    **  INCOMING IO : Store XML and create IO
    **************************************************************************/
    $ArchiveTransferReply =
        $GLOBALS['RecordsManagementController']->importMessage(
            $GLOBALS['message']['document']
        );
    
    $_SESSION['RecordsManagement']['ArchiveTransferReply'] = $ArchiveTransferReply;
    
    // Store to docserver
    $GLOBALS['logger']->write(
        'Store XML in docserver', 
        'INFO'
    );
    $fileInfos = array(
        'tmpPath' => $GLOBALS['message']['dirPathOnTmp'],
        'tmpDir' => $GLOBALS['message']['dirPathOnTmp'],
        'tmpFileName' => $GLOBALS['message']['filename'],
        'size' => $GLOBALS['message']['dirPathOnTmp'] 
            . DIRECTORY_SEPARATOR . $GLOBALS['message']['filename']
    );
    
    // CV: avoid creation of subdir with batch number
    $wb_sav = $GLOBALS['wb'];
    unset($GLOBALS['wb']);
    $storeInfos = 
        $GLOBALS['docserverControler']->storeResourceOnDocserver(
            'io_coll',
            $fileInfos
        );
    $GLOBALS['wb'] =  $wb_sav;
    
    $ArchiveTransferReply->io_path = str_replace('\\', '#', $storeInfos['destination_dir']);
    $ArchiveTransferReply->io_filename = $storeInfos['file_destination_name'];
    
    // Control & add local identifiers
    //****************************************************************************************
    // Archival Agency
    $ArchivalAgencyIdentification = 
        $GLOBALS['message']['xpath']->query(
            '//seda:ArchivalAgency/seda:Identification'
        )->item(0)->nodeValue;
    
    $RefArchivalAgency = 
        $GLOBALS['RefOrganizations']->query(
            '//RefOrganization['
            . 'Identification="'.$ArchivalAgencyIdentification.'"'
            . ' and @role="ArchivalAgency"'
            .']'
        )->item(0);
    if(!$RefArchivalAgency) {
        $GLOBALS['logger']->write(
            'Unkown Archival Agency', 
            'ERROR'
        ); 
        return false;
    }
    $GLOBALS['logger']->write(
            'Archival Agency is ' . $RefArchivalAgency->entity_id, 
            'INFO'
        ); 
    
    $_SESSION['RecordsManagement']['ArchiveTransferReply']
        ->ArchivalAgency[0]->entity_id = $RefArchivalAgency->entity_id;
    $GLOBALS['message']['ArchivalAgencyEntityId'] = $RefArchivalAgency->entity_id;
    
    // Transferring Agency
    $TransferringAgencyIdentification = 
        $GLOBALS['message']['xpath']->query(
            '//seda:TransferringAgency/seda:Identification'
        )->item(0)->nodeValue;
    
    $RefTransferringAgency = $GLOBALS['RefOrganizations']->query(
            '//RefOrganization['
            . 'Identification="'.$TransferringAgencyIdentification.'"'
            . ' and @role="TransferringAgency"'
            .']'
        )->item(0);
    if(!$RefTransferringAgency) {
        $GLOBALS['logger']->write(
            'Unkown Transferring Agency', 
            'ERROR'
        ); 
        return false;
    }
    $GLOBALS['logger']->write(
            'Transferring Agency is ' . $RefTransferringAgency->entity_id, 
            'INFO'
        ); 
    $_SESSION['RecordsManagement']['ArchiveTransferReply']
        ->TransferringAgency[0]->entity_id = $RefTransferringAgency->entity_id;
    $GLOBALS['message']['TransferringAgencyEntityId'] = $RefTransferringAgency->entity_id;
    
    $key = 
        $GLOBALS['RecordsManagementController']->saveMessage(
            $ArchiveTransferReply
        );
    
    if (!$key) {
        $GLOBALS['logger']->write(
            'Unable to save IO', 
            'ERROR'
        );
        return false;
    } 
    
    
    /**************************************************************************
    **  Process ReplyCode
    **************************************************************************/
    if($ArchiveTransfer = 
        $GLOBALS['RecordsManagementController']->searchArchiveTransfer(
            $GLOBALS['message']['document']
        )
    ) {
        /**************************************************************************
        **  LOCAL IO : IO Found
        **************************************************************************/
        $GLOBALS['logger']->write(
            'ArchiveTransfer found in Maarch with io_id ' . $ArchiveTransfer->io_id, 
            'INFO'
        );      
        
        if($ArchiveTransfer->io_status != 'BRSAS') {
            $GLOBALS['logger']->write(
                'ArchiveTransfer has already been accepted or rejeted!', 
                'ERROR'
            );  
            return false;
        }
        
        $ReplyCode = 
            $GLOBALS['message']['xpath']->query(
                '//seda:ReplyCode'
            )->item(0)->nodeValue;
        if(!$ReplyCode) {
            $GLOBALS['logger']->write(
                'Unable to retrieve Reply Code', 
                'ERROR'
            );  
            return false;
        }
        
        if(substr($ReplyCode, 0, 1) === '0') {
            $ArchiveTransfer->io_status = "BROK";
            $event_type = 'ArchiveTransferAccept';
        } else {
            $ArchiveTransfer->io_status = "BRKO";
            $event_type = 'ArchiveTransferReject';
        }
       
        /* -------------------------------------------------------------------------
        - save IO ArchiveTransfer
        ------------------------------------------------------------------------- */
        $key = $GLOBALS['RecordsManagementController']->save($ArchiveTransfer);
        if(!$key) {
            $GLOBALS['logger']->write(
                'Unable to save IO', 
                'ERROR'
            );
            return false;
        }
        
        
        $info = 
            $MessageController->getMessageText(
                $event_type,
                false,
                array($key->io_id)
            );

        $history->add(
            'rm_ios',
            $key->io_id,
            $history_action,
            $event_type,
            $info,
            $_SESSION['config']['databasetype'],
            'records_management'
        ); 
        
    } else {
        $GLOBALS['logger']->write(
            'Unable to retrieve ArchiveTransfer', 
            'ERROR'
        );
        $GLOBALS['ReplyCode'] = 213;
        return false;
    }
    
    $info = 
        $MessageController->getMessageText(
            'ArchiveTransferReplyCreate',
            false,
            array($key->io_id, $ArchiveTransfer->io_id)
        );

    $history->add(
        'rm_ios',
        $key->io_id,
        'CREATE',
        'ArchiveTransferReplyCreate',
        $info,
        $_SESSION['config']['databasetype'],
        'records_management'
    ); 
    
    return true;
}

function createArchiveTransferReply() {
    /* -------------------------------------------------------------------------
    - reply IO
    ------------------------------------------------------------------------- */
    $ArchiveTransferReply = 
        $GLOBALS['RecordsManagementController']->createArchiveTransferReply(
            $_SESSION['RecordsManagement']['ArchiveTransfer']
        );
    
    $ArchiveTransferReply->io_status = 'INIT';
    $ArchiveTransferReply->ReplyCode = $GLOBALS['ReplyCode'];
    
    $ReplyComment = 
        $GLOBALS['RecordsManagementController']->query(
            '//xsd:simpleType[@name="ReplyCode"]'
            . '//xsd:enumeration[@value="'.$GLOBALS['ReplyCode'].'"]'
            . '//das:name'  
        )->item(0);
    if($ReplyComment) {
        $Comment = 
            $GLOBALS['RecordsManagementController']->addComment(
                $ArchiveTransferReply,
                $ReplyComment->nodeValue
            );
        $GLOBALS['logger']->write(
            $ReplyComment->nodeValue, 
            'ERROR'
        );
    } else {
        $GLOBALS['logger']->write(
            'Reject IO with ReplyCode ' .  $GLOBALS['ReplyCode'],
            'ERROR'
        );
    }
    
    $key = $GLOBALS['RecordsManagementController']->save($ArchiveTransferReply);    
    if(!$key) {
        $GLOBALS['logger']->write(
                'Unable to save reply IO', 
                'ERROR'
            );
        return false;
    }
       
    //*************************************
    // 2 - Generate SEDA Message
    //************************************* 
    $XSL = new DOMDocument();
    $XSL->load(
        $_SESSION['config']['corepath'] 
        . 'modules/records_management/schema/ArchiveTransferReply__export.xsl'
    );
    $XSLT = new XSLTProcessor();
    $XSLT->importStylesheet($XSL);
    $SEDA = $XSLT->transformToDoc($ArchiveTransferReply);

    $SEDAFile = 'ArchiveTransferReply_'.$ArchiveTransferReply->io_id.'.xml';
    $SEDAPath = $GLOBALS['outgoingDirectory'] . $SEDAFile;
    $SEDA->save($SEDAPath);
        
    //*************************************
    // 3 - Validate SEDA Message
    //*************************************    
    libxml_clear_errors();
    libxml_use_internal_errors(true);
    $valid = 
        $SEDA->schemaValidate(
            $_SESSION['config']['corepath'] 
            . 'modules/records_management/schema/'
            . 'seda/v1.0/seda_v1-0_archivetransfer.xsd'
        );
    if(!$valid) {
        $libxml_errors = libxml_get_errors();
        foreach ($libxml_errors as $libxml_error) {
            $GLOBALS['logger']->write(
                'Reply IO error:' .  $libxml_error->code . ": " . trim($libxml_error->message), 
                'ERROR'
            );
        }
        libxml_clear_errors();
        libxml_use_internal_errors(false);
        return false;
    }
    libxml_use_internal_errors(false);

    //*************************************
    // 4 - Archive SEDA Message
    //*************************************       
    // Get file infos
    $fileInfos = array(
        'tmpDir' => $GLOBALS['outgoingDirectory'],
        'tmpFileName' => $SEDAFile,
        'size' => filesize($SEDAPath)
    );

    // Save to docserver
    require_once 'core/class/docservers_controler.php';
    $docserverController = new docservers_controler();
    
    $wb_sav = $GLOBALS['wb'];
    unset($GLOBALS['wb']);
    $storeInfos = 
        $GLOBALS['docserverControler']->storeResourceOnDocserver(
            'io_coll',
            $fileInfos
        );
    $GLOBALS['wb'] =  $wb_sav;
        
    //*************************************
    // 5 - Save path & filename
    //*************************************
    $ArchiveTransferReply->io_path = str_replace('\\', '#', $storeInfos['destination_dir']);
    $ArchiveTransferReply->io_filename = $storeInfos['file_destination_name'];
    $ArchiveTransferReply->io_status = 'SENT';

    $key = false;
    $key = $GLOBALS['RecordsManagementController']->save($ArchiveTransferReply, $saveChildren=false);
    if(!$key) {
        $GLOBALS['logger']->write(
                'Unable to save reply IO', 
                'ERROR'
            );
        return false;
    }
    return true;
}
