<?php

/*
 *  Copyright 2008-2012 Maarch
 *
 *  This file is part of Maarch Framework.
 *
 *  Maarch Framework is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Maarch Framework is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @brief API to interface with SEDA MESSAGES
 * @file
 * @author Cyril VAZQUEZ  <dev@maarch.org>
 * @date $date$
 * @version $Revision$
 * @ingroup records_management
 */

function processMessage($messageFilename)
{    
    //echo $incomingDirectory . "\r\n";
    //echo $GLOBALS['message']['filename'] . "\r\n";
    //echo $tmpDirectory . "\r\n";
    $GLOBALS['message']['fileext'] = 
        $GLOBALS['func']->extractFileExt($GLOBALS['message']['filename']);
    
    $iosDirNameOnTmp = rand() . DIRECTORY_SEPARATOR;
    $GLOBALS['message']['dirPathOnTmp'] = 
        $GLOBALS['tmpDirectory'] . DIRECTORY_SEPARATOR . $iosDirNameOnTmp;
    $GLOBALS['logger']->write(
        'Creation of the tmp subdir:' . $GLOBALS['message']['dirPathOnTmp'], 
        'DEBUG'
    );
    if (!mkdir($GLOBALS['message']['dirPathOnTmp'], 0777)) {
        Bt_exitBatch(
            32, 'Pb with creation of a tmp dir in the tmp dir:' . $GLOBALS['message']['dirPathOnTmp']
        );
    }
    $GLOBALS['logger']->write(
        'Copy the message to the tmp subdir', 
        'DEBUG'
    );
    $cp = @copy(
        $GLOBALS['incomingDirectory'] . DIRECTORY_SEPARATOR . $GLOBALS['message']['filename'],
        $GLOBALS['message']['dirPathOnTmp'] . DIRECTORY_SEPARATOR . $GLOBALS['message']['filename']
    );
    if (!$cp) {
        Bt_exitBatch(
            33, 
            'Pb with copy in the tmp dir source/destination:' 
                . $GLOBALS['incomingDirectory'] . DIRECTORY_SEPARATOR . $GLOBALS['message']['filename']
                . ' / ' . $GLOBALS['message']['dirPathOnTmp'] . DIRECTORY_SEPARATOR . $GLOBALS['message']['filename']
        );
    }
    if (strtoupper($GLOBALS['message']['fileext']) == '7Z') {
        $GLOBALS['logger']->write(
            'It is a 7z archive, so extract it !', 
            'DEBUG'
        );
        extractMessage(
            $GLOBALS['message']['filename'], 
            $GLOBALS['message']['dirPathOnTmp']
        );
    }
    //retrieve the ios in the new tmp dir
    $MessageContents = array();
    $MessageContents = 
        $GLOBALS['func']->getFilesInFolder($GLOBALS['message']['dirPathOnTmp']);
    
    if (count($MessageContents) == 0) {
        Bt_exitBatch(
            34, 'No message found in the tmp dir:' . $GLOBALS['message']['dirPathOnTmp']
        );
    }
    
    if(!controlMessage($MessageContents[0])) {
        $GLOBALS['logger']->write(
            'Message : ' . $GLOBALS['message']['dirPathOnTmp'] . DIRECTORY_SEPARATOR
            . $GLOBALS['message']['filename'] . ' from incoming ' . $GLOBALS['incomingDirectory'] . $GLOBALS['message']['filename']
            . ' is invalid!', 
            'ERROR', 150
        );
        return false;
    } else {
        return true;
    }
}

/**
 * extract IOS
 * @param string $GLOBALS['message']['filename'] name of the ios
 * @param string $tmpDirectory path to the tmp dir
 * @return array $returnArray array of information of the extracted ios
 */
function extractMessage()
{
    if (DIRECTORY_SEPARATOR == "/") {
        $command = "7z x -y -o" . escapeshellarg($GLOBALS['message']['dirPathOnTmp']) 
                 . " " . escapeshellarg($GLOBALS['message']['dirPathOnTmp'] . $GLOBALS['message']['filename']);
    } else {
        $command = "\"" . str_replace(
            "\\", "\\\\", 
            $GLOBALS['docserversFeatures']['DOCSERVERS']['PATHTOCOMPRESSTOOL']
        )
        . "\" x -y -o" . escapeshellarg($GLOBALS['message']['dirPathOnTmp']) . " " 
        . escapeshellarg($GLOBALS['message']['dirPathOnTmp'] . $GLOBALS['message']['filename']);
    }
    $tmpCmd = '';
    $execError = '';
    //echo $command;exit;
    exec($command, $tmpCmd, $execError);
    if ($execError > 0) {
        Bt_exitBatch(
            24, 'Pb with extract:' . $command . ' ' . print_r($tmpCmd)
        );
    } else {
        //delete the 7z origin on tmp
        unlink($GLOBALS['message']['dirPathOnTmp'] . $GLOBALS['message']['filename']);
    }
}

/**
 * extract IOS
 * @param string $messageFilePath path of the ios
 * @return array $returnArray array of information of the ios control
 */
function controlMessage($tmpMessageFilename)
{
    $xml = new DOMDocument();
    $xml->load(
        $GLOBALS['message']['dirPathOnTmp'] 
        . DIRECTORY_SEPARATOR 
        . $tmpMessageFilename
    );
    
    if($xml->documentElement == false) {
        return false;
    }
    
    $GLOBALS['message']['filename'] = $tmpMessageFilename;
    $GLOBALS['message']['document'] = $xml;
    $GLOBALS['message']['xpath'] = new DOMXPath($GLOBALS['message']['document']);
    $rootNamespace = $xml->lookupNamespaceUri($xml->namespaceURI);
    $GLOBALS['message']['xpath']->registerNamespace('seda', $rootNamespace); 
    
    switch($rootNamespace) {
    case "fr:gouv:culture:archivesdefrance:seda:v1.0" :
        $GLOBALS['logger']->write(
            'Message uses seda V1', 
            'INFO'
        );
        break;
    
    case "fr:gouv:ae:archive:draft:standard_echange_v0.2":
    default:
        $GLOBALS['logger']->write(
                'Message uses seda V0.2', 
                'INFO'
            );
        $version = seda02to10();
        if(!$version) {
            $GLOBALS['logger']->write(
                'Error during conversion from SEDA 0.2 to 1.0!', 
                'ERROR'
            );
            return false;
        }
    }
    
    return true;
}

function seda02to10() {
    $xsl = new DOMDocument();
    $xsl->load(
        $_SESSION['config']['corepath']
        . 'modules/records_management/schema/transferts_v02tov10.xsl'
    );
    
    $xslt = new XSLTProcessor();
    $xslt->importStylesheet($xsl);
    
    $GLOBALS['message']['document'] = $xslt->transformToDoc($GLOBALS['message']['document']);
    $GLOBALS['message']['xpath'] = new DOMXPath($GLOBALS['message']['document']);
    $rootNamespace = $GLOBALS['message']['document']->lookupNamespaceUri($xml->namespaceURI);
    $GLOBALS['message']['xpath']->registerNamespace('seda', $rootNamespace); 
    
    $GLOBALS['message']['document']->save(
        $GLOBALS['message']['dirPathOnTmp'] 
        . DIRECTORY_SEPARATOR 
        . basename($GLOBALS['message']['filename'], '.xml') . '_v1.xml' 
    );
    $GLOBALS['message']['filename'] = basename($GLOBALS['message']['filename'], '.xml') . '_v1.xml';
    
    return true;
}

function prepareDocserver()
{
    //retrieve the good docserver
    $query = "select * from " . _DOCSERVERS_TABLE_NAME 
           . " where docserver_id = '" 
           . $GLOBALS['targetDocserver']
           . "' and coll_id = '" . $GLOBALS['collection'] . "'";
    Bt_doQuery($GLOBALS['db'], $query);
    if ($GLOBALS['db']->nb_result() == 0) {
        Bt_exitBatch(13, 'Docserver not found:' . $GLOBALS['targetDocserver']);
        break;
    } else {
        $docserversRecordset = $GLOBALS['db']->fetch_object();
        $GLOBALS['docserver'] = $GLOBALS['func']->object2array(
            $docserversRecordset
        );
    }
    $query = "select fingerprint_mode from " . _DOCSERVER_TYPES_TABLE_NAME
           . " where docserver_type_id = '" 
           . $GLOBALS['docserver']['docserver_type_id'] . "'";
    Bt_doQuery($GLOBALS['db'], $query);
    $fingerprintRecordset = $GLOBALS['db']->fetch_object();
    $GLOBALS['docserver']['fingerprint_mode'] = $fingerprintRecordset->fingerprint_mode;
    
    $resultPath = array();
    $resultPath = Ds_createPathOnDocServer(
        $GLOBALS['docserver']['path_template']
    );
    if ($resultPath['error'] <> "") {
        Bt_putFileInError(
            $GLOBALS['incomingDirectory'] 
                . $GLOBALS['messages'][$GLOBALS['currentMessage']], 
            $GLOBALS['messages'][$GLOBALS['currentMessage']], 
            $GLOBALS['batchDirectory'] . DIRECTORY_SEPARATOR . 'error', 
            $GLOBALS['wb']
        );
        Bt_exitBatch(
            18, $resultPath['error']
        );
    }
    $GLOBALS['docserver']['pathOnDocserver'] = $resultPath['destinationDir'];
    $GLOBALS['logger']->write(
        "target path on docserver:" . $GLOBALS['docserver']['pathOnDocserver'], 
        'INFO'
    );
}

function rejectMessage()
{
    $GLOBALS['logger']->write(
        "Rejecting Message...", 
        'INFO'
    );
    
    if($GLOBALS['ReplyCode']) {
        $GLOBALS['logger']->write(
            "Reply Code is " . $GLOBALS['ReplyCode'], 
            'INFO'
        );
        switch($GLOBALS['message']['type']) 
        {
        case 'ArchiveTransfer':
            
            if($_SESSION['RecordsManagement']['ArchiveTransfer']) {
                $GLOBALS['logger']->write(
                    "Maarch ArchiveTransfer available, will generate ArchiveTransferReply", 
                    'INFO'
                );
                $_SESSION['RecordsManagement']['ArchiveTransfer']->io_status = 'BRKO';
                $GLOBALS['RecordsManagementController']->save($_SESSION['RecordsManagement']['ArchiveTransfer'], false);
                createArchiveTransferReply();
            } else {
                $GLOBALS['logger']->write(
                    "Maarch ArchiveTransfer unavailable, no Reply generated", 
                    'INFO'
                );
            }
            break;
            
        case 'ArchiveTransferReply':
            // ???
            break;
            
        }  
    }
    
    Bt_putFileInError(
        $GLOBALS['incomingDirectory'] 
            . $GLOBALS['messages'][$GLOBALS['currentMessage']], 
        $GLOBALS['messages'][$GLOBALS['currentMessage']], 
        $GLOBALS['batchDirectory'] . DIRECTORY_SEPARATOR . 'error', 
        $GLOBALS['wb']
    );
    $GLOBALS['RecordsManagementController']->commit();

}
