<?php

/*
 *  Copyright 2008-2012 Maarch
 *
 *  This file is part of Maarch Framework.
 *
 *  Maarch Framework is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Maarch Framework is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @brief Batch to process the SEDA 2 MAARCH import
 *
 * @file
 * @author  Laurent Giovannoni  <dev@maarch.org>
 * @date $date$
 * @version $Revision$
 * @ingroup records_management
 */

/**
 * *****   LIGHT PROBLEMS without an error semaphore
 *  101 : Configuration file missing
 *  102 : Configuration file does not exist
 *  103 : Error on loading config file
 *  104 : SQL Query Error
 *  105 : a parameter is missing
 *  106 : Maarch_CLITools is missing
 *  107 : Stack empty for the request
 *  108 : There are still documents to be processed
 *  109 : An instance of the batch for the required policy and cyle is already
 *        in progress
 *  110 : Problem with collection parameter
 *  111 : Problem with the php include path
 *  112 : Problem with the setup of esign
 *  150 : Problem with an IOS, put it on error dir, execution has continue 
 *        with corrects IOS
 * ****   HEAVY PROBLEMS with an error semaphore
 *  11   : Problem with policy administration : Cycle step not found
 *  12  : Docserver type not found
 *  13  : Docserver not found
 *  14  : ...
 *  15  : Error to copy file on docserver
 *  16  : ...
 *  17  : Tmp dir not exists
 *  18  : Problem to create path on docserver, maybe batch number 
 *        already exists
 *  19  : Tmp dir not empty
 *  20  : ...
 *  21  : Problem to create directory on the docserver
 *  22  : Problem during transfert of file (fingerprint control)
 *  23  : Problem with compression
 *  24  : Problem with extract
 *  25  : Pb with fingerprint of the source
 *  26  : File deletion impossible
 *  27  : Resource not found
 *  28  : The docserver will be full at 95 percent
 *  29  : Error persists
 *  30  : Incoming dir not exists
 *  31  : Incoming dir empty
 *  32  : Pb with creation of a tmp dir in the tmp dir
 *  33  : Pb with copy in the tmp dir source/destination
 *  34  : No Ios found in the tmp dir
 *  35  : Pb with insertion item in DB
 */

date_default_timezone_set('Europe/Paris');
try {
    include('load_seda2maarch.php');
    include('messages.php');
    include('ios.php');
    include('archives.php');
    include('archiveObjects.php');
    include('documents.php');
    include('resources.php');
    if ($GLOBALS['customLang'] <> '' && file_exists($GLOBALS['customLang'])) {
        include($GLOBALS['customLang']);
    }
} catch (IncludeFileError $e) {
    echo "Maarch_CLITools required ! \n (pear.maarch.org)\n";
    exit(106);
}

/******************************************************************************/
/* beginning */
$GLOBALS['state'] = 'LOAD_REF';
while ($GLOBALS['state'] <> 'END') {
    if (isset($GLOBALS['logger'])) {
        $GLOBALS['logger']->write('STATE:' . $GLOBALS['state'], 'INFO');
    }
    switch($GLOBALS['state']) {
         /**********************************************************************/
        /*                          LOAD_REF                                  */
        /* Load References Organizations / Agreements                         */
        /**********************************************************************/
        case 'LOAD_REF' :
            $GLOBALS['RefOrganizations'] = 
                $GLOBALS['RecordsManagementController']->enumerate(
                    'RefOrganization'
                );
            
            $GLOBALS['ArchivalAgreements'] = 
                $GLOBALS['RecordsManagementController']->enumerate(
                    'ArchivalAgreement'
                );
            
            $GLOBALS['state'] = 'CONTROL_DIR';
            break;
        /**********************************************************************/
        /*                          CONTROL_DIR                               */
        /* Checking the incoming dir is empty                                 */
        /**********************************************************************/
        case 'CONTROL_DIR' :
            if ($GLOBALS['func']->isDirEmpty($GLOBALS['incomingDirectory'])) {
                Bt_exitBatch(
                    31, 'incoming dir empty:' . $GLOBALS['incomingDirectory']
                );
            }
            $GLOBALS['messages'] = 
                $GLOBALS['func']->getFilesInFolder(
                    $GLOBALS['incomingDirectory']
                );
            $GLOBALS['cptMessages'] = count($GLOBALS['messages']);
            if($GLOBALS['cptMessages'] == 0) {
                $GLOBALS['logger']->write('No message to process', 'INFO');
                $GLOBALS['state'] = 'END';
            } else {
                $GLOBALS['logger']->write(
                    'Number of messages to process:' . $GLOBALS['cptMessages'], 'INFO'
                );
                $GLOBALS['currentMessage'] = 0;
                Bt_updateWorkBatch();
                $GLOBALS['logger']->write('Batch number:' . $GLOBALS['wb'], 'INFO');
                //prepare docserver
                prepareDocserver();
                $GLOBALS['state'] = 'A_MESSAGE';
            }
            break;
        /**********************************************************************/
        /*                          A_MESSAGE                                 */
        /* Browse each file on the incoming dir                               */
        /**********************************************************************/
        case 'A_MESSAGE' :
            $GLOBALS['logger']->write(
                '******************************',
                'INFO'
            );
            $GLOBALS['logger']->write(
                'Processing Message' . $GLOBALS['currentMessage'],
                'INFO'
            );
            $GLOBALS['logger']->write(
                '******************************',
                'INFO'
            );
            $GLOBALS['logger']->write('BEGIN THE SQL TRANSACTION', 'INFO');
            Bt_doQuery($GLOBALS['db'], 'START TRANSACTION');
            $GLOBALS['RecordsManagementController']->startTransaction();
            $_SESSION['RecordsManagement'] = array(); 
            
            if ($GLOBALS['currentMessage'] < $GLOBALS['cptMessages']) {
                $GLOBALS['ReplyCode'] = false;
                $GLOBALS['message'] = array();
                $GLOBALS['message']['filename'] = $GLOBALS['messages'][$GLOBALS['currentMessage']];
                $GLOBALS['logger']->write(
                    'Current message:' 
                    . $GLOBALS['message']['filename'],
                    'INFO'
                );
                //check the IOS, fill $GLOBALS['message']
                $messageProcessed = 
                    processMessage($GLOBALS['message']['filename']);
                    
                if (!$messageProcessed) {
                    $GLOBALS['logger']->write(
                        'Processing KO for the import, skipping entire message', 
                        'ERROR', 150
                    );
                    rejectMessage();
                    $GLOBALS['currentMessage']++;
                    $GLOBALS['state'] = 'A_MESSAGE';
                } else {
                    $GLOBALS['logger']->write('Message OK, process IO!', 'INFO');
                    $GLOBALS['state'] = 'PROCESS_IO';
                }
            } else {
                $GLOBALS['logger']->write(
                    'No more message to process', 
                    'INFO'
                );
                $GLOBALS['state'] = 'END';
            }
            break;
        /**********************************************************************/
        /*                           PROCESS_IO                               */
        /* Import or update the IO                                            */
        /**********************************************************************/
        case 'PROCESS_IO' :            
            $io_processed = processIo();
            if (!$io_processed) {
                $GLOBALS['logger']->write(
                    'Process KO for the IO', 
                    'ERROR', 150
                );
                rejectMessage();
                $GLOBALS['currentMessage']++;
                $GLOBALS['state'] = 'A_MESSAGE';
            } else {
                $GLOBALS['logger']->write(
                    'Import/update OK for the IO', 
                    'INFO'
                );
                $GLOBALS['state'] = 'GET_ARCHIVES';
            }
            break;
        /**********************************************************************/
        /*                        GET_ARCHIVES                                */
        /* Process ArchiveTransfer/Contains                                   */
        /**********************************************************************/
        case 'GET_ARCHIVES' :
            // Get archives
            $GLOBALS['archives'] = 
                $GLOBALS['message']['xpath']->query(
                    '//seda:Archive'
                );
            $GLOBALS['cptArchives'] = $GLOBALS['archives']->length;
            $GLOBALS['logger']->write(
                'Number of archives to process:' . $GLOBALS['cptArchives'],
                'INFO'
            );
            $GLOBALS['currentArchive'] = 0; 
            $GLOBALS['state'] = 'AN_ARCHIVE';
            break;
        /**********************************************************************/
        /*                          AN_ARCHIVE                                */
        /* Process ArchiveTransfer/Contains                                   */
        /**********************************************************************/
        case 'AN_ARCHIVE' :            
            if ($GLOBALS['currentArchive'] < $GLOBALS['cptArchives']) {     
                $GLOBALS['logger']->write(
                    'Processing Archive ' . $GLOBALS['currentArchive'],
                    'INFO'
                );
                $GLOBALS['logger']->write(
                    '******************************',
                    'INFO'
                );
                $GLOBALS['archive'] = array();
                $GLOBALS['archive']['element'] = 
                    $GLOBALS['archives']->item($GLOBALS['currentArchive']);
                
                $processArchive = processArchive();
                
                if (!$processArchive) {
                    $GLOBALS['logger']->write(
                        'Processing KO for the Archive, skipping entire message', 
                        'ERROR', 150
                    );
                    rejectMessage();
                    $GLOBALS['currentMessage']++;
                    $GLOBALS['state'] = 'A_MESSAGE';
                } else {
                    $GLOBALS['logger']->write(
                        'Import/update OK for the Archive', 
                        'INFO'
                    );
                    $GLOBALS['state'] = 'GET_ARCHIVE_DOCUMENTS';
                }
            } else {
                $GLOBALS['logger']->write(
                    'No more archive to process, next message', 
                    'INFO'
                );
                // No more archives, save and go to next message
                $GLOBALS['logger']->write('END OF THE SQL TRANSACTION', 'INFO');
                Bt_doQuery($GLOBALS['db'], 'COMMIT');     
                $GLOBALS['RecordsManagementController']->commit();               
                $GLOBALS['currentMessage']++;
                $GLOBALS['state'] = 'A_MESSAGE';
            }
            break;
        /**********************************************************************/
        /*                        GET_ARCHIVE_DOCUMENTS                       */
        /*                                                                    */
        /**********************************************************************/
        case 'GET_ARCHIVE_DOCUMENTS' :
            // get Documents
            $GLOBALS['documents'] = 
                $GLOBALS['message']['xpath']->query(
                    './seda:Document',
                    $GLOBALS['archive']['element']
                );
            $GLOBALS['cptDocuments'] = $GLOBALS['documents']->length;
            $GLOBALS['logger']->write(
                'Number of documents to process for archive:' . $GLOBALS['cptDocuments'],
                'INFO'
            );
            $GLOBALS['currentDocument'] = 0; 
            $GLOBALS['state'] = 'AN_ARCHIVE_DOCUMENT';
            break;
        /**********************************************************************/
        /*                        GET_ARCHIVE_ARCHIVE_OBJECTS                 */
        /*                                                                    */
        /**********************************************************************/
        case 'GET_ARCHIVE_ARCHIVE_OBJECTS' :
            // Get archiveObjects
            $GLOBALS['archiveObjects'] = 
                $GLOBALS['message']['xpath']->query(
                    './seda:ArchiveObject', 
                    $GLOBALS['archive']['element']
                );
            $GLOBALS['cptArchiveObjects'] = $GLOBALS['archiveObjects']->length;
            $GLOBALS['logger']->write(
                'Number of archives objects to process:' . $GLOBALS['cptArchiveObjects'],
                'INFO'
            );
            $GLOBALS['currentArchiveObject'] = 0; 
            $GLOBALS['state'] = 'AN_ARCHIVEOBJECT';
            break;
        /**********************************************************************/
        /*                  AN_ARCHIVE_DOCUMENT                                */
        /* Extract file from the item and put it on the docserver             */
        /**********************************************************************/
        case 'AN_ARCHIVE_DOCUMENT' :            
            if ($GLOBALS['currentDocument'] < $GLOBALS['cptDocuments']) {
                $GLOBALS['logger']->write(
                    'Processing Archive Document ' . $GLOBALS['currentDocument'],
                    'INFO'
                );
                $GLOBALS['logger']->write(
                    '******************************',
                    'INFO'
                );
                $GLOBALS['document'] = array();
                $GLOBALS['document']['element'] = 
                    $GLOBALS['documents']->item($GLOBALS['currentDocument']);
                
                $DocumentProcessed = processDocument();
                
                if(!$DocumentProcessed) {
                    $GLOBALS['logger']->write(
                        'Processing KO for the Archive Document, skipping entire message', 
                        'ERROR', 150
                    );
                    rejectMessage();
                    $GLOBALS['currentMessage']++;
                    $GLOBALS['state'] = 'A_MESSAGE';
                } else {
                    $GLOBALS['logger']->write(
                        'Process OK for the Archive Document, process next Archive Document', 
                        'INFO'
                    );
                    $GLOBALS['currentDocument']++;
                    $GLOBALS['state'] = 'AN_ARCHIVE_DOCUMENT';
                }
            } else {
                // No more document, process ArchiveObjects
                $GLOBALS['logger']->write(
                    'No more document to process for archive, get Archive Objects', 
                    'INFO'
                );
                $GLOBALS['state'] = 'GET_ARCHIVE_ARCHIVE_OBJECTS';
            }
            break;
        /**********************************************************************/
        /*                          AN_ARCHIVEOBJECT                          */
        /* Process each items of the current IO                               */
        /**********************************************************************/
        case 'AN_ARCHIVEOBJECT' :            
            if ($GLOBALS['currentArchiveObject'] < $GLOBALS['cptArchiveObjects']) {
                $GLOBALS['logger']->write(
                    'Processing Archive Object ' . $GLOBALS['currentArchiveObject'],
                    'INFO'
                );
                $GLOBALS['logger']->write(
                    '******************************',
                    'INFO'
                );
                $GLOBALS['archiveObject'] = array();
                $GLOBALS['archiveObject']['element'] = 
                    $GLOBALS['archiveObjects']->item($GLOBALS['currentArchiveObject']);
                
                $processArchiveObject = processArchiveObject();
                
                if (!$processArchiveObject) {
                    $GLOBALS['logger']->write(
                        'Processing KO for the Archive object, skipping entire message', 
                        'ERROR', 150
                    );
                    rejectMessage();
                    $GLOBALS['currentMessage']++;
                    $GLOBALS['state'] = 'A_MESSAGE';
                } else {
                    // Get documents
                    $GLOBALS['logger']->write(
                        'Process OK for Archive Object, get Documents', 
                        'INFO'
                    );
                    $GLOBALS['state'] = 'GET_ARCHIVEOBJECT_DOCUMENTS';
                }
            } else {
                // No more archiveObjects, next archive
                $GLOBALS['logger']->write(
                    'No more Archive Objects, process next Archive', 
                    'INFO'
                );
                $GLOBALS['currentArchive']++;
                $GLOBALS['state'] = 'AN_ARCHIVE';
            }
            break;
            
        /**********************************************************************/
        /*                  GET_ARCHIVEOBJECT_DOCUMENTS                       */
        /*                                                                    */
        /**********************************************************************/
        case 'GET_ARCHIVEOBJECT_DOCUMENTS' :
            // get Documents
            $GLOBALS['documents'] = 
                $GLOBALS['message']['xpath']->query(
                    './seda:Document',
                    $GLOBALS['archiveObject']['element']
                );
            $GLOBALS['cptDocuments'] = $GLOBALS['documents']->length;
            $GLOBALS['logger']->write(
                'Number of documents to process for archive object:' . $GLOBALS['cptDocuments'],
                'INFO'
            );
            $GLOBALS['currentDocument'] = 0; 
            $GLOBALS['state'] = 'AN_ARCHIVEOBJECT_DOCUMENT';
            break;            
        /**********************************************************************/
        /*               AN_ARCHIVEOBJECT_DOCUMENT                            */
        /* Extract file from the item and put it on the docserver             */
        /**********************************************************************/
        case 'AN_ARCHIVEOBJECT_DOCUMENT' :            
            if ($GLOBALS['currentDocument'] < $GLOBALS['cptDocuments']) {
                $GLOBALS['logger']->write(
                    'Processing Archive Object Document ' . $GLOBALS['currentDocument'],
                    'INFO'
                );
                $GLOBALS['logger']->write(
                    '******************************',
                    'INFO'
                );
                $GLOBALS['document'] = array();
                $GLOBALS['document']['element'] = 
                    $GLOBALS['documents']->item($GLOBALS['currentDocument']);
                
                $DocumentProcessed = processDocument();
                
                if(!$DocumentProcessed) {
                    $GLOBALS['logger']->write(
                        'Processing KO for the Document, skipping entire message', 
                        'ERROR', 150
                    );
                    rejectMessage();
                    $GLOBALS['currentMessage']++;
                    $GLOBALS['state'] = 'A_MESSAGE';
                } else {
                    $GLOBALS['logger']->write(
                        'Process OK for Document, next Document', 
                        'INFO'
                    );
                    $GLOBALS['currentDocument']++;
                    $GLOBALS['state'] = 'AN_ARCHIVEOBJECT_DOCUMENT';
                }
            } else {
                // No more document, next archiveObject
                $GLOBALS['logger']->write(
                    'No more Document, process next ArchiveObject', 
                    'INFO'
                );
                $GLOBALS['currentArchiveObject']++;
                $GLOBALS['state'] = 'AN_ARCHIVEOBJECT';
            }
            break;
    }
}
$GLOBALS['logger']->write('End of process', 'INFO');
Bt_logInDataBase(
    $GLOBALS['totalProcessedResources'], 0, 'process without error'
);
Ds_washTmp($GLOBALS['tmpDirectory']);
unlink($GLOBALS['lckFile']);

/*@$GLOBALS['db']->disconnect();
@$GLOBALS['db2']->disconnect();
@$GLOBALS['db3']->disconnect();*/

exit($GLOBALS['exitCode']);
