<?php

require_once (
    'core/tests/class/DataObjectController.php'
);

class RecordsManagementController
    extends DataObjectController
{
    
    public function RecordsManagementController()
    {
        parent::__construct();
        parent::loadXSD(
            'modules/records_management/schema/RecordsManagement.xsd'
        );
    }
        
    /*************************************************************************/
    //
    // IOS / MESSAGES CREATION
    //
    /*************************************************************************/
    function createArchiveTransfer()
    {
        /************************************************************************** 
        ** Create dataObject    
        **************************************************************************/
        $ArchiveTransfer = $this->create('ArchiveTransfer');
        
        /************************************************************************** 
        ** Manage TransferringAgency    
        **************************************************************************/
        $TransferringAgency = 
            $this->addOrganization(
                $ArchiveTransfer,
                'TransferringAgency'
            );

        /* Add Contact   
        **************************************************************************/
        if(isset($_SESSION['RecordsManagement']['Contact'])) {
            $Contact = $this->loadXML(
                $_SESSION['RecordsManagement']['Contact']
            );
            $TransferringAgency[] = $Contact;
        }
        
        
        /************************************************************************** 
        ** Manage ArchivalAgency    
        **************************************************************************/
        $ArchivalAgency = 
            $this->addOrganization(
                $ArchiveTransfer,
                'ArchivalAgency'
            );
        
        /************************************************************************** 
        ** Return message
        **************************************************************************/
        return $ArchiveTransfer;
    }
    
    function saveMessage(
        $ArchiveTransfer
    ) {
        $this->startTransaction();
         
        // Delete relations for deleted archives 
        // (before save because action will remove deleted objects)
        if($ArchiveTransfer->io_id) {
            $this->deleteIOArchiveRelations($ArchiveTransfer); 
        } 
        
        $ArchiveTransfer_key = $this->save($ArchiveTransfer);
        
        // Create new relations for created archives or new 
        $this->createIOArchiveRelations($ArchiveTransfer); 
        
        $this->commit(); 
        
        return $ArchiveTransfer_key;
    }
    
    function readMessage(
        $io_id
    ) {
        $ArchiveTransfer = 
            $this->read(
                'ArchiveTransfer', 
                $io_id
            );

        $IOArchivesRelations = 
            $this->enumerate(
                'IOArchiveRelation', 
                $io_id
            );

        for($i=0; $i<$IOArchivesRelations->length; $i++) {
            $IOArchivesRelation = $IOArchivesRelations[$i];
            $item_id = $IOArchivesRelation->item_id;
            $Archive = 
                $this->read(
                    'Archive', 
                    $item_id
                );
            $ArchiveTransfer[] = $Archive;
        }
        
        return $ArchiveTransfer;
    
    }
    
    function deleteIOArchiveRelations(
        $Message
    ) {
        $Archives = $Message->Archive;
        $ArchiveLength = $Archives->length;
        for($i=0; $i<$ArchiveLength; $i++) {
            $Archive = $Archives[$i];
            if($relation = 
                $this->read(
                    'IOArchiveRelation',
                    $Message->io_id . ' ' 
                    . $Archive->item_id
                )
            ) {
                if(
                    $Message->isDeleted() 
                    || $Archive->isDeleted()) {
                    $this->delete($relation);
                    $this->save($relation);
                }
            }                
        } 
    }
    
    function createIOArchiveRelations(
        $Message
    ) {
        $Archives = $Message->Archive;
        $ArchiveLength = $Archives->length;
        for($i=0; $i<$ArchiveLength; $i++) {
            $Archive = $Archives[$i];
            $relation = 
                $this->read(
                    'IOArchiveRelation',
                    $Message->io_id . ' ' 
                    . $Archive->item_id
                );
            if(!$relation) {
                $relation = 
                    $this->create(
                        'IOArchiveRelation'
                    );
                $relation->io_id = $Message->io_id;
                $relation->item_id = $Archive->item_id;
                $this->save($relation);
            }
        } 
    
    }
    
    function addComment(
        $Message,
        $CommentText=false
    ) {
        $Comment = $this->create('Comment', $Message);
        
        if(isset($_SESSION['RecordsManagement']['Contact'])) {
            $Contact = $this->loadXML(
                $_SESSION['RecordsManagement']['Contact']
            );
            $Comment->author = $Contact->PersonName;
        }
        $Comment->date = date("Y-m-d H:i:s");
        
        if($CommentText) 
            $Comment->Comment = $CommentText;
        
        $Message[] = $Comment;
        return $Comment;
    }
    
    function updateComment(
        $Comment,
        $CommentText
    ) {
        $Comment->Comment = $CommentText;
        
        if(isset($_SESSION['RecordsManagement']['Contact'])) {
            $Contact = $this->loadXML(
                $_SESSION['RecordsManagement']['Contact']
            );
            $Comment->author = $Contact->PersonName;
        }
        $Comment->date = date("Y-m-d H:i:s");
    }
           
    function getMessageXml(
        $Message
    ) {
        $DocserverController = new DataObjectController();
        $DocserverController->loadXSD(
            $_SESSION['config']['corepath']
            . 'core/schema/Docserver.xsd'
        );
        
        $Docserver = 
            $DocserverController->read(
                'Docserver',
                $Message->docserver_id
            );
        
        $XmlMessage = new DOMDocument();
        $XmlMessage->load(
            $Docserver->path_template 
                . str_replace('#', DIRECTORY_SEPARATOR, $Message->io_path) 
                . $Message->io_filename
        );
        
        return $XmlMessage;
    }
    
    function createArchiveTransferReply(
        $ArchiveTransfer=false
    ) {       
        /************************************************************************** 
        ** Create dataObject    
        **************************************************************************/
        $ArchiveTransferReply = $this->create('ArchiveTransferReply');
        $ArchiveTransferReply->Date = date('Y-m-d H:i:s');    
        
        if($ArchiveTransfer) {
            $ArchiveTransferReply->TransferIdentifier = 
                $ArchiveTransfer->TransferIdentifier;
            
            $ArchiveTransferReply->TransferReplyIdentifier = 
                $ArchiveTransfer->TransferIdentifier . '_Reply';

            /************************************************************************** 
            ** Manage TransferringAgency    
            **************************************************************************/
            $ArchiveTransferTransferringAgency = 
                $ArchiveTransfer->TransferringAgency[0];
            
            $TransferringAgency = $this->copy(
                $ArchiveTransferTransferringAgency
            );
            
            $ArchiveTransferReply[] = $TransferringAgency;
                    
            /************************************************************************** 
            ** Manage ArchivalAgency    
            **************************************************************************/
            $ArchiveTransferArchivalAgency = 
                $ArchiveTransfer->ArchivalAgency[0];
            
            $ArchivalAgency = $this->copy(
                $ArchiveTransferArchivalAgency
            );
            
            $ArchiveTransferReply[] = $ArchivalAgency;
            
            /* Add Contact   
            **************************************************************************/
            if(isset($_SESSION['RecordsManagement']['Contact'])) {
                $Contact = $this->loadXML(
                    $_SESSION['RecordsManagement']['Contact']
                );
                $ArchivalAgency[] = $Contact;
            }
        } 
        /************************************************************************** 
        ** Return message
        **************************************************************************/
        return $ArchiveTransferReply;
    }
    
    function showMessage($Seda)
    {
        
        $XSL = new DOMDocument();
        $XSL->load(
            $_SESSION['config']['corepath'] .
            'modules/records_management/schema/seda/V1.0/seda2html/seda2html_v10.xsl'
        );

        $XSLT =  new XSLTProcessor();
        $XSLT->importStyleSheet($XSL);

        $HTML = $XSLT->transformToXml($Seda);
 
        echo $HTML;
    
    }
    
    /*************************************************************************/
    //
    // ORGANIZATIONS
    //
    /*************************************************************************/
    function listRefContactTransferringAgencies($RefContact) 
    {
        $RefContactTransferringAgencies = new DataObjectList();
        $RefContactRefOrganizations = 
            $this->enumerate('RefContactRefOrganization', $RefContact->user_id);
        $l = $RefContactRefOrganizations->length; 
        for($i=0; $i<$l; $i++) {
            $RefContactRefOrganization = $RefContactRefOrganizations->item($i);
            $TransferringAgency = 
                $this->read(
                    'RefOrganization', 
                    $RefContactRefOrganization->entity_id . ' TransferringAgency'
                );
            if($TransferringAgency) {
                $RefContactTransferringAgencies[] = $TransferringAgency;
                //var_dump($RefContactTransferringAgencies);
            }
        }
        return $RefContactTransferringAgencies;
    }
    
    function listRefOriginatingAgencyContacts($RefOriginatingAgency) 
    {
        $RefOriginatingAgencyContacts = new DataObjectList();
        $RefOrganizationsRefContacts = 
            $this->enumerate('RefOrganizationRefContact', $RefOriginatingAgency->entity_id);
        $l = $RefOrganizationsRefContacts->length; 
        for($i=0; $i<$l; $i++) {
            $RefOrganizationsRefContact = $RefOrganizationsRefContacts->item($i);
            $Contact = 
                $this->read(
                    'RefContact', 
                    $RefOrganizationsRefContact->user_id
                );
            if($Contact) {
                $RefOriginatingAgencyContacts[] = $Contact;
                //var_dump($RefContactTransferringAgencies);
            }
        }
        return $RefOriginatingAgencyContacts;
    }
    
    function listRefTransferringAgencies()
    {
        $RefTransferringAgencies = 
            $this->enumerate(
                'RefOrganization', 
                'TransferringAgency'
            );
        return $RefTransferringAgencies;
    }
    
    function listRefArchivalAgencies()
    {
        $RefArchivalAgencies = 
            $this->enumerate(
                'RefOrganization', 
                'ArchivalAgency'
            );
        return $RefArchivalAgencies;
    }
    
    function listRefOriginatingAgencies()
    {
        $RefOriginatingAgencies = 
            $this->enumerate(
                'RefOrganization', 
                'OriginatingAgency'
            );
        return $RefOriginatingAgencies;
    }
    
    function listRefRepositories()
    {
        $RefRepositories = 
            $this->enumerate(
                'RefOrganization', 
                'RefRepository'
            );
        return $RefRepositories;
    }
    
    function addTransferringAgency(
        $parentObject
    ) {
        $TransferringAgency = 
            $this->addOrganization(
                $parentObject,
                'TransferringAgency'
            );
        return $TransferringAgency;
    }
    
    function addArchivalAgency(
        $parentObject
    ) {
        $ArchivalAgency = 
            $this->addOrganization(
                $parentObject,
                'ArchivalAgency'
            );
        return $ArchivalAgency;
    }
    
    function addRepository(
        $parentObject
    ) {
        $Repository = 
            $this->addOrganization(
                $parentObject,
                'Repository'
            );
        return $Repository;
    }
    
    function addOriginatingAgency(
        $parentObject
    ) {
        $OriginatingAgency = 
            $this->addOrganization(
                $parentObject,
                'OriginatingAgency'
            );
        return $OriginatingAgency;
    }
    
    function addOrganization(
        $ParentObject,
        $role
    ) {
        $Organization = $this->create($role, $ParentObject);
        $Organization->role = $role;
        $ParentObject[] = $Organization;
        return $Organization;
    }
        
    // If oldOrganization provided, new Organization replaces it
    function selectRefOrganization(
        $parentObject,
        $entity_id,
        $role,
        $oldOrganization = false
    ) {
        $RefOrganization = 
            $this->read(
                'RefOrganization', 
                $entity_id . ' ' . $role
            );
        
        $Organization = 
            $this->appendRefOrganization(
                $parentObject,
                $RefOrganization,
                $role,
                $oldOrganization
            );
        
        return $Organization;
    }
    
    function appendRefOrganization(
       $parentObject,
       $RefOrganization,
       $role,
       $oldOrganization = false
    ) {
        $Organization = 
            $this->copy(
                $RefOrganization, 
                $role, 
                $resetKey = true,
                $parentObject
            );
        if($oldOrganization) {
            $this->delete($oldOrganization);
            $this->save($oldOrganization);
        }
        $parentObject[] = $Organization;
        
        return $Organization;
    }
    
    function addRefOrganizationRefAddress(
        $RefOrganization
    ) {
        $RefAddress = $this->read(
            'RefAddress',
            $RefOrganization->entity_id . ' ' . '*'
        );
        if($RefAddress) {
            $Address = $this->copy(
                $RefAddress, 
                'Address', 
                true,
                $RefOrganization
            );
            $Address->parent_type = $RefOrganization->nodeName;
            $RefOrganization[] = $Address;
            return $Address;
        }
    }
    
    function addContact(
        $Organization
    ) {
        $Contact = $this->create('Contact', $Organization);
        $Organization[] = $Contact;
        return $Contact;
    }
    
    function addRefContact(
        $Organization, 
        $user_id
    ) {
        $RefContact = $this->read('RefContact', $user_id);
        $Contact = 
            $this->copy(
                $RefContact, 
                'Contact', 
                true,
                $Organization
            );
        $Organization[] = $Contact;
        return $Contact;
    }
    
    function addRefContactRefAddress(
        $RefContact
    ) {
        $RefAddress = $this->read(
            'RefAddress',
            $RefContact->entity_id . ' ' . $RefContact->user_id
        );
        if($RefAddress) {
            $Address = 
                $this->copy(
                    $RefAddress, 
                    'Address',
                    true,
                    $RefContact
                );
            $Address->parent_type = 'Contact';
            $RefContact[] = $Address;
            return $Address;
        }
    }
    
    function getRefContactRefOrganizations($Contact, $OrganizationRole)
    {
        $return = array();
        $ContactUserId = $Contact->user_id;
        $users_entities = 
            $this->enumerate('users_entities');
        $l = $users_entities->length;  
        for($i=0; $i<$l; $i++) {
            $users_entity = $users_entities->item($i);
            if($ContactUserId == $users_entity->user_id) {
                $RefOrganization = 
                    $this->read(
                        'RefOrganization', 
                        $users_entity->entity_id . ' ' . $OrganizationRole
                    );
                if($RefOrganization) {
                    $return[] = $RefOrganization;
                }
            }
        }
        return $return;
    
    }
    
    /*************************************************************************/
    //
    // ITEMS / ARCHIVES / ARCHIVE OBJECTS
    //
    /*************************************************************************/
    function addArchive($Message)
    {
        /************************************************************************** 
        ** Create DataObject
        **************************************************************************/
        $Archive = $this->create('Archive', $Message);
        $Message[] = $Archive;
        
        /************************************************************************** 
        ** Add ContentDescription (1...1)
        **************************************************************************/
        $ContentDescription = $this->addContentDescription($Archive);
        
        /************************************************************************** 
        ** Add AccessRestrictionRule (1...1)
        **************************************************************************/
        $AccessRestrictionRule = $this->addAccessRestrictionRule($Archive);         
               
        return $Archive;
    }
    
    function addArchiveObject($Archive)
    {
        $ArchiveObject = $this->create('ArchiveObject', $Archive);      
        $Archive[] = $ArchiveObject;
        return $ArchiveObject;
    }
    
    function addContentDescription($Item)
    {
        /************************************************************************** 
        ** Create DataObject
        **************************************************************************/
        $ContentDescription = $this->create('ContentDescription', $Item);
        $Item[] = $ContentDescription;       
        
        return $ContentDescription;
    }
    
    function addAccessRestrictionRule($ParentElement)
    {
        $AccessRestrictionRule = $this->create('AccessRestrictionRule', $ParentElement);
        $AccessRestrictionRule->StartDate = date('Y-m-d');
        $ParentElement[] = $AccessRestrictionRule;
        return $AccessRestrictionRule;
    }
    
    function addAppraisalRule($ParentElement)
    {
        $AppraisalRule = $this->create('AppraisalRule', $ParentElement);
        $AppraisalRule->StartDate = date('Y-m-d');
        $ParentElement[] = $AppraisalRule;
        return $AppraisalRule;
    }
    
    function addKeyword($ParentElement) 
    {
        $Keyword = $this->create('Keyword', $ParentElement);
        $ParentElement[] = $Keyword;
        return $Keyword;
    }
    
    function addCustodialHistory($ParentElement) 
    {
        $CustodialHistory = $this->create('CustodialHistory', $ParentElement);
        $ParentElement[] = $CustodialHistory;
        return $CustodialHistory;
    }
    
    function addDocument($ParentElement) 
    {
        $Document = $this->create('Document', $ParentElement);
        $ParentElement[] = $Document;
        return $Document;
    }
    
    /*************************************************************************/
    //
    // FILE PLAN
    //
    /*************************************************************************/
    function createFilePlan()
    {
        $FilePlan = $this->create('FilePlan');
       
        return $FilePlan;
    }
    
    /*************************************************************************/
    //
    // SEDA 2 MAARCH
    //
    /*************************************************************************/    
    function exportMessage(
        $IO
    ) {
        $XSL = new DOMDocument();
        $XSL->load(
            $_SESSION['config']['corepath'] .
            'modules/records_management/schema/SEDA__export.xsl'
        );

        $XSLT =  new XSLTProcessor();
        $XSLT->importStyleSheet($XSL);

        $SEDA = $XSLT->transformToDoc($IO);
 
        return $SEDA;
    }
    
    function importMessage(
        $SEDA
    ) {
        $XSL = new DOMDocument();
        $XSL->load(
            $_SESSION['config']['corepath'] .
            'modules/records_management/schema/SEDA__import.xsl'
        );

        $XSLT =  new XSLTProcessor();
        $XSLT->importStyleSheet($XSL);

        $IOXML = $XSLT->transformToXML($SEDA);
        
        $IO = $this->loadXML($IOXML);
        
        return $IO;
    }
        
    function searchArchiveTransfer(
        $SedaArchiveTransfer
    ) {
        $SedaXPath = new DOMXPath($SedaArchiveTransfer);
        $rootNamespace = $SedaArchiveTransfer->lookupNamespaceUri($SedaArchiveTransfer->namespaceURI);
        $SedaXPath->registerNamespace('seda', $rootNamespace); 
        
        // Search Transfer Identifier
        $SedaTransferIdentifier = 
            $SedaXPath->query('//seda:TransferIdentifier')->item(0)->nodeValue;
        
        $ArchiveTransfers = 
            $this->enumerate(
                'ArchiveTransfer',
                $SedaTransferIdentifier
            );
        $nb = $ArchiveTransfers->length;
        if($nb == 0) return false;
        
        $SedaTransferringAgencyIdentification = 
            $SedaXPath->query('//seda:TransferringAgency/seda:Identification')->item(0)->nodeValue;
        for($i=0; $i<$nb; $i++) {
            $ArchiveTransfer = $ArchiveTransfers->item($i);
            $this->readChildren($ArchiveTransfer);
            
            $TransferringAgency = $ArchiveTransfer->TransferringAgency[0];
            if($TransferringAgency->Identification 
                == $SedaTransferringAgencyIdentification)
                return $this->readMessage(
                        $ArchiveTransfer->io_id
                    );
        }
        
    }
    
}