package controleur;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.Random;

import droid.Droid;
import droid.Droid.Profile;
import java.io.IOException;
import java.security.PrivilegedActionException;
import java.util.logging.Logger;
import maarcharchivetransfer.fileManager;

import parserXML.Parser;
import vue.DescriptiveForm;

import maarcharchivetransfer.myLogger;
import modele.AbstractArchive;
import modele.AbstractArchiveObject;
import modele.AbstractDocument;
import modele.MyFile;

public class DescriptiveFormControler extends AbstractControler {

	private myLogger logger = new myLogger(System.getProperty("user.home")+java.io.File.separator+"maarchtmp"+java.io.File.separator+"archivetransfers"+java.io.File.separator);

	private Droid droid = new Droid();
	
	public DescriptiveFormControler() {	}
	
	public AbstractArchiveObject getPreviousArchiveObject(){
		return archiveTransfer.getPreviousArchiveObject();
	}
	
	//article suivant
	public AbstractArchiveObject getNextArchiveObject(){
		return archiveTransfer.getNextArchiveObject();
	}
	
	@Override
	public void removeDocument(int indexDocument) {
		archiveTransfer.removeDocumentFromCurrentArchiveObject(indexDocument);
	}

	@Override
	public void addDocument(File file) {
		
		//si file est un fichier
		if(file.isFile()) {
			addFileDocument(file);
		}
		else if(file.isDirectory()){
			File[] files = file.listFiles();
			for(File f : files){
				addDocument(f);
			}
		}
	}
		
	private void addFileDocument(File file) {

		if(file.exists()){			
			MyFile doc = new  MyFile(file);
			if(archiveTransfer.agreementRespected(doc) == 0 ){
				archiveTransfer.addDocumentToCurrentArchiveObject(doc);
				logger.log("----------File "+ doc.getName() +" added to the item----------", Level.INFO);
			}
			else if(archiveTransfer.agreementRespected(doc) == 1 ){
				logger.log("----------File "+ doc.getName() +" not added to the item : format not allowed----------", Level.INFO);
			}
			else if(archiveTransfer.agreementRespected(doc) == 2 ){
				logger.log("----------File "+ doc.getName() +" not added to the item : maximum number of transfers reached----------", Level.INFO);
			}
			else if(archiveTransfer.agreementRespected(doc) == 3 ){
				logger.log("----------File "+ doc.getName() +" not added to the item : maximum size per transfer reached----------", Level.INFO);
			}
		}
		
	}
	

	
	private void addFileDocumentDroid(File file) {

		if(file.exists()){			
			MyFile doc = new  MyFile(file);
			Profile profile = droid.generateFormats(file);
			if(profile != null){
				doc.setFormat(profile.getFormat());
				doc.setPronom(profile.getPronom());
			}
			if(archiveTransfer.agreementRespected(doc) == 0){
				
				archiveTransfer.addDocumentToCurrentArchiveObject(doc);
				logger.log("----------File "+ doc.getName() +" added to the item----------", Level.INFO);
			}
			else if(archiveTransfer.agreementRespected(doc) == 1 ){
				logger.log("----------File "+ doc.getName() +" not added to the item : format not allowed----------", Level.INFO);
			}
			else if(archiveTransfer.agreementRespected(doc) == 2 ){
				logger.log("----------File "+ doc.getName() +" not added to the item : maximum number of transfers reached----------", Level.INFO);
			}
			else if(archiveTransfer.agreementRespected(doc) == 3 ){
				logger.log("----------File "+ doc.getName() +" not added to the item : maximum size per transfer reached----------", Level.INFO);
			}
		}
		
	}


	public void parser(String path){
		parser = new Parser(path);
		archiveTransfer = parser.parseArchiveTransferXml();
	}
	
	public void compression(){

		List<AbstractArchiveObject> articles = archiveTransfer.getArchive().getArchiveObjects();
		Random rand = new Random();
                fileManager fM = new fileManager();
		for(AbstractArchiveObject art : articles){
			if(art.getIsDigital()){
				List<AbstractDocument> documents = art.getDocuments();
				ArrayList<MyFile> filesToZip =  new ArrayList<MyFile>();
				for(AbstractDocument f : documents){
					if(f instanceof MyFile){
						filesToZip.add((MyFile)f);
					}
				}
				
				if(!filesToZip.isEmpty()){
					try {	
						appZip.setFileList(filesToZip);
						appZip.zipIt(art.getNumArchiveObject(), archiveTransfer.getArchive().getTransferringAgencyArchiveIdentifier()); 
					} 
					catch (Exception e) {
						e.printStackTrace();
					}
				} else {
                                    logger.log("NO FILE FOR THE CURRENT ITEM " + art.getNumArchiveObject(), Level.SEVERE);
                                    logger.log("SEMAPHORE CREATION", Level.INFO);
                                    try {
                                        fM.createFile("semaphore", System.getProperty("user.home") 
                                            + "\\maarchtmp\\" + "\\archivetransfers\\" + archiveTransfer.getArchive().getTransferringAgencyArchiveIdentifier()
                                            + "_" + rand.nextInt()
                                    );
                    } catch (IOException ex) {
                        Logger.getLogger(DescriptiveFormControler.class.getName()).log(Level.SEVERE, null, ex);
                    } catch (PrivilegedActionException ex) {
                        Logger.getLogger(DescriptiveFormControler.class.getName()).log(Level.SEVERE, null, ex);
                    }
                                }
			} else {
                            try {
                                appZip.completeXmlFileWithoutElectronicItem(archiveTransfer.getArchive().getTransferringAgencyArchiveIdentifier(), art.getNumArchiveObject());
                            }
                            catch (Exception e) {
                                    e.printStackTrace();
                            }
                            
                        }
		}
		
	}
	

	public void setArchivalAgreement(String formatsList, String nbArticles,	String transferSize) {
		String[] formats = formatsList.split("\\|");
		int nbArticlesMax = Integer.parseInt(nbArticles);
		long transferSizeMax = Long.parseLong(transferSize);
		archiveTransfer.setArchivalAgreement(formats, nbArticlesMax, transferSizeMax );
	}

	@Override
	public void addObserver(DescriptiveForm descriptiveForm) {
		archiveTransfer.addObserver(descriptiveForm);
	}

	@Override
	public AbstractArchive getArchive() {
		return archiveTransfer.getArchive(); 
	}

	
}
