package droid;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import maarcharchivetransfer.fileManager;

public class Droid {	
	
	private Profile profile;

	/** repertoire contenant les fichiers du profil **/
	private static String tempDroidDirectory = "droid";
	
	private static String tempMaarchDirectory = "maarchtmp";
	
	private static String pathTempDroidDirectory;     //de type /home/maarchtmp/droidtmp/
	
	private static String profileFile = "profile.droid";
	
	private static String exportCsvFile = "export.csv";
	
	private boolean signaturesDownloaded = false;

	private String javaPath = "C:\\Program Files\\Java\\jdk1.6.0_31\\bin\\java.exe";
        //private String javaPath = "/home/maarch/jrockit-jdk1.6.0_33-R28.2.4-4.1.0/bin/java";

	private String droidCommandLineJar = "C:\\Users\\lgi\\Downloads\\droid-6.01\\droid-command-line-6.0.jar";
        //private String droidCommandLineJar = "/home/maarch/Documents/droid-6.01/droid-command-line-6.0.jar";

	
	
	public Droid(){
		//setTempDirectory();
		pathTempDroidDirectory = System.getProperty("user.home") 
				+ File.separator + tempMaarchDirectory 
				+ File.separator + tempDroidDirectory 
				+ File.separator ;
	}
	
	public Droid(String tempMaarchDirectory, String tempDroidDirectory){
		//setTempDirectory();
		Droid.tempMaarchDirectory = tempMaarchDirectory;
		Droid.tempDroidDirectory = tempDroidDirectory;
		pathTempDroidDirectory = System.getProperty("user.home") 
				+ File.separator + tempMaarchDirectory 
				+ File.separator + tempDroidDirectory 
				+ File.separator ;
	}
	
	
	
	
	
	/** telechargement des signatures 
	 * @param action **/
	public void downloadSignatures() 
			//throws ImportArchiveIOException, ImportArchiveInterruptedException 
	{
		
		String action = "Téléchargement des signatures";
	    
	    /** message erreur téléchargement des dernières signatures.*/
	    String messageErrCmdTelechgmtSign = "problème rencontré "
	    + "lors de l'éxécution de la commande du téléchargement des dernières signatures ";
	    
	    String messageIO = "commande droid de téléchargement de signatures";
	    
	    String messageInterrupted = "execution de la commande droid de téléchargement de signatures";
	
		//Téléchargement des dernières signatures
		String droidCmdToDownloadSignature = javaPath + " -jar " + droidCommandLineJar + " -d";
		 
		System.out.println("Commande de téléchargement des signatures: " + droidCmdToDownloadSignature);
		 
		this.signaturesDownloaded = processLancementCmd(droidCmdToDownloadSignature, 
				messageErrCmdTelechgmtSign,
				messageIO, 
				messageInterrupted,
				action);
		
	}
	
	
	public boolean generateProfile(File file){
		
		String action = "Création du profil pour le fichier " + file.getName() 
				+ " et execution du profil";
		
		String messageErrCmdAddFileToProfileAndRun = "Une erreur s'est produite lors de la création et de l'execution du profil droid";
		
		String messageIO = "";
		
		String messageInterrupted = "";
		
		String droidCmdToCreateProfilResult = javaPath + " -jar " + droidCommandLineJar + " -p " + "\"" + pathTempDroidDirectory + profileFile + "\" -a \"" + file.getAbsolutePath() + "\"";
		 
		System.out.println("Commande de création et execution des profils droid : " + droidCmdToCreateProfilResult);
		 
		return processLancementCmd(droidCmdToCreateProfilResult,
				messageErrCmdAddFileToProfileAndRun, 
				messageIO, 
				messageInterrupted, 
				action);
	}
	

	public boolean exportProfile(){
		boolean done = false;
		
		//TODO
		File profile = new File(pathTempDroidDirectory + profileFile);
		
		if(profile.exists()){
			String action = "Export du profil";
			
			String messageErrCmdAddFileToProfileAndRun = "Une erreur s'est produite lors de l'export du profil droid";
			
			String messageIO = "";
			
			String messageInterrupted = "";
			
			String droidCmdExportProfils = javaPath + " -jar " + droidCommandLineJar + " -p " + "\"" + pathTempDroidDirectory + profileFile +  "\" -e \"" + pathTempDroidDirectory + exportCsvFile + "\"";
			   
			System.out.println("Commande de création et execution des profils droid : " + droidCmdExportProfils);
			 
			done = processLancementCmd(droidCmdExportProfils,
					messageErrCmdAddFileToProfileAndRun, 
					messageIO, 
					messageInterrupted, 
					action);
		}
		return done;
	}


	public Profile generateFormats(File file) {
		
		Profile profile = null;
		
		try {
			if(!this.signaturesDownloaded){
				downloadSignatures();
			}

			deleteProfileAndExport();
			createTempDirectories();
			
			//TODO log

			if(file.exists() && generateProfile(file)){
				exportProfile();
			}
			
			profile = recuperationFormatFichier();//new Profile("application/test", "fmt/34");
			
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return profile;
		
	}
	

	
	public void createTempDirectories() throws IOException{
		//TODO logs
		
		fileManager fM = new fileManager();
		fM.createUserLocalDirTmp(System.getProperty("user.home") 
				+ File.separator + tempMaarchDirectory );
		fM.createUserLocalDirTmp(pathTempDroidDirectory);
			
	}
	
	public void recursiveDeleteFile(final File fileToDelete){
		
		if(fileToDelete!=null && fileToDelete.exists()){
			
			if(fileToDelete.isFile()){
				if(fileToDelete.delete()){
					;//TODO logs
				}
				
			}else if(fileToDelete.isDirectory()){
				for( File file : fileToDelete.listFiles()){
					recursiveDeleteFile(file);
					if(fileToDelete.delete()){
						;//TODO logs
					}
				}
				
			}	
		}
		
	}
	
	private void deleteProfileAndExport(){
		File profile = new File(pathTempDroidDirectory + profileFile);
		File export  = new File(pathTempDroidDirectory + exportCsvFile);
		recursiveDeleteFile(profile);
		recursiveDeleteFile(export);
	}

	
	public String getFileFormat(){
		return null;
	}
	
	
	public boolean parseCsv(){
		return (profile != null);
	}
	
	
	public String getPronomFile(String filePath){
		return profile.getPronom();
	}
	
	public String getFormatFile(String filePath){
		return profile.getFormat();
	}
	
	
	
	private boolean processLancementCmd(String droidCmd, 
			String messageError,
			String messageLogIOExcption,
			String messageLogInterruptedException, 
			String descAction)
	{
		boolean done = false;
		Process pExportProfils = null;
		try {
			pExportProfils = Runtime.getRuntime().exec(droidCmd);
			//TODO log print the action in process and corresponding action : command descAction + droidCmd
			System.out.println(descAction);
			System.out.println("Lancement de la commande droid: ");
			System.out.println(droidCmd); 
			if (null != pExportProfils) {
				pExportProfils.waitFor();
				done = true;
				System.out.println("Commande executée");
			}		 
		} catch (IOException e) {
			//TODO log : error + messageLogIOExcption
			System.out.println(messageLogIOExcption);
		} catch (InterruptedException e) {
			//TODO log : error + messageLogInterruptedException
			System.out.println(messageLogInterruptedException);
		} 
		
		return done;
	}
	

	
	public class Profile{
		
		private String format;
		private String pronom;
		
		public Profile(String format, String pronom){
			this.format = format;
			this.pronom = pronom;
		}
		
		public String getFormat(){
			return this.format;
		}
		
		public String getPronom(){
			return this.pronom;
		}
		
		public void setFormat(String format){
			this.format = format;
		}

		public void setPronom(String pronom){
			this.pronom = pronom;
		}
		
	}

	
	
	
	
	


private Profile recuperationFormatFichier()  {
	
	System.out.println("recuperationFormatFichiers - start ");
	Map<String,String> formatsFichiers = new HashMap<String, String> ();
	
	// lecture du fichier csv
	FileInputStream fileInFormat = null;
	Reader inEncoder = null;
	BufferedReader bufReaderDocFormat = null;
	File formatFichier = new File(pathTempDroidDirectory + exportCsvFile);
	
	try {
		fileInFormat = new FileInputStream(formatFichier);
		inEncoder = new InputStreamReader(fileInFormat, "ISO_8859_1");
		bufReaderDocFormat = new BufferedReader(inEncoder);
		
		String ligne = bufReaderDocFormat.readLine();
		System.out.println("ligne: " + ligne);
		int i = 0;
		//while (ligne != null) {
		if (ligne != null) {			
			System.out.println("ligne dans la boucle while: " + ligne);
			System.out.println("valeur de i dans la boucle while: " + i);
			ligne = bufReaderDocFormat.readLine();
			i++;
			//if (i > 0) {
				ligne = ligne.replaceAll("\\,\\,","\\,#\\,");
				System.out.println("après remplacement ,, ligne: " + ligne);
				ligne = ligne.replaceAll("\\,\"\\,\"","\\,\"#\\,\"");
				System.out.println("après remplacement ,'', ligne: " + ligne);
				String[] ChampLigne = ligne.split(",");
				String mimeCode = ChampLigne[15];				
				System.out.println("MIME code: " + mimeCode);
				String typeFichier = ChampLigne[8];
				System.out.println("typeFichier: " + typeFichier);
				String pronom = ChampLigne[14];
				System.out.println("format pronom: " + pronom);
				if (!typeFichier.equals("Folder")) {
					mimeCode = mimeCode.replaceAll("\"", "");
					pronom = pronom.replaceAll("\"", "");
				    System.out.println("nomFichier avant insertion dans la map: " + mimeCode);
				    System.out.println("format avant insertion dans la map: " + pronom);
				    profile = new Profile(mimeCode, pronom);
				}
			//}
			
			System.out.println("valeur de i dans la boucle while: " + i);
			ligne = bufReaderDocFormat.readLine();
			i++;
		}
		System.out.println("sortie de la boucle while");
	} catch (FileNotFoundException e) {
		String messageErrBufWriter = "une erreur est survenue le fichier exportProfils.csv n'a pas été trouvé lors de la recuperation du format du fichier";
            System.err.println(
            		messageErrBufWriter);
	} catch (UnsupportedEncodingException e) {
		String messageErrBufWriter = "une erreur est survenue "
            + "lors de la lecture du fichier: exportProfils.csv avec l'encodage UTF8 lors de la recuperation du format du fichier";
            System.err.println(
            		messageErrBufWriter);
	} catch (IOException e) {
		String messageErrBufWriter = "une erreur est survenue "
            + "lors de la lecture du fichier: exportProfils.csv lors de la recuperation du format du fichier";
            System.err.println(
            		messageErrBufWriter);
	} finally {
		if (null != bufReaderDocFormat) {
			try {
				System.out.println("fermeture de bufReaderDocFormat");
				bufReaderDocFormat.close();
			} catch (IOException e) {
				String messageErrBufWriter = "une erreur est survenue "
	                + "lors de fermeture du bufReaderDocFormat lors de la recuperation du format du fichier";
	                System.err.println(
	                		messageErrBufWriter);
			}
		}
		
		if (null != inEncoder) {
			try {
				System.out.println("fermeture de inEncoder");
				inEncoder.close();
			} catch (IOException e) {
				String messageErrBufWriter = "une erreur est survenue "
	                + "lors de fermeture du inEncoder lors de la recuperation du format du fichier";
	                System.err.println(
	                		messageErrBufWriter);
			}
		}
		
		if (null != fileInFormat) {
			try {
				System.out.println("fermeture de fileInFormat");
				fileInFormat.close();
			} catch (IOException e) {
				String messageErrBufWriter = "une erreur est survenue "
	                + "lors de fermeture du fileInFormat lors de la recuperation du format du fichier";
	                System.err.println(
	                		messageErrBufWriter);
			}
		}
		System.out.println("fin du finally");
		System.out.println("taille formatsFichiers: " + formatsFichiers.size());
		
	}
	
	
	System.out.println("recuperationFormatFichiers - end ");
	
	return profile;
}
	
	
	
}
