package modele;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.table.AbstractTableModel;




public class FilesTable extends AbstractTableModel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6788265792286099034L;

	private List<AbstractDocument> data ;

    private String[] header = {"Nom", "Format", "Pronom", "Taille", "Supp." };

    public FilesTable() {
        super();
    	data = new ArrayList<AbstractDocument>();
    }
	
	public FilesTable(List<AbstractDocument> documents) {
        super();
        data = documents;
	}

	public void setData(List<AbstractDocument> documents) {
		data = documents;
	}

	public int getRowCount() {
        return data.size();
    }

    public int getColumnCount() {
        return header.length;
    }

    public String getColumnName(int columnIndex) {
        return header[columnIndex];
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
    	AbstractDocument doc = data.get(rowIndex);
    	if (doc instanceof MyFile){
    		MyFile document = (MyFile)doc;
	        switch(columnIndex){
	            case 0:
	                return document.getName();
	            case 1:
	                return document.getFormat();
	            case 2:
	                return document.getPronom();
	            case 3:
	                return String.format("%.1f Ko", document.getSizeInKo());
	            case 4:
	                return new JButton();
	            default:
	                return null;
	        }
    	}
    	//else{}  //si doc papier
		return null;
    }

    public void addFile(MyFile file) {
    	data.add(file);
        fireTableRowsInserted(data.size() -1, data.size() -1);
    }

    public void removeFile(int rowIndex) {
    	data.remove(rowIndex);
        fireTableRowsDeleted(rowIndex, rowIndex);
    }


    public Class<?> getColumnClass(int columnIndex){
    	switch(columnIndex){
    		case 4:
    			return JButton.class;
    		default:
    			return Object.class;
    	}
    }
    
    

	public void removeRow(int position){
		
		System.out.println("Suppression de la ligne "+ position);
		if(data!= null && position >= 0 && position < data.size()){
			data.remove(position) ;	
			this.fireTableDataChanged();	
		}

	}
	

	public void addRow(AbstractDocument data){		
		if(data!= null){
			this.data.add( data);
			this.fireTableDataChanged();
		}
	}
	

	public boolean isCellEditable(int row, int col){
		if(getValueAt(0, col) instanceof JButton)
			return true;
		return false;
	}
	
}
