package parserXML;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

import modele.AbstractArchive;
import modele.AbstractArchiveObject;
import modele.AbstractArchiveTransfer;
import modele.ArchiveObject;
import modele.ArchiveTransfer;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import java.io.File;


public class Parser {

	private AbstractArchiveTransfer archiveTransfert;
	private String path ;
	
	public Parser(String path){
		this.archiveTransfert = new ArchiveTransfer();
		this.path = path ;
	}
	
	
	public AbstractArchiveTransfer parseArchiveTransferXml(){
 
		
		
		AbstractArchive archive = archiveTransfert.getArchive();
		
		//archive
		String TransferringAgencyArchiveIdentifier;
		String transferringAgencyContactName;
		String originatingAgencyContactName;
		String transferringAgencyName;
		String originatingAgencyName;
		String transferIdentifier ; 
		String typeEntree;
		String date;
		
		//article
		String transferringAgencyObjectIdentifier;
		String description;
		String oldestDate;
		String latestDate;		
		String flag;
		String serie;
		String type;		
		String appraisalRule;
		String dua;
		
		try {
 
			File fXmlFile = new File(path);
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(fXmlFile);
			doc.getDocumentElement().normalize();
			
			Element eArchiveTransfer = doc.getDocumentElement();
			System.out.println("Message :" + eArchiveTransfer.getNodeName());
			
			//archive transfer identifier
			transferIdentifier = getTagValue("TransferIdentifier", eArchiveTransfer);
			archiveTransfert.setArchiveTransferId(transferIdentifier);

			//archive transfer date
			date = getTagValue("Date", eArchiveTransfer);
			archive.setDate(date);

			//node transferring agency
			Node nTransferringAgency = getFirstNodeByTagName("TransferringAgency", eArchiveTransfer);
			
            //transferringAgencyName = getTagValue("DepartmentName", nTransferringAgency);
            transferringAgencyName = getTagValue("Name", nTransferringAgency);
			archive.setTransferringAgency(transferringAgencyName);
            
			//responsable du versement
			transferringAgencyContactName = getTagValue("PersonName", nTransferringAgency);
			archive.setTransferringAgencyContact(transferringAgencyContactName);
			
			
			//archive
            // ****************************************************************
			NodeList nlArchive = eArchiveTransfer.getElementsByTagName("Archive");
            System.out.println("Archives : " + nlArchive.getLength());
            Node nArchive = nlArchive.item(0);
			
			//numero d'archive
			TransferringAgencyArchiveIdentifier = getTagValue("TransferringAgencyArchiveIdentifier", nArchive);
			archive.setTransferringAgencyArchiveIdentifier(TransferringAgencyArchiveIdentifier);
			
			//entite productrice
			Node nOriginatingAgency = getFirstNodeByTagName("OriginatingAgency", nArchive);		
			
			//nom entite productrice
			originatingAgencyName = getTagValue("Name", nOriginatingAgency);
			archive.setOriginatingAgency(originatingAgencyName);
			
			//service producteur
			originatingAgencyContactName = getTagValue("DepartmentName", nOriginatingAgency);
			archive.setOriginatingAgencyComment(originatingAgencyContactName);
			
			//type d entree
			typeEntree = getTagValue("CustodialHistoryItem", nArchive);
			archive.setType(typeEntree);
			

			//regle de gestion
			Node nAppraisalRule = getFirstNodeByTagName("AppraisalRule", nArchive);
			
			//code regle de gestion
			appraisalRule = getTagValue("Code", nAppraisalRule);
			
			//dua
			dua = getTagValue("Duration", nAppraisalRule);
			
			//artiles
			if (nArchive.getNodeType() == Node.ELEMENT_NODE) {
				NodeList nLArticles = ((Element) nArchive).getElementsByTagName("ArchiveObject");
				for (int i = 0; i < nLArticles.getLength(); i++) {
			 
					Node nNode = nLArticles.item(i);
					if (nNode.getNodeType() == Node.ELEMENT_NODE) {
						
						AbstractArchiveObject article = new ArchiveObject();
						 
						Element eArticle = (Element) nNode;
	 
						//id (cote)
						transferringAgencyObjectIdentifier = getTagValue("TransferringAgencyObjectIdentifier", eArticle);
						article.setNumArchiveObject(transferringAgencyObjectIdentifier);
						
                        //description
						description = getTagValue("Name", eArticle);
						article.setDescription(description);
                        
                        //type
                        type = getTagValue("Description", eArticle);
						article.setArchiveType(type);
                        
						//document
						Node nDocument = getFirstNodeByTagName("Document", eArticle);
						if(nDocument == null){
							article.setIsDigital(false);
						} else {	
							Node nSize = getFirstNodeByTagName("Size", (Element) nDocument);
                            if(nSize == null){
                                article.setIsDigital(false);
                            } else {
                                article.setIsDigital(true);
                            }							
						}
						
        				//content description
						Node nContentDescription = getFirstNodeByTagName("ContentDescription", eArticle);

						//serie ou sous-serie
						serie = getTagValue("FilePlanPosition", nContentDescription);
						article.setSerie(serie);
						
						//support
						flag = getTagValue("Format", nContentDescription);
						article.setFlag(flag);
						
						//description
						//description = getTagValue("Description", nContentDescription);
						//article.setDescription(description);
						
						//date extreme debut
						oldestDate = getTagValue("OldestDate", nContentDescription);
						article.setDateB(oldestDate);
						
						//date extreme fin
						latestDate = getTagValue("LatestDate", nContentDescription);
						article.setDateE(latestDate);
						
						article.setAppraisalRule(appraisalRule);
						article.setDua(dua);
						
						archive.addArchiveObject(article);
					}
				}
			}
		} 
		catch (Exception e) {
			e.printStackTrace();
		}
		
		return this.archiveTransfert;
	}
 
	private static String getTagValue(String tag, Node node) {
		if (node != null && node.getNodeType() == Node.ELEMENT_NODE) {
			return getTagValue( tag, (Element)node);
		}
		else return "";
	}

	private static String getTagValue(String tag, Element element){
		String value = "";
		if(element != null){
			Node node = element.getElementsByTagName(tag).item(0);
			if(node != null && node.getChildNodes().item(0) != null){
				value = node.getChildNodes().item(0).getNodeValue();
			}
		}
		System.out.println("-- " + tag + " : '" + value +"'");
		return value;
	}
	
	private static Node getFirstNodeByTagName(String tag, Node nArchive){
		Element element = (Element) nArchive;
		return element.getElementsByTagName(tag).item(0);				
	}
        
}
