package vue;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.Observable;
import java.util.Observer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellRenderer;

import maarcharchivetransfer.MaarchArchiveTransfer;
import modele.AbstractArchive;
import modele.AbstractArchiveObject;
import modele.FilesTable;

import controleur.AbstractControler;
import controleur.DescriptiveFormControler;

import netscape.javascript.JSObject;

public class DescriptiveForm extends JApplet implements Observer {
	
	/**
	 * http://127.0.0.1/maarch_entreprise_trunk/modules/records_management/applet_controller.php 100 121
	 */
	private static final long serialVersionUID = 9124610021627832275L;

	//interface graphique
	private JTextField jtfBr 		= new JTextField();
	
	private JComboBox cbbEntite 	= new JComboBox();
	private JComboBox cbbService 	= new JComboBox();
	private JLabel lblDate 			= new JLabel(); 
	private JLabel lblResp 			= new JLabel();
	private JComboBox cbbRespVer 	= new JComboBox();
	private JComboBox cbbTypeEntree	= new JComboBox();
    private JComboBox cbbArt        = new JComboBox();

	private JLabel lblArt 			= new JLabel();
	private JLabel lblTypeArt 		= new JLabel();
	private JComboBox cbbType 		= new JComboBox();
	private JTextField jtfSerie 	= new JTextField();
	private JTextArea txtaDesc 		= new JTextArea("..." , 2, 50);
	private JComboBox cbbRule 		= new JComboBox();
	private JTextField jtfDua 		= new JTextField();
	private JTextField jtfDateB 	= new JTextField();
	private JTextField jtfDateE 	= new JTextField();
	
	private final JButton btnOpen 			= new JButton("Parcourir");
	private JButton btnOK 					= new JButton("Clôturer et envoyer à l'archiviste");
    private JButton btnClose 				= new JButton("Fermer");
	private final JProgressBar progressBar 	= new JProgressBar(0, 100);
	

	private JPanel panelFiles 		= new JPanel();
        private JTable files;	
        private FilesTable filesTable 	= new FilesTable();
        private JScrollPane spFiles 	= new JScrollPane(files);

	//controleur
	private AbstractControler controler = new DescriptiveFormControler();
	
	private MaarchArchiveTransfer maarchArchiveTransfer ; 

	//parameters
	private String url;
	private String archiveTransferId;
	private String agreementId;

	private AbstractArchive archive;

	
        /**
        * @param args the command line arguments
        */
        public static void main(String[] args) {
            try{
                System.out.println(args[0]);
                System.out.println(args[1]);
                System.out.println(args[2]);
                DescriptiveForm DescriptiveForm = new DescriptiveForm();
                DescriptiveForm.init();
            }
        catch(Exception e) {
            String exMessage = e.toString();
            System.out.println(exMessage);
        }
        }
        
	public void init(){  

        this.setSize(1080, 560);

        this.gui();

        System.out.println("----------BEGIN PARAMETERS----------");
        this.url = this.getParameter("url");
        this.archiveTransferId = this.getParameter("archiveTransferId");
        this.agreementId = this.getParameter("agreementId");        
        System.out.println("URL : " + this.url);
        System.out.println("ARCHIVE TRANSFER ID : " + this.archiveTransferId);
        System.out.println("AGREEMENT ID : " + this.agreementId);
        System.out.println("----------END PARAMETERS----------");

        try {
        	maarchArchiveTransfer = new MaarchArchiveTransfer(this.url, this.archiveTransferId, this.agreementId);
        	maarchArchiveTransfer.editArchiveTransfer();
        } catch (Exception ex) {
            Logger.getLogger(MaarchArchiveTransfer.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        controler.parser(maarchArchiveTransfer.getPathToXml());
        
        controler.setArchivalAgreement(maarchArchiveTransfer.getFormatsList(), maarchArchiveTransfer.getNbArticles(), maarchArchiveTransfer.getTransferSize());
        
        controler.addObserver(this);
		
        archive = controler.getArchive();
		
        this.fillDescriptiveForm();
		
		
	}


	/*
	private void initArchive() {
		// TODO Auto-generated method stub
		this.archive = new Archive();		
	}
	*/

	private String formatDate(String date){
		String annee = date.substring(0, 4);
		String mois = date.substring(5, 7);
		String jour = date.substring(8, 10);;
		return jour+"/"+mois+"/"+annee;
	}


	public void fillDescriptiveForm() {

		String txt = archive.getTransferringAgencyArchiveIdentifier();
		jtfBr.setText(txt);
		jtfBr.setToolTipText(txt);
		
		txt = archive.getOriginatingAgency();
		cbbEntite.removeAllItems();
		cbbEntite.addItem(txt); 
		cbbEntite.setToolTipText(txt);

		txt = archive.getOriginatingAgencyComment();
		cbbService.removeAllItems();
		cbbService.addItem(txt);
		cbbService.setToolTipText(txt);

		txt = formatDate(archive.getDate());
		lblDate.setText(txt); 	
		lblDate.setToolTipText(txt);

		txt = archive.getTransferringAgencyContact();
		lblResp.setText(txt);
		lblResp.setToolTipText(txt);

		txt = archive.getTransferringAgency();
		cbbRespVer.removeAllItems();
		cbbRespVer.addItem(txt);
		cbbRespVer.setToolTipText(txt);

		txt = archive.getType();
		cbbTypeEntree.removeAllItems();
		cbbTypeEntree.addItem(txt);
		cbbTypeEntree.setToolTipText(txt);

		
		AbstractArchiveObject article = archive.getCurrentArchiveObject();
		
		if(article != null){
			txt = "Article "+ article.getNumArchiveObject();
			lblArt.setText(txt);
			lblArt.setToolTipText(txt);

			txt = article.getFlag();
			btnOpen.setEnabled(article.getIsDigital());
			panelFiles.setEnabled(article.getIsDigital());
			lblTypeArt.setText(txt);
			lblTypeArt.setToolTipText(txt);

			txt = article.getArchiveType();
			cbbType.removeAllItems();
			cbbType.addItem(txt);
			cbbType.setToolTipText(txt);

			txt = article.getSerie();
			jtfSerie.setText(txt);
			jtfSerie.setToolTipText(txt);

			txt = article.getDescription();
			txtaDesc.setText(txt);
			txtaDesc.setToolTipText(txt);

			txt = article.getAppraisalRule();
			cbbRule.removeAllItems();
			cbbRule.addItem(txt);
			cbbRule.setToolTipText(txt);

			txt = "" + article.getDua() ;
			jtfDua.setText(txt);
			jtfDua.setToolTipText(txt);

			txt = formatDate(article.getDateB());
			jtfDateB.setText(txt); 
			jtfDateB.setToolTipText(txt);	

			txt = formatDate(article.getDateE());
			jtfDateE.setText(txt); 
			jtfDateE.setToolTipText(txt);
			

			filesTable.setData(article.getDocuments());
			filesTable.fireTableDataChanged();
			
			
		}	
			
	}



	@Override
	public void update(Observable o, Object arg) {
		if(arg instanceof AbstractArchiveObject){
			AbstractArchiveObject article = (AbstractArchiveObject) arg;
			if(article != null){
				String txt = "Article "+ article.getNumArchiveObject();
				lblArt.setText(txt);
				lblArt.setToolTipText(txt);

				txt = article.getFlag();
				btnOpen.setEnabled(article.getIsDigital());
				panelFiles.setEnabled(article.getIsDigital());
				lblTypeArt.setText(txt);
				lblTypeArt.setToolTipText(txt);

				txt = article.getArchiveType();
				cbbType.removeAllItems();
				cbbType.addItem(txt);
				cbbType.setToolTipText(txt);

				txt = article.getSerie();
				jtfSerie.setText(txt);
				jtfSerie.setToolTipText(txt);

				txt = article.getDescription();
				txtaDesc.setText(txt);
				txtaDesc.setToolTipText(txt);

				txt = article.getAppraisalRule();
				cbbRule.removeAllItems();
				cbbRule.addItem(txt);
				cbbRule.setToolTipText(txt);

				txt = "" + article.getDua() ;
				jtfDua.setText(txt);
				jtfDua.setToolTipText(txt);

				txt = formatDate(article.getDateB());
				jtfDateB.setText(txt); 
				jtfDateB.setToolTipText(txt);	

				txt = formatDate(article.getDateE());
				jtfDateE.setText(txt); 
				jtfDateE.setToolTipText(txt);
				

				filesTable.setData(article.getDocuments());
				filesTable.fireTableDataChanged();
				
			}
		}
	}
	
	
	
	public void gui(){

		this.setFont(new Font("TimesRoman",Font.PLAIN, 12));
		Font police 		= new Font("Arial", Font.BOLD, 12);
		Color lightblue 	= Color.getHSBColor((float)0.5, (float)0.7, (float)0.729);
		FlowLayout fl 		= new FlowLayout(FlowLayout.RIGHT, 5, 8);
		
		
		
		//*********************************************************
		//*         PANEL TRANSFER ID
		//*********************************************************
		JPanel panelTransferId = new JPanel();
		panelTransferId.setMaximumSize(new Dimension(500, 30));
		panelTransferId.setPreferredSize(new Dimension(500, 30));
		panelTransferId.setBackground(Color.WHITE);
		panelTransferId.setBorder(BorderFactory.createMatteBorder(0,0, 2, 0,lightblue));
		
		//label archive transfer id
		JLabel lblBr = new JLabel("Bordereau SEDA : ");
		lblBr.setPreferredSize(new Dimension(390, 20));
		lblBr.setHorizontalAlignment(JLabel.RIGHT);
		panelTransferId.add(lblBr);
		
		//text field archive transfer id
        jtfBr.setPreferredSize(new Dimension(80, 20));
		jtfBr.setHorizontalAlignment(JLabel.RIGHT);
        jtfBr.setFont(police);
        jtfBr.setEditable(false);
		panelTransferId.add(jtfBr);
		
		
				
		//*********************************************************
		//*         PANEL TRANSFER INFO
		//*********************************************************
		JPanel panelTransferInfo = new JPanel(fl);
		panelTransferInfo.setMaximumSize(new Dimension(500, 200));
		panelTransferInfo.setPreferredSize(new Dimension(500, 200));
		panelTransferInfo.setBackground(Color.WHITE);
		panelTransferInfo.setBorder(BorderFactory.createMatteBorder(0, 0, 2, 0, lightblue));
		
		//label entite
		JLabel lblEntite = new JLabel("Entité productrice");
		lblEntite.setHorizontalAlignment(JLabel.RIGHT);
		lblEntite.setPreferredSize(new Dimension(180, 20));
		panelTransferInfo.add(lblEntite);
		
		//combobox entite 
		cbbEntite.setPreferredSize(new Dimension(290, 20));
		cbbEntite.setEditable(false);
		panelTransferInfo.add(cbbEntite);
		
		//label service
		JLabel lblService = new JLabel("Service producteur");
		lblService.setHorizontalAlignment(JLabel.RIGHT);
		lblService.setPreferredSize(new Dimension(180, 20));
		panelTransferInfo.add(lblService);
		
		//combobox Service 
		cbbService.setPreferredSize(new Dimension(290, 20));
		cbbService.setEditable(false);
		panelTransferInfo.add(cbbService);
		
		//label Date de versement
		JLabel lblDateVer = new JLabel("Date de versement");
		lblDateVer.setPreferredSize(new Dimension(380, 20));
		lblDateVer.setHorizontalAlignment(JLabel.RIGHT);
		panelTransferInfo.add(lblDateVer);
		
		//label Date
		lblDate.setPreferredSize(new Dimension(90, 20));
		lblDate.setHorizontalAlignment(JLabel.RIGHT);
		panelTransferInfo.add(lblDate);
		
		//label Responsable du versement
		JLabel lblRespVer 	= new JLabel("Responsable du versement");
		lblRespVer.setHorizontalAlignment(JLabel.RIGHT);
		lblRespVer.setPreferredSize(new Dimension(240, 20));
		panelTransferInfo.add(lblRespVer);
		
		//label Identite du Responsable du versement
		lblResp.setPreferredSize(new Dimension(230, 20));
		lblResp.setHorizontalAlignment(JLabel.RIGHT);
		panelTransferInfo.add(lblResp);
		
		//combobox Responsable du versement	
		cbbRespVer.setPreferredSize(new Dimension(320, 20));
		cbbRespVer.setEditable(false);
		panelTransferInfo.add(cbbRespVer);
		
		//label Type entree
		JLabel lblTypeEnt = new JLabel("Type d'entrée");
        lblTypeEnt.setPreferredSize(new Dimension(290, 20));
		lblTypeEnt.setHorizontalAlignment(JLabel.RIGHT);
		panelTransferInfo.add(lblTypeEnt);
		
		//combobox Type entree	
		cbbTypeEntree.setPreferredSize(new Dimension(180, 20));
		cbbTypeEntree.setEditable(false);
		panelTransferInfo.add(cbbTypeEntree);
		
		
		
		//*********************************************************
		//*         PANEL ARCHIVE OBJECTS (= ARTICLES)
		//*********************************************************
		JPanel panelTransferedItems = new JPanel(fl);
		panelTransferedItems.setMaximumSize(new Dimension(500, 200));
		panelTransferedItems.setPreferredSize(new Dimension(500, 200));
		panelTransferedItems.setBackground(Color.WHITE);

		//label Article x/x-xxxxx
		lblArt.setPreferredSize(new Dimension(160, 20));
		lblArt.setHorizontalAlignment(JLabel.CENTER);
		panelTransferedItems.add(lblArt);
		
		//label Type Article
		lblTypeArt.setPreferredSize(new Dimension(220, 20));
		lblTypeArt.setHorizontalAlignment(JLabel.CENTER);
		panelTransferedItems.add(lblTypeArt);

		//button "<"
		final JButton btnArrowL = new JButton("<");
		panelTransferedItems.add(btnArrowL);
		btnArrowL.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
			    if (e.getSource() == btnArrowL) {
			        controler.getPreviousArchiveObject();
			    } 
			}
		});

		//button ">"
		final JButton btnArrowR = new JButton(">");
		panelTransferedItems.add(btnArrowR);
		btnArrowR.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
			    if (e.getSource() == btnArrowR) {
			        controler.getNextArchiveObject();
			    } 
			}
		});

		//label Type
		JLabel lblType = new JLabel("Type");
		lblType.setPreferredSize(new Dimension(250, 20));
		lblType.setHorizontalAlignment(JLabel.RIGHT);
		panelTransferedItems.add(lblType);

		//combobox type
		cbbType.setPreferredSize(new Dimension(220,20));
		cbbType.setEditable(false);
		panelTransferedItems.add(cbbType);

		//label Serie	
		JLabel lblSerie	= new JLabel("Série ou sous-série");
		lblSerie.setPreferredSize(new Dimension(250, 20));
		lblSerie.setHorizontalAlignment(JLabel.RIGHT);
		panelTransferedItems.add(lblSerie);

		//textfield Serie
        jtfSerie.setFont(police);
        jtfSerie.setPreferredSize(new Dimension(220, 20));
        jtfSerie.setEditable(false);
		panelTransferedItems.add(jtfSerie);

		//panel + textarea description
		txtaDesc.setFont(new Font("Serif", Font.ITALIC, 12));
		txtaDesc.setMargin(new Insets(3,3,3,3));
		txtaDesc.setLineWrap(true);
		txtaDesc.setWrapStyleWord(true);
		txtaDesc.setEditable(false);
		JScrollPane spDesc = new JScrollPane(txtaDesc);
        spDesc.setPreferredSize(new Dimension(475, 32));
        spDesc.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);//VERTICAL_SCROLLBAR_ALWAYS);
		panelTransferedItems.add(spDesc);

		//label Rule
		JLabel lblRule = new JLabel("Règle de gestion");
		lblRule.setPreferredSize(new Dimension(130, 20));
		lblRule.setHorizontalAlignment(JLabel.RIGHT); 
		panelTransferedItems.add(lblRule);

		//combobox Rule
		cbbRule.setPreferredSize(new Dimension(220,20));
		cbbRule.setEditable(false);
		panelTransferedItems.add(cbbRule);

		//label Dua
		JLabel lblDua = new JLabel("DUA");
		lblDua.setPreferredSize(new Dimension(70, 20));
		lblDua.setHorizontalAlignment(JLabel.RIGHT);
		panelTransferedItems.add(lblDua);

		//TextField Dua
        jtfDua.setFont(police);
        jtfDua.setPreferredSize(new Dimension(50, 20));
        jtfDua.setHorizontalAlignment(JLabel.RIGHT);
        jtfDua.setEditable(false);
		panelTransferedItems.add(jtfDua);

		//label Dates
		JLabel lblDates	= new JLabel("Dates extrêmes");
		lblDates.setPreferredSize(new Dimension(290, 20));
		lblDates.setHorizontalAlignment(JLabel.RIGHT);
		panelTransferedItems.add(lblDates);

		//TextField DateB
        jtfDateB.setFont(police);
        jtfDateB.setPreferredSize(new Dimension(90, 20));
        jtfDateB.setHorizontalAlignment(JLabel.RIGHT);
        jtfDateB.setEditable(false);
		panelTransferedItems.add(jtfDateB);

		//TextField DateE 
        jtfDateE.setFont(police);
        jtfDateE.setPreferredSize(new Dimension(90, 20));
        jtfDateE.setHorizontalAlignment(JLabel.RIGHT);
        jtfDateE.setEditable(false);
		panelTransferedItems.add(jtfDateE);

		
		
		//*********************************************************
		//*         PANEL ARCHIVE TRANSFER (= PARTIE GAUCHE)
		//*********************************************************
		JPanel panelArchiveTransfer = new JPanel();
		panelArchiveTransfer.setLayout(new BoxLayout(panelArchiveTransfer, BoxLayout.Y_AXIS));
		panelArchiveTransfer.setBorder(BorderFactory.createMatteBorder(0, 0, 0, 1, lightblue));
		panelArchiveTransfer.setPreferredSize(new Dimension(510, 430));
		panelArchiveTransfer.setBackground(Color.WHITE);

		panelArchiveTransfer.add(panelTransferId);
		panelArchiveTransfer.add(panelTransferInfo);
		panelArchiveTransfer.add(panelTransferedItems);
		
		
		
		//*********************************************************
		//*         PANEL INFO
		//*********************************************************
		JPanel panelInfo = new JPanel();
		panelInfo.setMaximumSize(new Dimension(500, 30));
		panelInfo.setPreferredSize(new Dimension(500, 30));
		panelInfo.setBackground(Color.WHITE);
		panelInfo.setBorder(BorderFactory.createMatteBorder(0, 0, 2, 0, lightblue));
		
		//label info
		JLabel lblFiles = new JLabel("Veuillez joindre les fichiers électroniques de cet article");
		lblFiles.setPreferredSize(new Dimension(470, 20));
		panelInfo.add(lblFiles);
		
		
		
		//*********************************************************
		//*         PANEL FILES
		//*********************************************************
		panelFiles.setBackground(Color.WHITE);
		panelFiles.setMaximumSize(new Dimension(500, 400));
		panelFiles.setPreferredSize(new Dimension(500, 300));
		
		//panel Files : listes des fichiers pour l article courant
		files = new JTable(filesTable) {    
		    /**
			 * 
			 */
			private static final long serialVersionUID = 8089625508940895972L;
			public String getToolTipText(MouseEvent e) {
		        String tip = null;
		        java.awt.Point p = e.getPoint();
		        int rowIndex = rowAtPoint(p);
		        int colIndex = columnAtPoint(p);
		        int realColumnIndex = convertColumnIndexToModel(colIndex);
	
		        if (realColumnIndex == 0 && rowIndex != -1) {
		            tip = ""+getValueAt(rowIndex, colIndex);
		        }
		        return tip;
		    }
		    public Component prepareRenderer(TableCellRenderer renderer,
	                int rowIndex, int vColIndex) {
		    	Component c = super.prepareRenderer(renderer, rowIndex, vColIndex);
		    	if (rowIndex % 2 == 0 && !isCellSelected(rowIndex, vColIndex)) {
		    		c.setBackground(Color.getHSBColor((float)0.22, (float)0.04, (float)0.97));
		    	} else {
		    		c.setBackground(getBackground());
		    	}
		    	return c;
		    }
		};
		files.setDefaultRenderer(JButton.class, new BtnCellRenderer());

		files.getColumn("Supp.").setCellEditor(new DeleteButtonEditor(new JCheckBox()));
		
		files.setFillsViewportHeight(true);
		files.getColumnModel().getColumn(0).setPreferredWidth(160);
		files.getColumnModel().getColumn(1).setPreferredWidth(160);
		files.getColumnModel().getColumn(2).setPreferredWidth(82);
		files.getColumnModel().getColumn(3).setPreferredWidth(70);
		files.getColumnModel().getColumn(4).setPreferredWidth(28);
		files.setRowHeight(28);
		files.setShowHorizontalLines(false);
		files.setShowVerticalLines(false);

		
		spFiles = new JScrollPane(files);
		spFiles.setPreferredSize(new Dimension(480, 280));  //(new Dimension(475, 280));
		spFiles.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		spFiles.setPreferredSize(new Dimension(475, 280));
		panelFiles.add(spFiles, BorderLayout.CENTER);
			
		
		
		//*********************************************************
		//*         PANEL BROWSE
		//*********************************************************
		JPanel panelBrowse = new JPanel();
		panelBrowse.setPreferredSize(new Dimension(510, 50));
		panelBrowse.setBackground(Color.WHITE);
		
		//textfield chemin fichier 
		final JTextField tflFile = new JTextField("Ajouter un document à l'archive...");
        tflFile.setEditable(false);
        tflFile.setPreferredSize(new Dimension(350, 25));
        tflFile.setFont(new Font("Serif", Font.ITALIC, 12));
		panelBrowse.add(tflFile);
		
		//btn parcourir
		final JFileChooser fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

		
		btnOpen.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
			    if (e.getSource() == btnOpen) {
			        int returnVal = fc.showOpenDialog(DescriptiveForm.this);
	
			        if (returnVal == JFileChooser.APPROVE_OPTION) {
			            java.io.File file = fc.getSelectedFile();
			            //ajout dans JTable files
			            controler.addDocument(file);
			            try {
							tflFile.setText(file.getCanonicalPath());
						} catch (IOException e1) {}
			        }
			    } 
			}
		});
		panelBrowse.add(btnOpen);

		
		
		//*********************************************************
		//*         PANEL CONTAINS (= PARTIE DROITE)
		//*********************************************************
		JPanel panelContains = new JPanel();
		panelContains.setLayout(new BoxLayout(panelContains, BoxLayout.Y_AXIS));
		panelContains.setPreferredSize(new Dimension(510, 430));
		panelContains.setBorder(BorderFactory.createMatteBorder(0, 1, 0, 0, lightblue));
		panelContains.setBackground(Color.WHITE);
		panelContains.add(panelInfo);
		panelContains.add(panelFiles);
		panelContains.add(panelBrowse);
		
		
		
		//*********************************************************
		//*         PANEL ARCHIVE (= ARCHIVE TRANSFER + CONTAINS)
		//*********************************************************
		JPanel panelArchive = new JPanel(new GridLayout(1,2));
		panelArchive.setPreferredSize(new Dimension(520, 430));
		panelArchive.setBorder(new EmptyBorder(10, 10, 10, 10) );
		panelArchive.setBackground(Color.WHITE);
		panelArchive.add(panelArchiveTransfer);
		panelArchive.add(panelContains);
		
		
		
		//*********************************************************
		//*         PANEL PROGRESS BAR
		//*********************************************************
		JPanel panelProgressBar = new JPanel();
		panelProgressBar.setBorder(new EmptyBorder(20, 0, 20, 0) );
		progressBar.setPreferredSize(new Dimension(300, 20));
		panelProgressBar.setBackground(Color.WHITE);
        panelProgressBar.add(progressBar);
		
		

		//*********************************************************
		//*         PANEL BOUTONS
		//*********************************************************
        JPanel panelBtn	= new JPanel();
		panelBtn.setBackground(Color.WHITE);
		
		//btn Clôturer et envoyer à l'archiviste
		btnOK.addActionListener(
			new ActionListener(){
				public void actionPerformed(ActionEvent e){
                    btnOK.setEnabled(false);
                    progressBar.setIndeterminate(true);
                    getContentPane().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                    controler.compression();
                    getContentPane().setCursor(null);
                    System.out.println("---------- COMPRESSION OK ----------");
                    try {
                        String urlToSend = maarchArchiveTransfer.url + "?action=updateStatus&archiveTransferId=" + maarchArchiveTransfer.archiveTransferId + "&status=OK";
                        maarchArchiveTransfer.sendHttpRequest(urlToSend, "none");
                        maarchArchiveTransfer.processReturn(maarchArchiveTransfer.messageResult);
                        progressBar.setIndeterminate(false);
                    } catch(Exception ex) {
                        String exMessage = ex.toString();
                        System.out.println(exMessage);
                    }
                    try {
                        closeWindow("1");
                    } catch (InterruptedException ex) {
                        String exMessage = ex.toString();
                        System.out.println(exMessage);
                    }
                    
                }
			}
		);
		panelBtn.add(btnOK);
		
             
		//btn Annuler
		JButton btnCancel = new JButton("Annuler");
		btnCancel.addActionListener(
			new ActionListener(){
				public void actionPerformed(ActionEvent e){
		            try {
                        closeWindow("0");
                    } catch (InterruptedException ex) {
                        String exMessage = ex.toString();
                        System.out.println(exMessage);
                    }        
				}

			}
		);
		panelBtn.add(btnCancel);
		

		
		//*********************************************************
		//*         
		//*********************************************************
		this.add(panelArchive, BorderLayout.NORTH);
		this.add(panelProgressBar, BorderLayout.CENTER);
		this.add(panelBtn, BorderLayout.SOUTH);	
		
	}
	
    public void closeWindow(String showBR) throws InterruptedException
    {
        JSObject jso;
        jso = JSObject.getWindow(this);
        jso.call("ArchiveTransferApplet__close", new String[] {String.valueOf(showBR)});
        stop();
        destroy();
	    System.exit(0);
    }

	

}
