package zip;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import maarcharchivetransfer.fileManager;
import maarcharchivetransfer.myLogger;
import modele.MyFile;

public class AppZip {


	private List<MyFile> fileList;
	private String userLocalDirTmp;    //      /maarchtmp/archivetransfers/numarchive/documents/
	private String archiveTransferDir; //      /maarchtmp/archivetransfers/
	private String archiveDir;         //      /maarchtmp/archivetransfers/numarchive/
        private String currentXmlFile;
	private static long minSizeArchiveTransferDir = 100;
	private myLogger logger = new myLogger(System.getProperty("user.home")+java.io.File.separator+"maarchtmp"+java.io.File.separator+"archivetransfers"+java.io.File.separator);
    
	
	public AppZip(){
		fileList = new ArrayList<MyFile>();
	}
	
	public AppZip(List<MyFile> fileList ){
		this.fileList = fileList;
	}
	
	public void setFileList(List<MyFile> list ){
		this.fileList = list;
	}
	
	public void addFile(MyFile file){
		if (fileList == null){
			fileList = new ArrayList<MyFile>();
		}
		fileList.add(file);
	}
	
    public void zipIt(String archiveObjectId , String transferringAgencyArchiveId ) throws Exception{
    	 
        byte[] buffer = new byte[1024];
        archiveObjectId = archiveObjectId.toLowerCase();
        String zipFile = archiveObjectId +".zip";
    	transferringAgencyArchiveId = transferringAgencyArchiveId.toLowerCase();
    	
        try{
        	createDirIfNotExist(transferringAgencyArchiveId);
        	java.io.File dirFile = new java.io.File(this.archiveTransferDir);
        	if(dirFile.getUsableSpace() < AppZip.minSizeArchiveTransferDir*1024*1024){
        		logger.log("NOT ENOUGH FREE DISK SPACE IN TEMP DIRECTORY "+ this.archiveTransferDir, Level.INFO);
        		throw new Exception("Il n'y a pas assez d'espace disque libre dans le répertoire "+this.archiveTransferDir+" (minimum de "+AppZip.minSizeArchiveTransferDir+" Mo)");
        	}
	       	FileOutputStream fos = new FileOutputStream(this.userLocalDirTmp + zipFile);
	       	ZipOutputStream zos = new ZipOutputStream(fos);
	    
	       	System.out.println("----------COMPRESSING FILE : " + zipFile +"----------");
	       	logger.log("----------COMPRESSING FILE : " + zipFile +"----------", Level.INFO);
	       	int seq = 1;
	       	
	       	for(MyFile file : this.fileList){ 
	       		                  
		   		ZipEntry ze= new ZipEntry(String.format("_%03d_", seq) + file.getName());
		   		seq++;
		   		
		   		//verifier si fichiers ont des noms differents
		       	zos.putNextEntry(ze);
		
		       	FileInputStream in = new FileInputStream(file.getPath());
		
		       	int len;
		       	while ((len = in.read(buffer)) > 0) {
		       		zos.write(buffer, 0, len);
		       	}
		       	in.close();
		   		System.out.println("          FILE : " + file.getName() + " ADDED");
		       	logger.log("          FILE : " + file.getName() + " ADDED", Level.INFO);
	       	}
	       	
	       	zos.closeEntry();
	       	zos.close();
	    
	       	System.out.println("----------COMPRESSION COMPLETED----------");
	       	logger.log("----------COMPRESSION COMPLETED----------", Level.INFO);

	        fileManager fM = new fileManager();
	        String encodedXML = fM.encodeFile(this.userLocalDirTmp + zipFile);
	        long size = new java.io.File(this.userLocalDirTmp + zipFile).length()*8; // taille en bits
	       	completeXmlFile(transferringAgencyArchiveId, archiveObjectId, ""+size, encodedXML);
        }
        catch(IOException ex){
        	logger.log("----------XML TEMP FILE header.xml NOT FOUND----------", Level.SEVERE);
        	ex.printStackTrace();   
        }
        catch(SAXException ex){
        	logger.log("----------PARSING ERROR----------\n          "+ex.getMessage(), Level.SEVERE);
        	ex.printStackTrace();  
        }
        catch(ParserConfigurationException ex){
        	logger.log("----------PARSING ERROR----------", Level.SEVERE);
        	ex.printStackTrace();  
        }
        catch(TransformerException ex){
        	logger.log("----------ERROR WHILE SAVING XML FILE "+this.archiveDir+transferringAgencyArchiveId+".xml ----------", Level.INFO);
        	ex.printStackTrace();  
        }
  	}
    
    
   
	private void completeXmlFile(String archiveId, String archiveObjectId, String sizeZip, String encodedXML) throws SAXException, IOException, ParserConfigurationException, TransformerException{
		
		System.out.println("----------BEGIN FILLING XML FILE "+this.archiveDir+archiveId+".xml----------");
		logger.log("----------FILLING XML FILE "+this.archiveDir+archiveId+".xml----------", Level.INFO);
    	
                String filepath;
                if (this.currentXmlFile == null) {
                    this.currentXmlFile = this.archiveTransferDir + "headertmp.xml";
                } else {
                    this.currentXmlFile = this.archiveDir + archiveId + ".xml";
                }
                filepath = this.currentXmlFile;
                
		DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
		Document doc = docBuilder.parse(filepath);
		//doc.getDocumentElement().normalize();
		
		Element archiveTransfer = doc.getDocumentElement();
		
		//archive
		NodeList nlContains = archiveTransfer.getElementsByTagName("ArchiveObject");
		Node nArticle ; 
		int i = 0; //le premier contains est celui pour l'archive les suivants ceux des articles
		String transferringAgencyObjectId = "";
		Node nAttachment = null;
		Node nSize = null;
		while(i<nlContains.getLength() && (!transferringAgencyObjectId.equals(archiveObjectId))){
			nArticle = nlContains.item(i);
			if(nArticle.getNodeType() == Node.ELEMENT_NODE){
				transferringAgencyObjectId = ((Element)nArticle).getElementsByTagName("TransferringAgencyObjectIdentifier").item(0).getChildNodes().item(0).getNodeValue();
				transferringAgencyObjectId = transferringAgencyObjectId.toLowerCase();
				if(transferringAgencyObjectId.equals(archiveObjectId)){
					nAttachment = ((Element)nArticle).getElementsByTagName("Attachment").item(0);
					nSize = ((Element)nArticle).getElementsByTagName("Size").item(0);
				}
			}
			i++;
		}

		if((nAttachment != null) && (nAttachment.getNodeType() == Node.ELEMENT_NODE)){
			((Element)nAttachment).setAttribute("uri", "documents"+java.io.File.separator+archiveObjectId+".zip");
			((Element)nAttachment).setAttribute("filename", archiveObjectId+".zip");
			((Element)nAttachment).setTextContent(encodedXML);
		}
		if((nSize != null) && (nSize.getNodeType() == Node.ELEMENT_NODE)){
			((Element)nSize).setTextContent(sizeZip);
		}	
		
		// write the content into xml file
		TransformerFactory transformerFactory = TransformerFactory.newInstance();
		Transformer transformer = transformerFactory.newTransformer();
		DOMSource source = new DOMSource(doc);
		StreamResult result = new StreamResult(new java.io.File( this.archiveDir+archiveId+".xml"));
		transformer.transform(source, result);
		
    	System.out.println("----------END FILLING XML FILE "+this.archiveDir+archiveId+".xml----------");
		logger.log("----------XML FILE "+this.archiveDir+archiveId+".xml FILLED----------", Level.INFO);
	}

	public void completeXmlFileWithoutElectronicItem(String archiveId, String archiveObjectId) throws SAXException, IOException, ParserConfigurationException, TransformerException, Exception{
		
		System.out.println("----------BEGIN FILLING XML FILE "+this.archiveDir+archiveId+".xml----------");
		logger.log("----------1FILLING XML FILE "+this.archiveDir+archiveId+".xml----------", Level.INFO);
                try {
                    createDirIfNotExist(archiveId);
                } catch(IOException ex){
                    logger.log("----------CANNOT WRITE DIR----------", Level.SEVERE);
                    ex.printStackTrace(); 
                } 
                String filepath;
                if (this.currentXmlFile == null) {
                    this.currentXmlFile = this.archiveTransferDir + "headertmp.xml";
                } else {
                    this.currentXmlFile = this.archiveDir + archiveId + ".xml";
                }
                filepath = this.currentXmlFile;
                logger.log("----------2FILLING XML FILE "+this.archiveDir+archiveId+".xml----------", Level.INFO);
		DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
		Document doc = docBuilder.parse(filepath);
		//doc.getDocumentElement().normalize();
		
		Element archiveTransfer = doc.getDocumentElement();
		
		//archive
		NodeList nlContains = archiveTransfer.getElementsByTagName("ArchiveObject");
		Node nArticle ; 
		int i = 0; //le premier contains est celui pour l'archive les suivants ceux des articles
		String transferringAgencyObjectId = "";
		Node nAttachment = null;
		Node nSize = null;
		while(i<nlContains.getLength() && (!transferringAgencyObjectId.equals(archiveObjectId))){
			nArticle = nlContains.item(i);
			if(nArticle.getNodeType() == Node.ELEMENT_NODE){
				transferringAgencyObjectId = ((Element)nArticle).getElementsByTagName("TransferringAgencyObjectIdentifier").item(0).getChildNodes().item(0).getNodeValue();
				transferringAgencyObjectId = transferringAgencyObjectId.toLowerCase();
				if(transferringAgencyObjectId.equals(archiveObjectId)){
					nAttachment = ((Element)nArticle).getElementsByTagName("Attachment").item(0);
					nSize = ((Element)nArticle).getElementsByTagName("Size").item(0);
				}
			}
			i++;
		}

		if((nAttachment != null) && (nAttachment.getNodeType() == Node.ELEMENT_NODE)){
			((Element)nAttachment).setAttribute("uri", "documents"+java.io.File.separator+archiveObjectId+".zip");
			((Element)nAttachment).setAttribute("filename", archiveObjectId+".zip");
		}	
		
		// write the content into xml file
		TransformerFactory transformerFactory = TransformerFactory.newInstance();
		Transformer transformer = transformerFactory.newTransformer();
		DOMSource source = new DOMSource(doc);
		StreamResult result = new StreamResult(new java.io.File( this.archiveDir+archiveId+".xml"));
		transformer.transform(source, result);
		
                System.out.println("----------END FILLING XML FILE "+this.archiveDir+archiveId+".xml----------");
		logger.log("----------XML FILE "+this.archiveDir+archiveId+".xml FILLED----------", Level.INFO);
	}
	
	private void createDirIfNotExist(String transferringAgencyArchiveId) throws Exception {
    	System.out.println("----------BEGIN LOCAL DIR TMP IF NOT EXISTS----------");
        String os = System.getProperty("os.name").toLowerCase();
        boolean isUnix = os.indexOf("nix") >= 0 || os.indexOf("nux") >= 0;
        boolean isWindows = os.indexOf("win") >= 0;
        boolean isMac = os.indexOf("mac") >= 0;
        this.userLocalDirTmp = System.getProperty("user.home");
        fileManager fM = new fileManager();
        if (isWindows || isUnix || isMac) {
            this.userLocalDirTmp = this.userLocalDirTmp + java.io.File.separator+"maarchtmp";
            fM.createUserLocalDirTmp(this.userLocalDirTmp);
            this.userLocalDirTmp = this.userLocalDirTmp + java.io.File.separator+"archivetransfers"+ java.io.File.separator;
            
            fM.createUserLocalDirTmp(this.userLocalDirTmp);
            this.archiveTransferDir = this.userLocalDirTmp;
            this.userLocalDirTmp = this.userLocalDirTmp  + transferringAgencyArchiveId + java.io.File.separator; 
            fM.createUserLocalDirTmp(this.userLocalDirTmp);
            this.archiveDir = this.userLocalDirTmp;
            this.userLocalDirTmp = this.userLocalDirTmp + "documents"+ java.io.File.separator;             
        } else {
            System.out.println("Your OS is not supported!!");
            //Prévoir de sortir de l'application
        }
        fM.createUserLocalDirTmp(this.userLocalDirTmp);
        System.out.println("----------END LOCAL DIR TMP IF NOT EXISTS----------");
    	System.out.println("----------BEGIN ADDING XML FILE IN ARCHIVE----------");
    	System.out.println("----------END ADDING XML FILE IN ARCHIVE----------");

    }
    
    
}
