<?php

require_once 'core/tests/class/DataObjectController.php';
require_once 'core/tests/class/ViewController.php';

//*****************************************************************************
// 1 - Init
//*****************************************************************************
// Load returned View as source of data
$ViewController = new ViewController();
$view = $ViewController->loadHTML($_REQUEST['rm_entities']);

$messageController = new MessageController();
$messageController->loadMessageFile(
    'modules/records_management/lang/rm_entities.xml'
);

// Load Schema
$dataObjectController = new DataObjectController();
$dataObjectController->loadXSD(
    $_SESSION['config']['corepath'] 
    . 'modules/records_management/schema/records_management.xsd'
);

// Load data object from session
$rm_entity = $dataObjectController->loadXML($_SESSION['rm_entities']);

//*****************************************************************************
// 2 - Update DataObject
//*****************************************************************************
$log = array();
$log[] = "Update RM_ENTITY...";
$rm_entities_properties = $dataObjectController->getObjectProperties('rm_entities');
foreach($rm_entities_properties as $property) {
    $propertyName = $property->getName();
    $formElement = $ViewController->getElementById(
        $propertyName
    );

    if(!$formElement) continue;
    
    switch($propertyName) {
    case 'rm_entity_id':
        break;
        
    case 'latest_date':
    case 'oldest_date':
        $input_date = $formElement->getAttribute('value');
        if($input_date != '') {
            $input_date = substr($input_date, 6, 4) . "-" . substr($input_date, 3, 2) . "-" . substr($input_date, 0, 2);
            $rm_entity->$propertyName = $input_date;
            $log[] = $propertyName . " => date " . $input_date;
        } else {
            $log[] = $propertyName . " => null";
            $rm_entity->$propertyName = null;
        }
        break;
        
    default:
        $log[] = $propertyName . " => " . utf8_decode($formElement->getAttribute('value'));
        $rm_entity->$propertyName = utf8_decode($formElement->getAttribute('value'));
    }
    
}

//*****************************************************************************
// 3 - Validate / save and return
//*****************************************************************************
$return = new StdClass();
$return->status = 1;
$return->log = $log;
$return->errors = array();

//*************************************
// 3.1 - Validate
//*************************************
$valid = $dataObjectController->validate($rm_entity);
if(!$valid) {
    $return->status = 0;
    $validationErrors = $rm_entity->getValidationErrors();
    for($i=0; $i<count($validationErrors); $i++) {
        $validationError = $validationErrors[$i];
        $returnError = $validationError->message;
        switch($validationError->id){
        case '1824':
            $parts = explode("'", $validationError->message);
            $element = $parts[1];
            if(strstr($validationError->message, 'attribute')) {
                $attribute = $parts[3];
                $value = $parts[5];
                $type = $parts[7];
                $propertyDesc = $messageController->getMessageText($element . "." .$attribute);
                if($value == '') {
                    $returnError = "La valeur de '$propertyDesc' est obligatoire.";
                } else {
                    $returnError = "La valeur '$value' n'est pas autorisée pour '$propertyDesc'.";
                }
            } else {
                $value = $parts[3];
                $type = $parts[5];
                $propertyDesc = $messageController->getMessageText('rm_archives.' . $element);
                if($value == '') {
                    $returnError = "La valeur de '$propertyDesc' est obligatoire.";
                } else {
                    $returnError = "La valeur '$value' n'est pas autorisée pour '$propertyDesc'.";
                } 
            }
        }
        $return->errors[] = $returnError;
    }   
    
    echo json_encode($return); 
    return;
} // End if ntot valid

if( $rm_entity->latest_date != '' && $rm_entity->oldest_date =! ''
    && $rm_entity->oldest_date >= $rm_entity->latest_date) {
    $return->status = 0;
    $return->errors[] = "La date de début d'existence doit être antérieure à la date de fin.";
    echo json_encode($return); 
    return;
}

//*************************************
// 3.2 - Save
//*************************************
$key = $dataObjectController->save($rm_entity);
if(!$key) {
    $return->status = 0;
    $return->errors[] = "Erreur lors de la sauvegarde des données";
    echo json_encode($return); 
    return;
}
$_SESSION['rm_entities'] = $rm_entity->asXML();
//*************************************
// 3.4 - Return
//*************************************  
echo json_encode($return); 