<?php

/*
*   Copyright 2008-2012 Maarch
*
*   This file is part of Maarch Framework.
*
*   Maarch Framework is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Maarch Framework is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @brief Contains the entity controler page
*
*
* @file
* @author Arnaud Veber
* @author Laurent Giovannoni
* @date $date$
* @version $Revision$
* @ingroup entities
*/

//SPECIFIC PROCESS
//columns to show in the list
$showCols = array(
    'entity_id'         => array(
        'functionFormat' => '',
        'cssStyle'       => '',
    ),
    'is_archival_agency'         => array(
        'functionFormat' => 'isBoolean',
        'cssStyle'       => 'text-align: center;',
    ),
    'is_originating_agency'         => array(
        'functionFormat' => 'isBoolean',
        'cssStyle'       => 'text-align: center;',
    ),
    'is_transferring_agency'         => array(
        'functionFormat' => 'isBoolean',
        'cssStyle'       => 'text-align: center;',
    ),
    'is_control_authority'         => array(
        'functionFormat' => 'isBoolean',
        'cssStyle'       => 'text-align: center;',
    ),
    'is_repository'         => array(
        'functionFormat' => 'isBoolean',
        'cssStyle'       => 'text-align: center;',
    ),
);
if (isset($_REQUEST['orderField'])) {
    $showCols[$_REQUEST['orderField']]['cssStyle'] 
        .= 'background-image: url(static.php?filename=black_0.1.png); ';
}

//actions to show in the list
$actions = array(
    'create', 
    'read', 
    'update',
);

include_once 'apps/' . $_SESSION['config']['app_id'] . '/admin/' 
    . 'admin_standard_page_controller.php';

require_once('core/class/class_functions.php');
require_once('core/class/class_db.php');

function populate($dataObjectController, $viewController, $rm_entity, $mode)
{
    $view = $viewController->view;
    $db = new dbquery();
    $db->connect();
    /* --------
    - entity_id
    -------- */
    $entity_id = $viewController->getElementById('entity_id');
    $query = 'SELECT ';
        $query .= 'entity_id, ';
        $query .= 'entity_label, ';
        $query .= 'short_label, ';
        $query .= 'business_id ';
    $query .= 'FROM ';
        $query .= 'entities ';
    $query .= 'WHERE ';
        if ($mode == 'create') {
            $query .= "entity_id NOT IN (";
                $query .= 'SELECT ';
                    $query .= 'DISTINCT entity_id ';
                $query .= 'FROM ';
                    $query .= 'rm_entities';
            $query .= ")";
        } else {
            $query .= "entity_id = '" . $rm_entity->entity_id . "'";
        }
        $query .= " AND ";
            $query .= "business_id <> ''";
    $db->query($query);
	//$db->show();
    $i_max = $db->nb_result();
    if ($i_max > 1)
        $entity_id->addOption('', '');
    while($entity = $db->fetch_object()) {
        $option = $entity_id->addOption($entity->entity_id, $entity->entity_id);
        $option->setDataAttribute('label', $entity->entity_label);
        $option->setDataAttribute('business', $entity->business_id);
    }
    if ($i_max == 1)
        $option->setAttribute('selected', 'selected');   
           
    /* ------
    - status (5.4.4)
    ------ */
    $status = $viewController->getElementById('status');
    $statuses = $dataObjectController->getTypeEnumeration('status');
    $i_max = count($statuses);
    if ($i_max > 1)
        $status->addOption('', '');
    for($i=0; $i<$i_max; $i++) {
        $stat = $statuses[$i];
        $option = 
            $status->addOption(
                $stat->getAttribute('value'), 
                $stat->getElementsByTagName('name')->item(0)->nodeValue
            );
    }
    
    /* ------
    - detail_level (5.4.5)
    ------ */
    $detail_level = $viewController->getElementById('detail_level');
    $detail_levels = $dataObjectController->getTypeEnumeration('detail_level');
    $i_max = count($detail_levels);
    if ($i_max > 1)
        $detail_level->addOption('', '');
    for($i=0; $i<$i_max; $i++) {
        $detail = $detail_levels[$i];
        $option = $detail_level->addOption(
            $detail->getAttribute('value'), 
            $detail->getElementsByTagName('name')->item(0)->nodeValue
        );
    }
        
    /* ------
    - description_language (5.4.7)
    ------ */
    $sedaParser = new DataObjectController;
    $description_language = $viewController->getElementById('description_language');
    $languages = $dataObjectController->getTypeEnumeration('description_language');
    
    $i_max = $languages->length;
    if ($i_max > 1)
        $description_language->addOption('', '');
    for($i=0; $i<$i_max; $i++) {
        $language = $languages->item($i);
        $option = 
            $description_language->addOption(
                $language->getAttribute('value'), 
                $language->getElementsByTagName('name')->item(0)->nodeValue
            );
    }
}

/* ------------------------------------
- retrieve dataObject in $rm_agreements
------------------------------------- */
$rm_entity = $dataObject;

/* ----------------
- title of the page
---------------- */
$titleImageSource = $_SESSION['config']['businessappurl'].'static.php?filename=favicon.png';

if ($params['mode'] == 'create' || $params['mode'] == 'read' || $params['mode'] == 'update') {
    $viewController->loadHTMLFile('modules/records_management/html/rm_entities.html');
    $viewController->translate($messageController);
    populate($dataObjectController, $viewController, $rm_entity, $params['mode']);
    $viewController->loadDataObject($rm_entity);
    $_SESSION['rm_entities'] = $rm_entity->asXML();
}
if($params['mode'] == 'read') {
    $viewController->disableInputs();
    $viewController->getElementById('rm_entities_cancel')->enable();
}

switch($params['mode']) {
    case 'list' :
        $titleText = $messageController->getMessageText(
            'rm_entities_list', 
            false, 
            array(
                $objectList->length
            )
        );
        $str_returnShow = $listContent;
        break;
        
    case 'create' :
        $titleText = $messageController->getMessageText('rm_entities_create');
        break;
        
    case 'read' :
        $titleText = $messageController->getMessageText('rm_entities_read');
        break;
        
    case 'update' :
        $titleText = $messageController->getMessageText('rm_entities_update');
        break;
}

?>
<h1>
    <img src="<?php echo $titleImageSource; ?>" />
    <?php echo $titleText; ?>
</h1>
<div class="<?php echo $params['objectName'] ?>">
    <div id="returnAjax"><br /><br /></div>
    <?php
    if ($params['mode'] == 'create' || $params['mode'] == 'read' || $params['mode'] == 'update') {
        /* --------
        - show view
        -------- */
        $viewController->showView();
    } else {
        if (isset($str_returnShow) && !empty($str_returnShow))
            echo $str_returnShow;
    }
    ?>
</div>
<?php if ($str_defaultJs) echo $str_defaultJs; ?>
