<?xml version="1.0" encoding="UTF-8" ?>
<!-- *********************************************************************** -->
<!--                                                                         -->
<!--  SEDA V1.0 XSLT DISPLAY HTML                                            -->
<!--                                                                         -->
<!-- *********************************************************************** -->

<xsl:stylesheet 
  version="1.0" 
  xmlns="http://www.w3.org/1999/xhtml" 
  xmlns:seda="fr:gouv:culture:archivesdefrance:seda:v1.0" 
  xmlns:das="maarch:dataAccessService" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
  xmlns:ccts="urn:un:unece:uncefact:documentation:standard:CoreComponentsTechnicalSpecification:2"
>
  <xsl:output method="html" indent="yes" media-type="text/html" encoding="UTF-8"/>
  <xsl:param name="afficheDocument" select="defaultstring"/>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  ROOT ELEMENTS                                                      -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template match="/">
    <html>
      <head>
        <title>
                    Bordereau SEDA (<xsl:value-of select="seda:ArchiveTransfer/seda:TransferIdentifier"/>)
        </title>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
      </head>
      <body>
        <div id="div_actions">
          <input type="button" value="Imprimer" onClick="printBR();"/>
          <xsl:choose>
            <xsl:when test=".//seda:Archive">
              <input type="button" value="Dérouler les Articles" onClick="removeDisplay();"/>
            </xsl:when>
          </xsl:choose>
        </div>
        <div align="center">
          <table id="printArea" class="wrapper" width="1000px" cellspacing="0" cellpadding="10">
            <div id="div_actions_placeHolder"/>
            <tr>
              <td width="100%">
                <xsl:apply-templates select="./*" />
              </td>
            </tr>
          </table>
        </div>
        <style>
                    @media screen {
                        /* tags */
                            body, h1, h2, h3, h4, h5, p, blockquote, input{margin: 0;padding: 0;}
                            body {font-family: sans-serif;background-color: #F2F2F2;}
                            input[type='button'] {
                                background-color: #1C99C5;
                                color: white;
                                border: 1px solid black;
                                border-radius: 5px;
                                padding: 5px;
                                padding-left: 10px;
                                padding-right: 10px;
                                font-weight: bold;
                                font-size: larger;
                                cursor: pointer;
                                margin-right: 15px;
                            }
                            input[type='button']:hover {background-color: #FFDC1F;color: black;}
                        /* id */
                            #div_actions {
                                position: fixed;
                                width: 100%;
                                background-color: rgba(0, 0, 0, 0.7);
                                text-align: right;
                                padding-top: 10px;
                                padding-bottom: 10px;
                                border-bottom: solid 1px black;
                            }
                        /* classes */
                            .wrapper {background-color: #D1E3F5; border: dashed 1px #A8B1B9;}
                            .rm_io {background-color: #E4EEF8; border: dashed 1px #A8B1B9;}
                            .rm_archive {background-color: #E4EEF8; border: dashed 1px #A8B1B9;}
                            .rm_archiveObject {background-color: #E4EEF8; border: dashed 1px #A8B1B9;}
                    }

                    @media print {
                        /* tags */
                            body, h1, h2, h3, h4, h5, p, blockquote, input{margin: 0;padding: 0;}
                            body {font-family: sans-serif;}
                            td {border: dashed 1px #E8F1F9;}
                        /* classes */
                            .wrapper {border: dashed 1px #A8B1B9;}
                            .rm_io {border: dashed 1px #A8B1B9;}
                            .rm_archive {border: dashed 1px #A8B1B9;}
                            .rm_archiveObject {border: dashed 1px #A8B1B9;}
                    }
        </style>
        <script language="JavaScript">
          <![CDATA[
                        /*******
                        * onLoad
                        *******/
                            
                            fix_divActions();
                            resizeImage();
                        
                        /**********
                        * functions
                        **********/
                            
                            function fix_divActions()
                            {
                                var div_actions = document.getElementById('div_actions');
                                var div_actions_placeHolder = document.getElementById('div_actions_placeHolder');
                                
                                var height_div_actions = div_actions.offsetHeight;
                                div_actions_placeHolder.style.height = height_div_actions + 'px';
                            }
                            
                            function resizeImage()
                            {
                                var td_logo  = document.getElementById('td_logo');
                                var img_logo = document.getElementById('img_logo');
                                
                                var td_height = td_logo.offsetHeight;
                                
                                img_logo.setAttribute('height', td_height + 'px');
                                img_logo.removeAttribute('style');
                            }
                            
                            function printBR()
                            {
                                if (!confirm('Imprimer ?'))
                                    return;
                                
                                removeDisplay();
                                var style = document.getElementsByTagName('style')[0].outerHTML;
                                var printArea = document.getElementById('printArea').outerHTML;
                                
                                win = window.open();
                                self.focus();
                                win.document.open();
                                win.document.title = "Bordereau SEDA";
                                win.document.write(printArea);
                                win.document.write(style);
                                win.document.close();
                                win.print();
                                win.close();
                            }
                            
                            function toggleDiv(id)
                            {
                                if (!id)
                                    return;
                                
                                element = id;
                                
                                offsetHeight = element.offsetHeight;
                                
                                if (offsetHeight != 0)
                                    element.setAttribute('style', 'display: none;');
                                else
                                    element.setAttribute('style', 'display: block;');
                            }
                            
                            function removeDisplay()
                            {
                                var tables = document.getElementsByTagName('table');
                                
                                i_max = tables.length;
                                
                                for (i=0; i!=(i_max-1); i++) {
                                    table = tables[i];
                                    if (table.getAttribute('style'))
                                        table.removeAttribute('style');
                                }
                            }
                    ]]>
        </script>
      </body>
    </html>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  seda:ArchiveTransfer                                               -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template match="seda:ArchiveTransfer">
    <table width="100%" class="rm_io" cellspacing="0" cellpadding="5">
      <tr>
        <td width="50%" valign="top" align="left">
          <h1>Transfert d'archives</h1>
          <h2>
            No: <xsl:value-of select="seda:TransferIdentifier"/>
          </h2>
          <br/>
          <h3>
            Date: <xsl:apply-templates select="seda:Date"/>
          </h3>
        </td>
        <td width="50%" valign="top" align="right" id="td_logo">
          <img src="../../modules/records_management/img/logo.png" style="display: none;" id="img_logo"/>
        </td>
      </tr>
      <tr>
        <td width="50%" valign="top" align="left">
          <xsl:apply-templates select="seda:TransferringAgency"/>
        </td>
        <td width="50%" valign="top" align="left">
          <xsl:apply-templates select="seda:ArchivalAgency"/>
        </td>
      </tr>
      <tr>
        <td colspan="2">
          <xsl:value-of select="seda:Comment"/>
        </td>
      </tr>
    </table>
    <br />
    <xsl:apply-templates select="seda:Archive"/>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  seda:ArchiveTransferReply                                          -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template match="seda:ArchiveTransferReply">
    <table width="100%" class="rm_io" cellspacing="0" cellpadding="5">
      <tr>
        <td width="50%" valign="top" align="left">
          <h1>Reponse au Transfert d'archives</h1>
          <h2>
              No: <xsl:value-of select="seda:TransferReplyIdentifier"/>
          </h2>
          <br/>
          <h3>
            Date: <xsl:apply-templates select="seda:Date"/>
          </h3>
        </td>
        <td width="50%" valign="top" align="right" id="td_logo">
          <img src="../../modules/records_management/img/logo.png" style="display: none;" id="img_logo"/>
        </td>
      </tr>
      <tr>
        <td width="50%" valign="top" align="left">
          <xsl:apply-templates select="seda:TransferringAgency"/>
        </td>
        <td width="50%" valign="top" align="left">
          <xsl:apply-templates select="seda:ArchivalAgency"/>
        </td>
      </tr>
    </table>
    <table width="100%" class="rm_io" cellspacing="0" cellpadding="5">
      <tr>
        <td valign="top" align="left">
          <h2>Réponse au transfert No <xsl:value-of select="seda:TransferIdentifier"/></h2>
        </td>
      </tr>
      <tr>
        <td valign="top" align="left" >
          <h3>Code retour <xsl:value-of select="seda:TransferIdentifier"/>
          <text> : </text><xsl:apply-templates select="seda:ReplyCode"/></h3>
        </td>
      </tr>
      <tr>
        <td valign="top" align="left">
          <xsl:value-of select="seda:Comment"/>
        </td>
      </tr>
    </table>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  Archive                                                            -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template match="seda:Archive">
    <blockquote style="margin-right: 0; margin-left: 25px;">
      <table width="100%" class="rm_archive" cellspacing="0" cellpadding="5">
        <tr>
          <td width="50%" valign="top" align="left">
            <table width="100%" cellspacing="0" cellpadding="2">
              <xsl:apply-templates select="seda:TransferringAgencyArchiveIdentifier"/>
              <xsl:apply-templates select="seda:ArchivalAgencyArchiveIdentifier"/>
              <xsl:apply-templates select="seda:ArchivalProfile"/>
              <xsl:apply-templates select="seda:DescriptionLanguage"/>
              <xsl:apply-templates select="seda:ServiceLevel"/>
            </table>
          </td>
          <td width="50%" valign="top" align="left">
            <table width="100%" cellspacing="0" cellpadding="2">
              <xsl:apply-templates select="seda:Name"/>
              <xsl:apply-templates select="seda:ArchivalAgreement"/>
              <xsl:apply-templates select="seda:ArchivalProfile"/>
              <xsl:apply-templates select="seda:DescriptionLevel"/>
            </table>
          </td>
        </tr>
        <tr>
          <td colspan="2">
            &#160;
          </td>
        </tr>
        <tr>
          <td colspan="2">
            <xsl:apply-templates select="seda:ContentDescription"/>
          </td>
        </tr>
        <tr>
          <td colspan="2">
            &#160;
          </td>
        </tr>
        <tr>
          <td width="50%" valign="top" align="left">
            <xsl:apply-templates select="seda:AppraisalRule"/>
          </td>
          <td width="50%" valign="top" align="left">
            <xsl:apply-templates select="seda:AccessRestrictionRule"/>
          </td>
        </tr>
        <tr>
          <td colspan="2">
            &#160;
          </td>
        </tr>
        <tr>
          <td colspan="2" align="left">
            <xsl:apply-templates select="seda:ContentDescription/seda:OriginatingAgency"/>
          </td>
        </tr>
      </table>
      <br />
      <xsl:apply-templates select="seda:ArchiveObject"/>
    </blockquote>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  ArchiveObject                                                      -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template match="seda:ArchiveObject">
    <xsl:variable name="id" select="@*" />
    <blockquote style="margin-right: 0; margin-left: 25px;">
      <h3 onClick="toggleDiv({$id});" style="cursor: pointer;">
        <xsl:value-of select="seda:Name"/>
      </h3>
      <table width="100%" id="{$id}" class="rm_archiveObject" cellspacing="0" cellpadding="5">
        <tr>
          <td>
            <table width="100%" cellspacing="0" cellpadding="2">
              <xsl:apply-templates select="seda:ArchivalAgencyObjectIdentifier"/>
              <xsl:apply-templates select="seda:DescriptionLevel"/>
              <xsl:apply-templates select="seda:TransferringAgencyObjectIdentifier"/>
              <xsl:apply-templates select="seda:Contains"/>
              <xsl:apply-templates select="seda:Document"/>
            </table>
            <table width="100%" cellspacing="0" cellpadding="2">
            <tr>
              <td colspan="2">
                &#160;
              </td>
            </tr>
            <tr>
              <td colspan="2">
                <xsl:apply-templates select="seda:ContentDescription"/>
              </td>
            </tr>
              <tr>
                <td colspan="2">
                    &#160;
                </td>
              </tr>
              <tr>
                <td width="50%" valign="top" align="left">
                  <xsl:apply-templates select="seda:AppraisalRule"/>
                </td>
                <td width="50%" valign="top" align="left">
                  <xsl:apply-templates select="seda:AccessRestrictionRule"/>
                </td>
              </tr>
              <tr>
                <td colspan="2">
                    &#160;
                </td>
              </tr>
              <tr>
                <td colspan="2" align="left">
                  <xsl:apply-templates select="seda:ContentDescription/seda:OriginatingAgency"/>
                </td>
              </tr>
            </table>
          </td>
        </tr>
      </table>
    </blockquote>
    <br />
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  seda:TransferringAgency                                            -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template 
      match="
        seda:TransferringAgency|
        seda:ArchivalAgency|
        seda:OriginatingAgency|
        seda:Requester
      "
    >
    <h2 style="cursor: pointer;">
      <xsl:call-template name="traduction">
        <xsl:with-param name="term" select="local-name()"/>
      </xsl:call-template>
    </h2>
    <table width="100%" cellspacing="0" cellpadding="5">
      <xsl:apply-templates select="seda:Identification"/>
      <xsl:apply-templates select="seda:Name"/>
      <xsl:apply-templates select="seda:BusinessType"/>
      <xsl:apply-templates select="seda:Description"/>
      <xsl:apply-templates select="seda:District"/>
      <xsl:apply-templates select="seda:LegalClassification"/>
      <xsl:apply-templates select="seda:TaxRegistration"/>
      <xsl:apply-templates select="seda:Address"/>
      <xsl:apply-templates select="seda:Contact"/>
      <xsl:apply-templates select="seda:Communication"/>
    </table>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  adresses                                                           -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template match="seda:Address">
    <tr>
      <td valign="top">
                Adresse
      </td>
      <td valign="top">
        <table width="100%" cellspacing="0" cellpadding="2">
          <xsl:apply-templates select="seda:BlockName"/>
          <xsl:apply-templates select="seda:BuildingName"/>
          <xsl:apply-templates select="seda:BuildingNumber"/>
          <xsl:apply-templates select="seda:StreetName"/>
          <xsl:apply-templates select="seda:Postcode"/>
          <xsl:apply-templates select="seda:CityName"/>
          <xsl:apply-templates select="seda:CitySub-DivisionName"/>
          <xsl:apply-templates select="seda:Country"/>
          <xsl:apply-templates select="seda:FloorIdentification"/>
          <xsl:apply-templates select="seda:PostOfficeBox"/>
          <xsl:apply-templates select="seda:RoomIdentification"/>
        </table>
      </td>
    </tr>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  adresses routines                                                  -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template 
      match="
        seda:BlockName[(. != '') or (@* !='')]|
        seda:BuildingName[(. != '') or (@* !='')]|
        seda:BuildingNumber[(. != '') or (@* !='')]|
        seda:CityName[(. != '') or (@* !='')]|
        seda:CitySub-DivisionName[(. != '') or (@* !='')]|
        seda:Country[(. != '') or (@* !='')]|
        seda:FloorIdentification[(. != '') or (@* !='')]|
        seda:Postcode[(. != '') or (@* !='')]|
        seda:PostOfficeBox[(. != '') or (@* !='')]|
        seda:RoomIdentification[(. != '') or (@* !='')]|
        seda:StreetName[(. != '') or (@* !='')]
      "
    >
    <xsl:value-of select='.' />
    <br />
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  contacts                                                           -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template 
      match="
        seda:TransferringAgency/seda:Contact|
        seda:ArchivalAgency/seda:Contact|
        seda:OriginatingAgency/seda:Contact
      "
    >
    <tr>
      <td valign="top">
                Contact
      </td>
      <td valign="top">
        <table width="100%" cellspacing="0" cellpadding="2">
          <xsl:apply-templates select="seda:PersonName"/>
          <xsl:apply-templates select="seda:Identification"/>
          <xsl:apply-templates select="seda:Responsibility"/>
          <xsl:apply-templates select="seda:DepartmentName"/>
          <xsl:apply-templates select="seda:Address"/>
          <xsl:apply-templates select="seda:Communication"/>
        </table>
      </td>
    </tr>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  communication                                                      -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template match="seda:Communication">
    <div class="{local-name()}">
      <label>
        <xsl:call-template name="traduction">
          <xsl:with-param name="term" select="local-name()"/>
        </xsl:call-template>
      </label>
      <div class="code-value">
        <xsl:apply-templates select="seda:Channel"/>
        <xsl:apply-templates select="seda:CompleteNumber"/>
        <xsl:apply-templates select="seda:URI"/>
      </div>
    </div>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  <tr><td>label</td><td>value</td></tr>                              -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template 
      match="
        seda:Identification[(. != '') or (@* !='')]|
        seda:Name[(. != '') or (@* !='')]|
        seda:BusinessType[(. != '') or (@* !='')]|
        seda:Description[(. != '') or (@* !='')]|
        seda:District[(. != '') or (@* !='')]|
        seda:LegalClassification[(. != '') or (@* !='')]|
        seda:TaxRegistration[(. != '') or (@* !='')]|
        seda:PersonName[(. != '') or (@* !='')]|
        seda:Responsibility[(. != '') or (@* !='')]|
        seda:DepartmentName[(. != '') or (@* !='')]|
        seda:ArchivalAgencyArchiveIdentifier[(. != '') or (@* !='')]|
        seda:ArchivalAgreement[(. != '') or (@* !='')]|
        seda:ArchivalProfile[(. != '') or (@* !='')]|
        seda:TransferringAgencyArchiveIdentifier[(. != '') or (@* !='')]|
        seda:DescriptionLevel[(. != '') or (@* !='')]|
        seda:DescriptionLanguage[(. != '') or (@* !='')]|
        seda:Code[(. != '') or (@* !='')]|
        seda:StartDate[(. != '') or (@* !='')]
      "
    >
    <tr>
      <td width="50%" valign="top">
        <xsl:call-template name="traduction">
          <xsl:with-param name="term" select="local-name()"/>
        </xsl:call-template>
      </td>
      <td width="50%" valign="top">
        <xsl:apply-templates />
      </td>
    </tr>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  Replace P....Y                                                     -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template match="seda:Duration[(. != '') or (@* !='')]">
    <tr>
      <td width="50%" valign="top">
        <xsl:call-template name="traduction">
          <xsl:with-param name="term" select="local-name()"/>
        </xsl:call-template>
      </td>
      <td width="50%" valign="top">
        <xsl:value-of select="substring-after(substring-before(.,'Y'), 'P')"/> an(s)
      </td>
    </tr>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  AppraisalRule|AccesRestrictionRule                                 -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template 
      match="
        seda:AccessRestrictionRule|
        seda:AppraisalRule
      "
    >
    <h3>
      <xsl:call-template name="traduction">
        <xsl:with-param name="term" select="local-name()"/>
      </xsl:call-template>
    </h3>
    <table width="100%" cellspacing="0" cellpadding="2">
      <xsl:apply-templates select="seda:Duration"/>
      <xsl:apply-templates select="seda:Code"/>
      <xsl:apply-templates select="seda:StartDate"/>
    </table>
  </xsl:template>

  <!-- *************************************************************** -->
  <!--                                                                 -->
  <!--  seda:code                                                      -->
  <!--                                                                 -->
  <!-- *************************************************************** -->
  <xsl:template match="seda:AccessRestrictionRule/seda:Code">
    <xsl:variable name="value" select="./text()"/>
    <xsl:variable name="table">seda/v1.0/codes/seda_v1-0_accessrestriction_code.xsd</xsl:variable>
    <xsl:variable name="meta">
      <xsl:value-of select="document($table)//xsd:enumeration[@value=$value]//xsd:annotation/xsd:documentation/ccts:Name"/>
    </xsl:variable>
    <tr>
      <td width="50%" valign="top">
        <xsl:call-template name="traduction">
          <xsl:with-param name="term" select="local-name()"/>
        </xsl:call-template>
      </td>
      <td width="50%" valign="top">
        <xsl:choose>
          <xsl:when test="$meta='0 an'">Immédiat</xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$meta"/>
          </xsl:otherwise>
        </xsl:choose>
                    &#160;
                    (<xsl:value-of select="$value"/>)
      </td>
    </tr>
  </xsl:template>

  <xsl:template match="seda:AppraisalRule/seda:Code">
    <xsl:variable name="value" select="./text()"/>
    <xsl:variable name="table">seda/v1.0/codes/seda_v1-0_appraisal_code.xsd</xsl:variable>
    <xsl:variable name="meta">
      <xsl:value-of select="document($table)//xsd:enumeration[@value=$value]//xsd:annotation/xsd:documentation/ccts:Name"/>
    </xsl:variable>
    <tr>
      <td width="50%" valign="top">
        <xsl:call-template name="traduction">
          <xsl:with-param name="term" select="local-name()"/>
        </xsl:call-template>
      </td>
      <td width="50%" valign="top">
        <xsl:value-of select="$meta"/>
      </td>
    </tr>
  </xsl:template>

  <xsl:template match="seda:ReplyCode">
    <xsl:variable name="value" select="./text()"/>
    <xsl:variable name="table">datatypes.xsd</xsl:variable>
    <xsl:text>[</xsl:text><xsl:value-of select="$value" /><xsl:text>]</xsl:text>
    <xsl:value-of select="document($table)//xsd:simpleType[@name='ReplyCode']//xsd:enumeration[@value=$value]/xsd:annotation/xsd:documentation[@xml:lang='fr']"/>
  </xsl:template>
  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  seda:ContentDescription                                            -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template match="seda:ContentDescription">
    <xsl:apply-templates select="seda:Repository"/>
    <xsl:apply-templates select="seda:CustodialHistory"/>
    <xsl:apply-templates select="seda:Description"/>
    <tr>
        <td>
            <xsl:call-template name="traduction">
                <xsl:with-param name="term" select="local-name(seda:OldestDate)"/>
            </xsl:call-template>
        </td>
        <td>
            <xsl:apply-templates select="seda:OldestDate"/>
        </td>
    </tr>
    <tr>
        <td>
            <xsl:call-template name="traduction">
                <xsl:with-param name="term" select="local-name(seda:LatestDate)"/>
            </xsl:call-template>
        </td>
        <td>
            <xsl:apply-templates select="seda:LatestDate"/>
        </td>
    </tr>
    <xsl:apply-templates select="seda:FilePlanPosition"/>
    <xsl:apply-templates select="seda:OtherDescriptiveData"/>
    <xsl:apply-templates select="seda:RelatedObjectReference"/>
    <xsl:apply-templates select="seda:ContentDescriptive"/>
    <xsl:apply-templates select="seda:AccessRestriction"/>
    <xsl:apply-templates select="seda:Language"/>
    <xsl:apply-templates select="seda:Format"/>
    <xsl:apply-templates select="seda:Size"/>
    <xsl:apply-templates select="seda:OtherMetadata"/>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  DATES                                                              -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template 
      match="
        seda:LatestDate|
        seda:OldestDate|
        seda:StartDate/text()|
        seda:Date/text()|
        seda:Receipt/text()|
        seda:Response/text()|
        seda:Submission/text()|
        seda:Issue/text()|
        seda:Creation/text()
      "
    >
    <xsl:variable name="date">
      <xsl:value-of select="."/>
    </xsl:variable>
    <xsl:variable name="year">
      <xsl:value-of select="substring($date, 1, 4)"/>
    </xsl:variable>
    <xsl:variable name="mm">
      <xsl:value-of select="substring($date, 6, 2)"/>
    </xsl:variable>
    <xsl:variable name="month">
      <xsl:choose>
        <xsl:when test="$mm='01'">Janvier</xsl:when>
        <xsl:when test="$mm='02'">Février</xsl:when>
        <xsl:when test="$mm='03'">Mars</xsl:when>
        <xsl:when test="$mm='04'">Avril</xsl:when>
        <xsl:when test="$mm='05'">Mai</xsl:when>
        <xsl:when test="$mm='06'">Juin</xsl:when>
        <xsl:when test="$mm='07'">Juillet</xsl:when>
        <xsl:when test="$mm='08'">Août</xsl:when>
        <xsl:when test="$mm='09'">Septembre</xsl:when>
        <xsl:when test="$mm='10'">Octobre</xsl:when>
        <xsl:when test="$mm='11'">Novembre</xsl:when>
        <xsl:when test="$mm='12'">Décembre</xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="day">
      <xsl:value-of select="substring($date, 9, 2)"/>
    </xsl:variable>
    <xsl:value-of select="$day"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="$month"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="$year"/>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  seda:Attachment                                                    -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template match="seda:Attachment">
    <!-- nothing -->
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  DIVERS                                                             -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template 
      match="
        seda:Language[(. != '') or (@* !='')]|
        seda:Control[(. != '') or (@* !='')]|
        seda:Copy[(. != '') or (@* !='')]|
        seda:Format[(. != '') or (@* !='')]|
        seda:Size[(. != '') or (@* !='')]|
        seda:CustodialHistory[(. != '') or (@* !='')]/seda:CustodialHistoryItem|
        seda:Description[(. != '') or (@* !='')]|
        seda:FilePlanPosition[(. != '') or (@* !='')]|
        seda:OtherDescriptiveData[(. != '') or (@* !='')]|
        seda:RelatedObjectReference[(. != '') or (@* !='')]|
        seda:Repository[(. != '') or (@* !='')]|
        seda:DeliveryAuthorizationIdentifier[(. != '') or (@* !='')]|
        seda:DeliveryIdentifier[(. != '') or (@* !='')]|
        seda:DeliveryRequestIdentifier[(. != '') or (@* !='')]|
        seda:UnitIdentifier[(. != '') or (@* !='')]|
        seda:ServiceLevel[(. != '') or (@* !='')]|
        seda:PKCS7Signature[(. != '') or (@* !='')]|
        seda:RelatedTransferReference[(. != '') or (@* !='')]|
        seda:TransferReplyIdentifier[(. != '') or (@* !='')]|
        seda:TransferAcceptanceIdentifier[(. != '') or (@* !='')]|
        seda:TransferRequestReplyIdentifier[(. != '') or (@* !='')]|
        seda:TransferringAgencyObjectIdentifier[(. != '') or (@* !='')]|
        seda:ArchivalAgencyObjectIdentifier[(. != '') or (@* !='')]|
        seda:KeywordContent[(. != '') or (@* !='')]|
        seda:KeywordReference[(. != '') or (@* !='')]|
        seda:KeywordType[(. != '') or (@* !='')]|
        seda:Creation[(. != '') or (@* !='')]|
        seda:Issue[(. != '') or (@* !='')]|
        seda:ItemIdentifier[(. != '') or (@* !='')]|
        seda:Purpose[(. != '') or (@* !='')]|
        seda:Receipt[(. != '') or (@* !='')]|
        seda:Response[(. != '') or (@* !='')]|
        seda:Status[(. != '') or (@* !='')]|
        seda:Submission[(. != '') or (@* !='')]|
        seda:Channel[(. != '') or (@* !='')]|
        seda:CompleteNumber[(. != '') or (@* !='')]
      "
    >
    <tr>
      <td width="50%" valign="top">
        <xsl:call-template name="traduction">
          <xsl:with-param name="term" select="local-name()"/>
        </xsl:call-template>
      </td>
      <td width="50%" valign="top">
        <xsl:apply-templates />
      </td>
    </tr>
  </xsl:template>

  <!-- ******************************************************************* -->
  <!--                                                                     -->
  <!--  TRADUCTION                                                         -->
  <!--                                                                     -->
  <!-- ******************************************************************* -->
  <xsl:template name="traduction">
    <xsl:param name="term"/>
    <xsl:choose>
      <xsl:when test="($term = 'ReplyCode')">Code retour</xsl:when>
      <xsl:when test="($term = 'Attachment')">Pièce jointe</xsl:when>
      <xsl:when test="($term = 'NonRepudiation')">Signature</xsl:when>
      <xsl:when test="($term = 'Integrity')">Empreinte</xsl:when>
      <xsl:when test="($term = 'Empreinte/Contains')">Valeur</xsl:when>
      <xsl:when test="($term = 'Channel')">Canal</xsl:when>
      <xsl:when test="($term = 'CompleteNumber')">Numéro</xsl:when>
      <xsl:when test="($term = 'URI')"/>
      <xsl:when test="($term = 'Name')">Nom</xsl:when>
      <xsl:when test="($term = 'ArchiveDelivery')">Communication</xsl:when>
      <xsl:when test="($term = 'ArchiveTransfer')">Transfert d'Archives</xsl:when>
      <xsl:when test="($term = 'Address')">Adresse</xsl:when>
      <xsl:when test="($term = 'Contact')">Contact</xsl:when>
      <xsl:when test="($term = 'Requester')">Demandeur d'archives</xsl:when>
      <xsl:when test="($term = 'TransferringAgency')">Service versant</xsl:when>
      <xsl:when test="($term = 'ArchivalAgency')">Service d'archives</xsl:when>
      <xsl:when test="($term = 'OriginatingAgency')">Service producteur</xsl:when>
      <xsl:when test="($term = 'AccessRestrictionRule')">Délai de communicabilité</xsl:when>
      <xsl:when test="($term = 'AccessRestriction/Code')">Délai de communicabilité</xsl:when>
      <xsl:when test="($term = 'Appraisal/Code')">Sort final</xsl:when>
      <xsl:when test="($term = 'ArchivalAgreement')">Convention</xsl:when>
      <xsl:when test="($term = 'ArchivalProfile')">Profil</xsl:when>
      <xsl:when test="($term = 'Comment')">Commentaire</xsl:when>
      <xsl:when test="($term = 'Creation')">Date de création</xsl:when>
      <xsl:when test="($term = 'CustodialHistory')">Historique</xsl:when>
      <xsl:when test="($term = 'CustodialHistoryItem')">Historique</xsl:when>
      <xsl:when test="($term = 'DescriptionLanguage')">Langue de la description</xsl:when>
      <xsl:when test="($term = 'Issue')">Date d'émission</xsl:when>
      <xsl:when test="($term = 'LatestDate')">Date de fin</xsl:when>
      <xsl:when test="($term = 'Language')">Langue du contenu</xsl:when>
      <xsl:when test="($term = 'Purpose')">Objet</xsl:when>
      <xsl:when test="($term = 'Receipt')">Date de réception</xsl:when>
      <xsl:when test="($term = 'Size')">Taille</xsl:when>
      <xsl:when test="($term = 'StartDate')">Date de départ du calcul</xsl:when>
      <xsl:when test="($term = 'Submission')">Date de soumission</xsl:when>
      <xsl:when test="($term = 'Status')">Etat</xsl:when>
      <xsl:when test="($term = 'Control')">Présence d'exigences de contrôle</xsl:when>
      <xsl:when test="($term = 'Country')">Pays</xsl:when>
      <xsl:when test="($term = 'FilePlanPosition')">Position dans le plan de classement</xsl:when>
      <xsl:when test="($term = 'Identification')">Identifiant</xsl:when>
      <xsl:when test="($term = 'OldestDate')">Date de début</xsl:when>
      <xsl:when test="($term = 'Type')">Type</xsl:when>
      <xsl:when test="($term = 'OtherDescriptiveData')">Autres informations</xsl:when>
      <xsl:when test="($term = 'RelatedTransferReference')">Référence à un autre transfert</xsl:when>
      <xsl:when test="($term = 'ServiceLevel')">Niveau de service demandé</xsl:when>
      <xsl:when test="($term = 'TransferIdentifier')">Numéro</xsl:when>
      <xsl:when test="($term = 'TransferAcceptanceIdentifier')">Identifiant d'acceptation du transfert</xsl:when>
      <xsl:when test="($term = 'TransferReplyIdentifier')">Identifiant de la réponse au transfert</xsl:when>
      <xsl:when test="($term = 'Description')">Description</xsl:when>
      <xsl:when test="($term = 'Format')">Format</xsl:when>
      <xsl:when test="($term = 'Response')">Date de réponse</xsl:when>
      <xsl:when test="($term = 'AppraisalRule')">Règle de conservation</xsl:when>
      <xsl:when test="($term = 'Copy')">Exemplaire</xsl:when>
      <xsl:when test="($term = 'KeywordContent')">Mot-clé</xsl:when>
      <xsl:when test="($term = 'Keywords')">Indexation</xsl:when>
      <xsl:when test="($term = 'ArchivalAgencyArchiveIdentifier')">Cote service d'archive</xsl:when>
      <xsl:when test="($term = 'ArchivalAgencyObjectIdentifier')">Cote article service d'archive</xsl:when>
      <xsl:when test="($term = 'TransferringAgencyArchiveIdentifier')">Identifiant service versant</xsl:when>
      <xsl:when test="($term = 'TransferringAgencyObjectIdentifier')">Identifiant d'origine</xsl:when>
      <xsl:when test="($term = 'BlockName')">Quartier</xsl:when>
      <xsl:when test="($term = 'BuildingName')">Bâtiment</xsl:when>
      <xsl:when test="($term = 'BuildingNumber')">Numéro</xsl:when>
      <xsl:when test="($term = 'CityName')">Localité</xsl:when>
      <xsl:when test="($term = 'CitySub-DivisionName')">Arrondissement / quartier</xsl:when>
      <xsl:when test="($term = 'Code')">Code</xsl:when>
      <xsl:when test="($term = 'BusinessType')">Code de l'activité</xsl:when>
      <xsl:when test="($term = 'DeliveryAuthorizationIdentifier')">Référence de la dérogation</xsl:when>
      <xsl:when test="($term = 'DeliveryIdentifier')">Référence de la communication</xsl:when>
      <xsl:when test="($term = 'DeliveryRequestIdentifier')">Référence de la demande</xsl:when>
      <xsl:when test="($term = 'Contact/DepartmentName')">Service</xsl:when>
      <xsl:when test="($term = 'DescriptionLevel')">Niveau de description</xsl:when>
      <xsl:when test="($term = 'District')">Ressort territorial</xsl:when>
      <xsl:when test="($term = 'FloorIdentification')">Etage</xsl:when>
      <xsl:when test="($term = 'KeywordReference')">Identifiant dans le référentiel associé</xsl:when>
      <xsl:when test="($term = 'KeywordType')">Type</xsl:when>
      <xsl:when test="($term = 'LegalClassification')">Code de la catégorie juridique</xsl:when>
      <xsl:when test="($term = 'PersonName')">Nom</xsl:when>
      <xsl:when test="($term = 'PostOfficeBox')">Boite postale</xsl:when>
      <xsl:when test="($term = 'Postcode')">Code postal</xsl:when>
      <xsl:when test="($term = 'RelatedObjectReference')">Référence complémentaire</xsl:when>
      <xsl:when test="($term = 'Repository')">Lieu de dépôt</xsl:when>
      <xsl:when test="($term = 'Responsibility')">Attributions</xsl:when>
      <xsl:when test="($term = 'RoomIdentification')">Pièce</xsl:when>
      <xsl:when test="($term = 'StreetName')">Voie</xsl:when>
      <xsl:when test="($term = 'TransferRequestReplyIdentifier')">Référence de la réponse à la demande de transfert</xsl:when>
      <xsl:when test="($term = 'ItemIdentifier')">Identifiant unique d'un élément particulier dans le document</xsl:when>
      <xsl:when test="($term = 'UnitIdentifier')">Cote de l'objet communiqué</xsl:when>
      <xsl:when test="($term = 'Empreinte/UnitIdentifier')">Fichier</xsl:when>
      <xsl:when test="($term = 'Date')">Date</xsl:when>
      <xsl:when test="($term = 'Duration')">Durée d'utilité Administrative</xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$term"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

</xsl:stylesheet>