<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:seda="fr:gouv:culture:archivesdefrance:seda:v1.0"
  version="1.0">

  <xsl:output
    method="xml"
    version="1.0"
    encoding="UTF-8"
    indent="yes"/> 

  <!-- ======================================================== -->
  <!-- ====             RM_IO => ArchiveTransfer           ==== -->
  <!-- ======================================================== -->
  <xsl:template match="seda:ArchiveTransfer">
    <xsl:element name="ArchiveTransfer" >
      <!-- DataObject Required attributes -->
      <xsl:attribute name="io_type">
        <xsl:value-of select="local-name()"/>
      </xsl:attribute>
      <xsl:attribute name="io_status">
        <xsl:text>BRSAS</xsl:text>
      </xsl:attribute>
      
      <xsl:apply-templates select="./seda:*" />
      <xsl:comment>dataObjectLog operation="CREATE"</xsl:comment>
    </xsl:element>
  </xsl:template>

  <!-- ======================================================== -->
  <!-- ====             RM_IO => ArchiveTransferReply      ==== -->
  <!-- ======================================================== -->  
  <xsl:template match="seda:ArchiveTransferReply">
    <xsl:element name="ArchiveTransferReply" >
      <!-- DataObject Required attributes -->
      <xsl:attribute name="io_type">
        <xsl:value-of select="local-name()"/>
      </xsl:attribute>
      <xsl:attribute name="io_status">
        <xsl:text>BROK</xsl:text>
      </xsl:attribute>
      
      <xsl:apply-templates select="./seda:*" />
      <xsl:comment>dataObjectLog operation="CREATE"</xsl:comment>
    </xsl:element>
  </xsl:template>
  
  <!-- Comments -->
  <xsl:template match="seda:Comment">
    <xsl:element name="Comment" >
      <xsl:element name="Comment">
        <xsl:value-of select="text()" />
      </xsl:element>
      <xsl:comment>dataObjectLog operation="CREATE"</xsl:comment>
    </xsl:element>   
  </xsl:template>
  
  <!-- Dates -->
  <xsl:template match="seda:Date | seda:Creation | seda:Submission | seda:Response | seda:Receipt | seda:Issue">
    <xsl:element name="{local-name()}" >
      <xsl:choose>
        <xsl:when test="contains(text(), 'T')">
          <xsl:value-of select="substring-before(text(),'T')"/>
            <xsl:text> </xsl:text>
          <xsl:value-of select="substring-after(text(),'T')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="text()"/>
          <xsl:text>T00:00:00</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:element>   
  </xsl:template>
  
  <!-- ======================================================== -->
  <!-- ====    ORGANIZATIONS                                ==== -->
  <!-- ======================================================== -->
  <!-- ArchivalAgency -->
  <xsl:template match="seda:ArchivalAgency | seda:TransferringAgency | seda:OriginatingAgency | seda:ControlAuthority | seda:Repository" >
    <xsl:element name="{local-name()}" >
      <xsl:attribute name="role">
        <xsl:value-of select="local-name()"/>
      </xsl:attribute>
      <xsl:apply-templates select="./seda:*" />
      <xsl:comment>dataObjectLog operation="CREATE"</xsl:comment>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="seda:Communication">
    <!-- Not managed yet -->
  </xsl:template>
  
  <!-- ======================================================== -->
  <!-- ====    RM_ARCHIVES => ArchiveTransfer/Archive      ==== -->
  <!-- ======================================================== -->
  <xsl:template match="seda:Archive | seda:ArchiveObject">
    <xsl:element name="{local-name()}">
      <xsl:attribute name="item_type">
        <xsl:value-of select="local-name()"/>
      </xsl:attribute>
      <xsl:apply-templates select="./*" />
      <xsl:comment>dataObjectLog operation="CREATE"</xsl:comment>
    </xsl:element>
  </xsl:template>
  
  <!-- DataObjects -->
  <xsl:template match="seda:AccessRestrictionRule | seda:AppraisalRule | seda:Address | seda:Contact | seda:ContentDescription">
    <xsl:element name="{local-name()}">
      <xsl:apply-templates select="./*" />
      <xsl:comment>dataObjectLog operation="CREATE"</xsl:comment>
    </xsl:element>
  </xsl:template> 
   
  <xsl:template match="seda:Duration">
    <xsl:element name="{local-name()}" >
      <xsl:value-of select="substring-after(substring-before(text(),'Y'), 'P')"/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="seda:Keyword">
    <xsl:element name="{local-name()}" >
      <xsl:apply-templates select="./*" />
      <xsl:comment>dataObjectLog operation="CREATE"</xsl:comment>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="seda:CustodialHistory">
    <xsl:for-each select="./seda:CustodialHistoryItem">
      <xsl:element name="CustodialHistory" >
        <xsl:element name="CustodialHistoryItem" >
          <xsl:value-of select='text()'/>
        </xsl:element>
        <xsl:comment>dataObjectLog operation="CREATE"</xsl:comment>
      </xsl:element>
    </xsl:for-each>    
  </xsl:template>
  
  <xsl:template match="seda:Document">
    <xsl:element name="Document" >
      <xsl:attribute name="docserver_id">
        <xsl:text>RM_BATCH</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="type_id">
        <xsl:text>100</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="typist">
        <xsl:text>SEDA2MAARCH</xsl:text>
      </xsl:attribute>       
      <xsl:apply-templates select="./seda:*"/>
      <xsl:comment>dataObjectLog operation="CREATE"</xsl:comment>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="seda:Size">
    <xsl:element name="Size">
      <xsl:value-of select="text()" />
    </xsl:element>
    <xsl:element name="unitCode">
      <xsl:value-of select="@unitCode" />
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="seda:Copy">
    <xsl:element name="Copy">
      <xsl:choose>
        <xsl:when test="text() = 'true'">
          <xsl:text>Y</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>N</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="seda:Attachment">
    <xsl:variable name="filename_ext" select="substring-after(./@filename, '.')" />
    <xsl:choose>
      <xsl:when test="$filename_ext != ''">
        <xsl:element name="format" >
          <xsl:value-of select="$filename_ext" />
        </xsl:element>
      </xsl:when>
      <xsl:otherwise>
        <xsl:element name="format" >
          <xsl:text>Physical</xsl:text>
        </xsl:element>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="seda:*">
    <xsl:element name="{local-name()}">
      <xsl:value-of select="text()" />
      <xsl:apply-templates select="./*" />
    </xsl:element>
  </xsl:template>
  
</xsl:stylesheet>